import sys, os
sys.path.insert(0, os.getcwd())
import warnings
if not sys.warnoptions:
    warnings.simplefilter("ignore")
    os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from joblib import Parallel, delayed
from scipy.stats import entropy
from src.lib import config
from src.process.results_process import previous_results
from src.process.data_process import process_drug, process_oncostratifier, process_cosmic, process_BEM, process_genesets
from src.models import oncostratifier_scores
from scipy.stats import false_discovery_control
import collections
from functools import reduce
import argparse
import gseapy as gp
from gseapy import barplot, dotplot


R_STATE=123
file_loader = lambda apath: pd.read_csv(apath, sep='\t', index_col=None)
geneconversions={'MLL': 'KMT2A', 'MLL2': 'KMT2D', 'MLL3': 'KMT2C', 'MLLT4': 'AFDN', 'FAM123B':'AMER1', 'LPHN2':'ADGRL2',
                 'MKL1': 'MRTFA'}#['MLLT4', 'MLL']

def argparser():
    """
    The `argparser` function is a Python function that creates a command-line argument parser for
    visualizing oncostratifier_by_drugs, with default values for the main gene, cancer, main division,
    and variant version.
    :return: The function `argparser` returns the parsed arguments from the command line.
    """
    parser = argparse.ArgumentParser(description='Visualization of oncostratifier_by_drugs')
    parser.add_argument('--main_gene', '-mg', metavar='the-main-gene', dest='main_gene', type=str,
                        help='Choose the main gene', default=None)
    parser.add_argument('--cancer', '-c', metavar='the-cancer', dest='cancer', type=str,
                        help='Choose cancer', default=None)
    parser.add_argument('--main_division', '-md', metavar='the-main-division', dest='main_division', type=str,
                        help='Choose the first level of division: MutDep or Mut', 
                        choices=['MutDep', 'Mut', ''], default='Mut')
    parser.add_argument('--variant', '-v', metavar='the-variant-version', dest='variant_version', type=str,
                        help='Choose variant version', default='cmp_2022')
    args = parser.parse_args()
    print(f'Running args:{args}')
    return args

def get_result(folder_name='oncostratifier_by_drug', cancer=None, gene=None, return_item_names_only=False, parallel=False):
    """
    The `get_result` function retrieves results from a specified folder based on the provided
    parameters, such as cancer type and gene, and returns the results as a pandas DataFrame.
    
    :param folder_name: The `folder_name` parameter is a string that specifies the name of the folder
    where the results are stored. By default, it is set to 'oncostratifier_by_drug', defaults to
    oncostratifier_by_drug (optional)
    :param cancer: The `cancer` parameter is used to specify the type of cancer for which you want to
    get the result. It is an optional parameter and can be set to `None` if you want to get results for
    all cancers
    :param gene: The "gene" parameter is used to specify a specific gene for which you want to retrieve
    results
    :param return_item_names_only: The parameter `return_item_names_only` is a boolean flag that
    determines whether the function should return only the names of the items (cancers and genes) found
    in the folder, or the actual data from the files, defaults to False (optional)
    :param parallel: The `parallel` parameter is a boolean flag that determines whether to load the
    files in parallel or sequentially. If `parallel` is set to `True`, the files will be loaded in
    parallel using multiple processes. If `parallel` is set to `False`, the files will be loaded
    sequentially, defaults to False (optional)
    :return: different values depending on the input parameters:
    """
    onco_drug_folder = config.FHT_RESULTS_DIR / folder_name
    if cancer is not None and gene is not None:
        res_loc = onco_drug_folder / f'entropy_{cancer}_{gene}.csv'
        if res_loc.exists():
            return pd.read_csv(res_loc, sep='\t', index_col=None)
        else:
            return None
    elif cancer is None and gene is None:
        pattern = f"*.csv"
    elif cancer is not None and gene is None:
        pattern = f"*_{cancer}_*.csv"
    elif cancer is None and gene is not None:
        pattern = f"*_{gene}.csv"
    else:
        pattern = f"*_{cancer}_{gene}.csv"
    list_of_paths = list(onco_drug_folder.glob(pattern))
    if len(list_of_paths)==0:
        return None
    elif return_item_names_only: 
        item_name_dicts = {'cancers': [], 'genes':[]}
        for res_path in list_of_paths:
            item_name_dicts['cancers'].append(res_path.stem.split('_', 1)[1].split('_',1)[0])
            item_name_dicts['genes'].append(res_path.stem.split('_', 1)[1].split('_',1)[1])
        item_name_dicts['cancers']=np.unique(item_name_dicts['cancers'])
        item_name_dicts['genes']=np.unique(item_name_dicts['genes'])
        return item_name_dicts
    elif len(list_of_paths)==1:
        return pd.read_csv(list_of_paths[0], sep='\t', index_col=None)
    
    cache_name = f'{folder_name}/combined_{cancer}_{gene}_{len(list_of_paths)}results.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        return cache_dict['data']
    else:
        if parallel:
            results_list = Parallel(n_jobs=-1)(delayed(file_loader)(res_path) for res_path in list_of_paths) #file_loader
        else:
            results_list = []
            for res_path in list_of_paths:
                results_list.append(pd.read_csv(res_path, sep='\t', index_col=None))
        if len(results_list)>0:
            cache_dict = {'data': pd.concat(results_list)}
            config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)
            return cache_dict['data']
        else:
            return None

def categorize_drugs(df, cutoff=0.05, only_significant=False, returned_categories=None): 
    """
    The function `categorize_drugs` categorizes drugs based on certain criteria and returns a DataFrame
    with additional columns for drug information.
    
    :param df: The input dataframe containing drug data
    :param cutoff: The cutoff parameter is used to determine the significance level for categorizing
    drugs. Any drug with a p-value less than the cutoff will be considered significant
    :param only_significant: The parameter "only_significant" is a boolean flag that determines whether
    to include only significant results in the output. If set to True, only drugs with a p-value less
    than the cutoff will be included in the output. If set to False, all drugs will be included
    regardless of significance, defaults to False (optional)
    :param returned_categories: The parameter "returned_categories" is a list of categories that you
    want to filter the dataframe on. Only the rows with categories in this list will be returned in the
    final dataframe. If you don't provide a value for this parameter, all categories will be included in
    the final dataframe
    :return: The function `categorize_drugs` returns a modified DataFrame `df` with additional columns
    for drug categories and various name combinations.
    """
    if df is None:
        return None
    df['category'] = 'NotFound' 
    if only_significant:
        df = df[df['p']<cutoff]
    if len(df) == 0:
        return None
    df.loc[(df['p']<cutoff)&(df['diffEntropy']<0)&(df['mutDep_sensitive_n']>=df['mutDep_resistant_n']), 'category']='co-dependency'
    df.loc[(df['p']<cutoff)&(df['diffEntropy']<0)&(df['mutDep_sensitive_n']<df['mutDep_resistant_n']), 'category']='exclusive-dependency'
    df.loc[(df['p']<cutoff)&(df['diffEntropy']>0)&(df['others_sensitive_n']>=df['others_resistant_n']), 'category']='stratifying non-essentiality'
    df.loc[(df['p']<cutoff)&(df['diffEntropy']>0)&(df['others_sensitive_n']<df['others_resistant_n']), 'category']='stratifying essentiality'
    if returned_categories is not None:
        df = df[df['category'].isin(returned_categories)]
    if len(df) == 0:
        return None
    df['drug_id'] = df['candidateDrug'].str.split(pat='_', n=1, expand=True)[0]
    df['drug_version'] = df['candidateDrug'].str.rsplit(pat='_', n=1, expand=True)[1]
    df['drug_name'] = df['candidateDrug'].str.split(pat='_', n=1, expand=True)[1].str.rsplit(pat='_', n=1, expand=True)[0]
    df['drug_id_name'] = df['drug_id']+'_'+df['drug_version']
    df['drug_name_version'] = df['drug_name']+'_'+df['drug_version']
    
    df['name_for_cancer'] = df['screenGene']+' | '+df['candidateDrug']
    df['name_for_gene'] = df['cancer']+' | '+df['candidateDrug']
    df['name_for_drug'] = df['cancer']+' | '+df['screenGene']
    df['name'] = df['cancer']+' | '+df['screenGene'] + ' | ' + df['candidateDrug']
    df['name_w_drug_name'] = df['cancer']+' | '+df['screenGene'] + ' | ' + df['drug_name']
    df['name_w_drug_id_name'] = df['cancer']+' | '+df['screenGene'] + ' | ' + df['drug_id_name']
    df['name_w_drug_name_version'] = df['cancer']+' | '+df['screenGene'] + ' | ' + df['drug_name_version']
    
    df['name_for_cancer_w_drug_name'] = df['screenGene']+' | '+df['drug_name']
    df['name_for_cancer_w_drug_id_name'] = df['screenGene']+' | '+df['drug_id_name']
    df['name_for_cancer_w_drug_name_version'] = df['screenGene']+' | '+df['drug_name_version']
    
    
    df['name_for_gene_w_drug_name'] = df['cancer']+' | '+df['drug_name']
    df['name_for_gene_w_drug_id_name'] = df['cancer']+' | '+df['drug_id_name']
    df['name_for_gene_w_drug_name_version'] = df['cancer']+' | '+df['drug_name_version']
    
    return df
        

def setCover(setList, target=None, setList_labels=None):
    """
    The function `setCover` finds the smallest subset of sets that covers a given target set.
    
    :param setList: setList is a list of sets. Each set represents a subset of elements
    :param target: The target parameter is the set that we want to cover. It is initially set to the
    union of all the sets in setList
    :param setList_labels: The setList_labels parameter is a list that contains labels or identifiers
    for each set in the setList. It is used to keep track of the labels of the sets that are included in
    the final cover
    :return: The function `setCover` returns two lists: `bestCover` and `bestCover_labels`.
    """
    if not setList: return None, None
    if setList_labels is None: setList_labels  = list(range(len(setList)))
    if target  is None: target  = set.union(*setList)
    bestCover = []
    bestCover_labels = []
    for i,(values, values_labels) in enumerate(zip(setList,setList_labels )):
        remaining = target - values
        if remaining == target: continue
        if not remaining: return [values], [values_labels]
        subCover, subCover_labels = setCover(setList[i+1:],remaining, setList_labels[i+1:])
        if not subCover: continue
        if not bestCover or len(subCover)<len(bestCover)-1:
            bestCover = [values] + subCover
            bestCover_labels = [values_labels] + subCover_labels
    return bestCover, bestCover_labels

def find_mincover(main_division, main_cancer, main_gene, variant_version):
    """
    The function `find_mincover` takes in several parameters, including the main division, main cancer,
    main gene, and variant version, and performs various operations to find the minimum cover for a
    given set of data.
    
    :param main_division: The main_division parameter is a string that represents the main division or
    category of the analysis. It is used to create a folder name for storing the results
    :param main_cancer: The main_cancer parameter is the name of the cancer type you want to analyze
    :param main_gene: The main_gene parameter is the gene of interest that you want to analyze in the
    find_mincover function
    :param variant_version: The `variant_version` parameter is a string that represents the version of
    the variant. It is expected to be in the format "var_source_var_version", where `var_source` is the
    source of the variant (e.g., 'cmp' for comparison) and `var_version` is the version
    """
    var_source, var_version = variant_version.split('_') #'cmp', '2022'  get_cfe('cellline', 'CG', 'Colorectal Carcinoma')
    folder_name = f'oncostratifier_by_drug_{main_division}'
    df = get_result(folder_name=folder_name, cancer=main_cancer, gene=main_gene)
    df = categorize_drugs(df, cutoff=0.05, only_significant=True, returned_categories=['stratifying essentiality', 'stratifying non-essentiality'])    
    df_out_loc = config.FHT_RESULTS_DIR / folder_name / 'cover' / f'stratifying_cover_{main_cancer}_{main_gene}.csv'
    txt_out_loc = config.FHT_RESULTS_DIR / folder_name / 'cover' / f'stratifying_cover_{main_cancer}_{main_gene}.txt'
    df["mutDep_sensitive_list"]= df["mutDep_sensitive_list"].str.split(",", expand = False)
    df["mutDep_resistant_list"]= df["mutDep_resistant_list"].str.split(",", expand = False)
    df = df.sort_values(['cancer','screenGene'])
    all_res = []
    res_lines = []
    for cancer in df['cancer'].unique():
        cancer_df = df[df['cancer']==cancer]
        for gene in cancer_df['screenGene'].unique():
            cancer_gene_df = cancer_df[cancer_df['screenGene']==gene]
            if len(cancer_gene_df)>1:
                sign_markers = analyze_differential_mutation_results(main_division=main_division, cancer=cancer, gene=gene, variant_version=variant_version)
                potential_drugs = sign_markers['candidateDrug'].unique()
                cancer_gene_df['z_order']=1
                cancer_gene_drug_index_df = cancer_gene_df.set_index('candidateDrug')
                cancer_gene_drug_index_df.loc[potential_drugs, 'z_order']=0
                cancer_gene_df = cancer_gene_drug_index_df.sort_values(by='z_order').reset_index()
                all_target_ccles = []
                for with_resp_list in cancer_gene_df['mutDep_with_response_list'].values: all_target_ccles+=with_resp_list.split(',')
                unique_target_ccles = set(all_target_ccles)
                drug_names = cancer_gene_df['candidateDrug'].values.astype('str')
                sensitivity = [set(asens_list) for asens_list in cancer_gene_df['mutDep_sensitive_list'].values]
                cover, cover_drugs = setCover(sensitivity, unique_target_ccles)
                reduced_cover=[]
                if len(cover)==0:
                    for unique_target_ccle in unique_target_ccles:
                        unique_target_ccles_cp = unique_target_ccles.copy()
                        reduced_unique_target_ccles = unique_target_ccles_cp.remove(unique_target_ccle)
                        reduced_cover, reduced_cover_drugs = setCover(sensitivity, reduced_unique_target_ccles)
                        if len(reduced_cover)!=0:
                            break
                if len(cover)!=0 or len(reduced_cover)!=0:
                    covering_drugs = []
                    covering_drugs_str_list = []
                    covering_drugs_str = f'{cancer}\t{gene} ({len(unique_target_ccles)}):\t--{len(cover)}-- | '
                    for coverpart in cover:
                        [covering_drugs.append(drug_names[i]) for i in range(len(sensitivity)) if coverpart==sensitivity[i]]
                        cover_id = [i for i in range(len(sensitivity)) if coverpart==sensitivity[i]][0]
                        covering_drugs_str = f'{covering_drugs_str} {drug_names[cover_id]}({len(sensitivity[cover_id])}) | '
                        covering_drugs_str_list.append(f'{drug_names[cover_id]}({len(sensitivity[cover_id])})')
                    one_res = [cancer, gene, ','.join(list(unique_target_ccles)), len(unique_target_ccles), len(cover), ','.join(covering_drugs_str_list)]
                    print(covering_drugs_str)
                    res_lines.append(covering_drugs_str)
                all_res.append(one_res)
    df = pd.DataFrame(all_res, columns=['Cancer', 'Gene', 'TargetCellLineList', 'TargetCellLineN', 'FoundCoverN', 'FoundCoverList'])
    config.ensure_dir(df_out_loc)
    df.to_csv(df_out_loc, index=False)
    with open(txt_out_loc, 'w') as f:
        f.writelines(res_lines)

def find_cover_with2sets():
    """
    The function `find_cover_with2sets` categorizes drugs based on their essentiality in different
    cancer types and identifies drug pairs that cover all resistant cell lines.
    """
    df = get_result()
    df = categorize_drugs(df, cutoff=0.05, only_significant=True, returned_categories=['stratifying non-essentiality', 'stratifying essentiality'])
    df["mutDep_sensitive_list"]= df["mutDep_sensitive_list"].str.split(",", expand = False)
    df["mutDep_resistant_list"]= df["mutDep_resistant_list"].str.split(",", expand = False)
    df = df.sort_values(['cancer','screenGene'])
    for cancer in df['cancer'].unique():
        cancer_df = df[df['cancer']==cancer]
        for gene in cancer_df['screenGene'].unique():
            cancer_gene_df = cancer_df[cancer_df['screenGene']==gene]
            if len(cancer_gene_df)>1:
                sensitive_dict = cancer_gene_df.set_index('candidateDrug')['mutDep_sensitive_list'].to_dict()
                resistant_dict = cancer_gene_df.set_index('candidateDrug')['mutDep_resistant_list'].to_dict()
                for drug1, drug1_resistant_celllines in resistant_dict.items():
                    for drug2, drug2_sensitive_celllines in sensitive_dict.items():
                        diff_set = np.setdiff1d(drug1_resistant_celllines, drug2_sensitive_celllines)
                        if len(diff_set)==0:
                            msg_text = f'{cancer}\t{gene}:\t'\
                            f'{drug1} ({len(sensitive_dict[drug1])},{len(drug1_resistant_celllines)}) - '\
                            f'{drug2} ({len(drug2_sensitive_celllines)},{len(resistant_dict[drug2])}) = '\
                            f'{diff_set}'
                            print(msg_text)

def find_cancer_types_matching_oncostratifier_cosmic():
    """
    The function `find_cancer_types_matching_oncostratifier_cosmic` finds cancer types that match with
    Oncostratifier and COSMIC databases.
    """
    onco_cancers = get_result(folder_name='oncostratifier_by_drug_mut', cancer=None, gene=None, return_item_names_only=True, parallel=False)['cancers']
    
    df = process_cosmic.get_cancer_gene_census(tiers=[1], roles=['oncogene'], somatic=True, mutation_types=[])
    df['Tumour Types(Somatic)'] = df['Tumour Types(Somatic)'].astype(str).str.lower()
    unique_df_tumour_types = np.unique(df['Tumour Types(Somatic)'].values)
    all_tumour_types = ', '.join(df['Tumour Types(Somatic)'].values).split(', ')
    unique_tumour_types = np.unique(all_tumour_types)
    
    onco2cosmic_cancers = {acancer: [] for acancer in onco_cancers}
    onco2cosmic_cancers["B-Cell Non-Hodgkin's Lymphoma"] = ['b-nhl']
    onco2cosmic_cancers['B-Lymphoblastic Leukemia'] = ['b-all']
    onco2cosmic_cancers["T-Cell Non-Hodgkin's Lymphoma"]= ['t cell lymphomas', 't-cell lymphoma']
    onco2cosmic_cancers['T-Lymphoblastic Leukemia']= ['t-all']
    onco2cosmic_cancers['Kidney Carcinoma']= ['renal', 'renal cell carcinoma']
    res = []
    for acancer in onco_cancers:
        found_list = [onco2cosmic_cancers[acancer]]
        acancer_orig_name = acancer
        #found_list.append(unique_tumour_types[np.where(acancer_orig_name.lower() == unique_tumour_types)[0]])
        found_list.append([a_utt for a_utt in unique_tumour_types if acancer_orig_name.lower() in a_utt])
        acancer_first_letters = ''.join([aword[0] for aword in acancer.split(' ')])
        if len(acancer_first_letters)>2:
            #found_list.append(unique_tumour_types[np.where(acancer_first_letters.lower() == unique_tumour_types)[0]])
            found_list.append([a_utt for a_utt in unique_tumour_types if acancer_first_letters.lower() in a_utt])
        for acancer_each_word in acancer.split(' '):
            if acancer_each_word.lower() not in ['cancer', 'carcinoma', 'sarcoma', 'lymphoma', 'and', 'other', 'squamous', 'cell']:
                found_list.append([a_utt for a_utt in unique_tumour_types if acancer_each_word.lower() in a_utt])
        onco2cosmic_cancers[acancer] = np.unique(np.concatenate(found_list, axis=None))
        
        matched_tumour_types = onco2cosmic_cancers[acancer]
        match_row_ids = []
        target_list = []
        tumour_type_list= []
        tumour_orig_list= []
        for df_row_id, df_row in df.iterrows():
            matching = [amatched_tumour_types in df_row['Tumour Types(Somatic)'] for amatched_tumour_types in matched_tumour_types]
            if np.any(matching):
                target_list.append(df_row['Gene Symbol'])
                match_row_ids.append(df_row_id)
                tumour_type_list.append(matched_tumour_types[matching][0])
                tumour_orig_list.append(df_row['Tumour Types(Somatic)'])
                print(f"{acancer}: {df_row['Gene Symbol']} - {df_row['Tumour Types(Somatic)']}")
        #target_list = df[df['Tumour Types(Somatic)'].isin(matched_tumour_types)]['Gene Symbol'].values
        #tumour_type_list = df[df['Tumour Types(Somatic)'].isin(matched_tumour_types)]['Tumour Types(Somatic)'].values
        res.append([acancer, target_list, matched_tumour_types, tumour_type_list, tumour_orig_list])
    res_df = pd.DataFrame(res, columns=['Cancer', 'Targets', 'Tumour Type', 'Tumour Type List', 'Last'])
    #for row_id, row in res_df.iterrows():
        #print(f'{row["Cancer"]} Oncogenes:')
        #print(f'{row["Targets"]}')
    
    print()

def differential_mutation_per_drug(row, cancer2mut_ccles, gene2mut_ccles, folder_name, file_ending):
    """
    The function performs a differential mutation analysis per drug for a given gene and cancer type.
    
    :param row: The `row` parameter is a pandas Series object that represents a single row of data from
    a DataFrame. It contains information about a specific gene, cancer type, and candidate drug
    :param cancer2mut_ccles: cancer2mut_ccles is a dictionary that maps cancer types to a list of
    mutated genes associated with that cancer type
    :param gene2mut_ccles: The parameter `gene2mut_ccles` is a dictionary that maps genes to mutated
    cell lines. It contains information about which cell lines have mutations in each gene
    :param folder_name: The `folder_name` parameter is a string that represents the name of the folder
    where the output files will be saved
    :param file_ending: The `file_ending` parameter is a string that specifies the file extension for
    the output file. It is used to create the file name for the output file for each drug
    """
    gene = row['screenGene']
    cancer_type = row['cancer']
    folder = config.FHT_RESULTS_DIR / folder_name / 'analysis' / f'differential_mutation' / f'{cancer_type}_{gene}'
    drug_name = row['candidateDrug']
    out_loc_per_drug = folder / f'{drug_name}{file_ending}'
    if not out_loc_per_drug.exists():
        config.ensure_dir(out_loc_per_drug)
        all_ccles_for_cancer = cancer2mut_ccles[cancer_type]
        partitioned_cls = []
        for group_name in ['mutDep_sensitive_list', 'mutDep_resistant_list', 'others_sensitive_list', 'others_resistant_list']:
            if pd.isnull(row[group_name]):
                partitioned_cls.append([])
            else:
                partitioned_cls.append(row[group_name].split(','))
        res_list = []
        for gene_to_try, gene_mutated in gene2mut_ccles.items():
            gene_not_mutated = np.setdiff1d(all_ccles_for_cancer, gene_mutated)
            mutation_per_gene = (np.unique(gene_mutated), gene_not_mutated)
            one_res_df = stratifier_scores.find_mutational_significance(mutation_per_gene, tuple(partitioned_cls))
            one_res_df[['screenGene', 'cancer', 'candidateDrug', 'label']] = row[['screenGene', 'cancer', 'candidateDrug', 'label']]
            one_res_df['differentialGene'] = gene_to_try
            res_list.append(one_res_df)
            del one_res_df
        if len(res_list)==1:
            res_list[0].to_csv(out_loc_per_drug)
        elif len(res_list)>1:
            pd.concat(res_list).to_csv(out_loc_per_drug)
        del res_list
        del row

def find_differential_mutations_per_cancer_gene(main_division, cancer, gene, variant_version):
    var_source, var_version = variant_version.split('_') #'cmp', '2022'  get_cfe('cellline', 'CG', 'Colorectal Carcinoma')
    folder_name = f'oncostratifier_by_drug_{main_division}'
    res_onco = get_result(folder_name=folder_name, cancer=cancer, gene=gene)
    res_onco = categorize_drugs(res_onco, cutoff=0.05, only_significant=True, returned_categories=['stratifying essentiality', 'stratifying non-essentiality'])
    
    if cancer not in process_BEM.ccle2tcga.keys():
        return 1
    
    #cache_name = f'oncostratifier_processed/gene_and_cancer2mut_cmp_2022.xz'
    #cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    #cfe_mutation = process_BEM.get_cfe('cellline', 'CG', cancer)
    #cfe_genes = cfe_mutation[cfe_mutation['cancer_type_specific']]['CG'].values
    if 'cmp' in variant_version:
        file_ending = f'_{variant_version}_nosilent_cfegenes.xz'
        cache_name = f'oncostratifier_processed/gene_{gene}_and_cancer_{cancer}2mut_{variant_version}_nosilent_cfegenes.xz'
    else:
        file_ending = f'_{variant_version}.xz'
        cache_name = f'oncostratifier_processed/gene_{gene}_and_cancer_{cancer}2mut_{variant_version}.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is None:
        annotations = process_oncostratifier.get_annotation()
        cancer2cellline= annotations.reset_index().groupby('cancer_type')['model_name'].apply(list).to_dict()
        modelname2modelid = annotations.reset_index().set_index('model_name')['model_id'].to_dict()
        modelid2modelname = annotations.reset_index().set_index('model_id')['model_name'].to_dict()
        cosmicid2modelname = annotations.reset_index().set_index('COSMIC_ID')['model_name'].to_dict()
        
        if 'cmp' in variant_version:
            mutation = process_oncostratifier.get_mutation(source=variant_version.split('_')[0], version=variant_version.split('_')[1])
            cfe_mutation = process_BEM.get_cfe('cellline', 'CG', cancer)
            cfe_genes = cfe_mutation[cfe_mutation['cancer_type_specific']]['CG'].values
            mutation = mutation[mutation['effect']!='silent']
            mutation['model_name'] = mutation['model_id'].map(modelid2modelname)
            mutation = mutation[~mutation['model_name'].isnull()]
            unique_celllines_in_mutation = mutation['model_name'].unique()
            mutation['mutated'] = (mutation['rna_mutation'].str.len() != 0).astype(int)
            mutation = mutation.set_index('gene_symbol').loc[[geneconversions.get(agene, agene)for agene in cfe_genes]].reset_index()
            cache_dict = {'gene2mut_ccles': mutation[mutation['mutated']==1].groupby('gene_symbol')['model_name'].apply(list).to_dict(),
                        'cancer2mut_ccles': {cancer: np.intersect1d(cancer_ccles, unique_celllines_in_mutation) for cancer, cancer_ccles in cancer2cellline.items()}}
        elif 'cfe' == variant_version.split('_')[0]:
            variant_type = variant_version.split('_')[1]
            mutation = process_BEM.get_cfe('cellline', variant_type, cancer).rename(columns=cosmicid2modelname)
            unique_celllines_in_mutation = mutation.columns.values[2:]
            mutation = mutation[mutation['cancer_type_specific']].drop(columns=['cancer_type_specific']).melt(id_vars=variant_type).rename(columns={'value': 'mutated', 'variable': 'model_name', variant_type: 'gene_symbol'})#['CG'].values
            cache_dict = {'gene2mut_ccles': mutation[mutation['mutated']==1].groupby('gene_symbol')['model_name'].apply(list).to_dict(),
                        'cancer2mut_ccles': {cancer: np.array(cancer_ccles) for cancer, cancer_ccles in cancer2cellline.items()}}
            
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)
        
    gene2mut_ccles, cancer2mut_ccles = cache_dict['gene2mut_ccles'], cache_dict['cancer2mut_ccles']
    
    if len(res_onco)>0:
        results = Parallel(n_jobs=-1)(delayed(differential_mutation_per_drug)(row.copy(), cancer2mut_ccles, gene2mut_ccles, folder_name, file_ending) for row_id, row in res_onco.iterrows())

def analyze_differential_mutation_results(main_division, cancer, gene, variant_version):
    """
    The function `analyze_differential_mutation_results` analyzes the results of a differential mutation
    experiment based on the main division, cancer type, gene, and variant version.
    
    :param main_division: The main_division parameter is a string that represents the main division of
    the experiment folder
    :param cancer: The "cancer" parameter represents the type of cancer being analyzed. It is used to
    filter the results based on the specified cancer type
    :param gene: The "gene" parameter is used to specify a specific gene for analysis. It is an optional
    parameter, so it can be set to None if you want to analyze all genes
    :param variant_version: The `variant_version` parameter is a string that specifies the version of
    the variant being analyzed. It is in the format "var_source_var_version", where `var_source` is the
    source of the variant (e.g., "cmp" for comparison) and `var_version` is the version number
    :return: either a pandas DataFrame containing the results of the analysis or None if there are no
    results.
    """
    if cancer not in process_BEM.ccle2tcga.keys():
        return None
    import seaborn as sns
    import matplotlib.pyplot as plt
    experiment_folder = f'oncostratifier_by_drug_{main_division}'
    var_source, var_version = variant_version.split('_')
    if 'cmp' in variant_version:
        file_ending = f'_{variant_version}_nosilent_cfegenes.xz'
    else:
        file_ending = f'_{variant_version}.xz'
    main_folder = config.FHT_RESULTS_DIR / experiment_folder / 'analysis' / f'differential_mutation'
    
    if cancer is not None and gene is not None:
        search_folder = main_folder / f'{cancer}_{gene}'
        pattern= f'*{file_ending}' 
    elif cancer is None and gene is None:
        search_folder = main_folder 
        pattern= f'*_*/*{file_ending}' 
    elif cancer is not None and gene is None:
        search_folder = main_folder 
        pattern= f'{cancer}_*/*{file_ending}' 
    elif cancer is None and gene is not None:
        search_folder = main_folder 
        pattern= f'*_{gene}/*{file_ending}' 
    list_of_paths = list(search_folder.glob(pattern))
    res_list = []
    for apath_idx, apath in enumerate(list_of_paths):
        ares = pd.read_csv(apath, index_col=None)
        #mutation = process_BEM.get_cfe('cellline', 'CG', ares['cancer'].unique()[0])
        #cfe_genes = mutation[mutation['cancer_type_specific']]['CG'].values
        #ares_cfegenes = ares.set_index('differentialGene').loc[[geneconversions.get(agene, agene)for agene in cfe_genes]]
        ares_cfegenes = ares.set_index('differentialGene')
        ares_filtered = ares_cfegenes[(ares_cfegenes['mutDep_not_mutated_n']!=0)&(ares_cfegenes['mutDep_mutated_n']!=0)]
        ares_filtered['mutdep_fisher_p_corrected'] = false_discovery_control(ares_filtered['mutdep_fisher_p'].values ,axis=0, method='bh')
        ares_filtered['others_fisher_p_corrected'] = false_discovery_control(ares_filtered['others_fisher_p'].values ,axis=0, method='bh')
        ares_mutdep_sign = ares_filtered[ares_filtered['mutdep_fisher_p_corrected']<0.05]
        sign_genes = len(ares_mutdep_sign)
        if sign_genes>0:
            print(f'{str(apath).split("/")[-2:]}: {sign_genes}\t {ares_mutdep_sign.index.values}')
        res_list.append(ares_mutdep_sign)
    if len(res_list)>0:
        return pd.concat(res_list)
    else:
        return None
    
    
def analysis_degenes(main_division):
    drug2gene, drug2pw = process_drug.get_compound_annotation(dict_key = 'DRUGID_DRUGNAME')
    geneset2gene, gene2geneset = process_genesets.get_all_pathway_dicts()
    sign_markers_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_{main_division}' / 'analysis' / f'differential_mutation' / 'only_significant.csv'
    sign_markers = pd.read_csv(sign_markers_loc, index_col=None)
    enrichment_folder = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_mut' / 'images' / 'enrichment_analysis'
    genesets_lookup = ['MSigDB_Hallmark_2020','KEGG_2021_Human', 'WikiPathway_2023_Human', 'Reactome_2022', 'GO_Biological_Process_2023', 'GO_Cellular_Component_2023', 'GO_Molecular_Function_2023']
    plots_dict = {'Hallmarks': ['MSigDB_Hallmark_2020'], 
                'Pathways': ['KEGG_2021_Human', 'WikiPathway_2023_Human', 'Reactome_2022'],
                'Gene Ontology': ['GO_Biological_Process_2023', 'GO_Cellular_Component_2023', 'GO_Molecular_Function_2023']}
    
    for entry_idx, entry in sign_markers.iterrows():
        cancer, oncogene, drug, marker = entry['cancer'], entry['screenGene'], entry['candidateDrug'], entry['differentialGene']
        plot_folder_loc = enrichment_folder / f'{cancer}_{oncogene}_{drug}_{marker}'
        config.ensure_dir(plot_folder_loc / 'a.png')
        enrichment_res_loc = plot_folder_loc / f'enrichrres_{cancer}_{oncogene}_{drug}_{marker}.csv'
        if enrichment_res_loc.exists():
            print('xd')
        drug_idname = drug.rsplit('_', maxsplit=1)[0]
        drug_gene_targets = drug2gene.get(drug_idname, '')
        drug_pw_targets = drug2pw.get(drug_idname,'').split(', ')
        '''
        drug_pw_foundtargets = {}
        for a_drug_pw_target in drug_pw_targets:
            tmp_a_drug_pw_target_dict = {a_geneset: a_geneset_genes for a_geneset, a_geneset_genes in geneset2gene.items() if a_drug_pw_target.lower() in a_geneset.lower()}
            drug_pw_foundtargets = {**drug_pw_foundtargets, **tmp_a_drug_pw_target_dict} 
        '''
        res_per_dge_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_{main_division}' / 'images' / 'mut_marker_dge_clustermap' / 'data' / f'{cancer}_{oncogene}_{marker}_deseq2_noxticks.pkl'
        dict_per_dge_res = config.cache_numpy(res_per_dge_loc, is_pickle=True, is_full_loc=True)
        if dict_per_dge_res is None:
            continue
        significant_dge_df = dict_per_dge_res['dge_df'] 
        exp_scaled_counts_df = dict_per_dge_res['expression_zscored'] 
        metadata_df = dict_per_dge_res['metadata']
        dge_genes_tested = dict_per_dge_res['genes_tested']
        significant_dge_df['cancer'] = cancer
        significant_dge_df['oncogene'] = oncogene
        significant_dge_df['drug'] = drug
        significant_dge_df['drug_gene_target'] = drug_gene_targets
        significant_dge_df['drug_pw_target'] = ','.join(drug_pw_targets)
        significant_dge_df['marker'] = marker
        significant_dge_df = significant_dge_df.reset_index(names='dge_gene')
        significant_dge_df['drug_gene_target_HIT'] = [a_dge_gene in drug_gene_targets for a_dge_gene in significant_dge_df['dge_gene']]
        print()
        print(f'{cancer} - {oncogene} - {drug}({drug_gene_targets}),({drug_pw_targets}) - {marker}')
        #for agene in significant_dge_df['dge_gene'].values: print(agene)
        de_gene_list = list(significant_dge_df['dge_gene'].values)
        background_gene_list = list(dge_genes_tested)
        #Libraries: https://maayanlab.cloud/Enrichr/#libraries
        
        with open(plot_folder_loc / f'backg_{cancer}_{oncogene}_{drug}_{marker}.txt', 'w') as f:
            f.writelines([f'{agene}\n' for agene in background_gene_list])
        with open(plot_folder_loc / f'genelist_{cancer}_{oncogene}_{drug}_{marker}.txt', 'w') as f:
            f.writelines([f'{agene}\n' for agene in de_gene_list])
        if False:#enrichment_res_loc.exists():
            enrichment_res = pd.read_csv(enrichment_res_loc, index_col=0)
        elif (plot_folder_loc / f'{genesets_lookup[0]}_table.txt').exists():
            enrichment_res = pd.concat([pd.read_csv(plot_folder_loc / f'{geneset_title}_table.txt', index_col=None, sep='\t').assign(Gene_set=geneset_title) for geneset_title in genesets_lookup])
            enrichment_res = enrichment_res.loc[:, ['Gene_set'] + enrichment_res.columns.tolist()[:-1]]
            enrichment_res.to_csv(enrichment_res_loc)
        else:
            try:
                enr_bg = gp.enrichr(gene_list=de_gene_list,
                        gene_sets=genesets_lookup,
                        # organism='human', # organism argment is ignored because user input a background
                        background=background_gene_list,#list(exp_scaled_counts_df.index.values),
                        outdir=None, # don't write to disk
                        )
                enrichment_res = enr_bg.results
                enrichment_res.to_csv(enrichment_res_loc)
            except Exception as e1:
                print(f'{cancer} - {oncogene} - {drug}({drug_gene_targets}),({drug_pw_targets}) - {marker} ERROR: {e1}')
                continue
        for plot_title, plot_gene_sets in plots_dict.items():
            enrichment_res_per_geneset = enrichment_res[enrichment_res['Gene_set'].isin(plot_gene_sets)]
            if sum(enrichment_res_per_geneset['Adjusted P-value'].values<0.05)>0:
                plt.clf()
                dotplot(enrichment_res_per_geneset,
                    column="Adjusted P-value",
                    x='Gene_set', # set x axis, so you could do a multi-sample/library comparsion
                    size=4,
                    top_term=8,
                    figsize=(4,6),
                    title = plot_title,
                    xticklabels_rot=45, # rotate xtick labels
                    show_ring=True, # set to False to revmove outer ring
                    marker='o',
                    ofname=plot_folder_loc / f'{cancer}_{oncogene}_{drug}_{marker}_{plot_title.strip()}_dotplot.pdf'
                    )
                plt.clf()
                barplot(enrichment_res_per_geneset,
                    column="Adjusted P-value",
                    group='Gene_set', # set group, so you could do a multi-sample/library comparsion
                    size=4,
                    top_term=5,
                    figsize=(4,6),
                    #color=['darkred', 'darkblue'] # set colors for group
                    #color = {'KEGG_2021_Human': 'salmon', 'MSigDB_Hallmark_2020':'darkblue'},
                    ofname=plot_folder_loc / f'{cancer}_{oncogene}_{drug}_{marker}_{plot_title.strip()}_barplot.pdf'
                    )



        

def analysis_degenes_gene_enrichment(main_division):
    import gseapy as gp
    from gseapy import Msigdb
    msig = Msigdb()
    msigdb_human_setlist = msig.list_category(dbver="2023.2.Hs") # Human
    drug2gene, drug2pw = process_drug.get_compound_annotation(dict_key = 'DRUGID_DRUGNAME')
    geneset2gene, gene2geneset = process_genesets.get_all_pathway_dicts()
    sign_markers_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_{main_division}' / 'analysis' / f'differential_mutation' / 'only_significant.csv'
    sign_markers = pd.read_csv(sign_markers_loc, index_col=None)
    
    for entry_idx, entry in sign_markers.iterrows():
        cancer, oncogene, drug, marker = entry['cancer'], entry['screenGene'], entry['candidateDrug'], entry['differentialGene']
        drug_idname = drug.rsplit('_', maxsplit=1)[0]
        drug_gene_targets = drug2gene.get(drug_idname, '')
        drug_pw_targets = drug2pw.get(drug_idname,'').split(', ')
        '''
        drug_pw_foundtargets = {}
        for a_drug_pw_target in drug_pw_targets:
            tmp_a_drug_pw_target_dict = {a_geneset: a_geneset_genes for a_geneset, a_geneset_genes in geneset2gene.items() if a_drug_pw_target.lower() in a_geneset.lower()}
            drug_pw_foundtargets = {**drug_pw_foundtargets, **tmp_a_drug_pw_target_dict} 
        '''
        res_per_dge_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_{main_division}' / 'images' / 'mut_marker_dge_clustermap' / 'data' / f'{cancer}_{oncogene}_{marker}_deseq2_noxticks.pkl'
        dict_per_dge_res = config.cache_numpy(res_per_dge_loc, is_pickle=True, is_full_loc=True)
        if dict_per_dge_res is None:
            continue
        significant_dge_df = dict_per_dge_res['dge_df'] 
        exp_scaled_counts_df = dict_per_dge_res['expression_zscored'] 
        metadata_df = dict_per_dge_res['metadata']
        significant_dge_df['cancer'] = cancer
        significant_dge_df['oncogene'] = oncogene
        significant_dge_df['drug'] = drug
        significant_dge_df['drug_gene_target'] = drug_gene_targets
        significant_dge_df['drug_pw_target'] = ','.join(drug_pw_targets)
        significant_dge_df['marker'] = marker
        significant_dge_df = significant_dge_df.reset_index(names='dge_gene')
        significant_dge_df['drug_gene_target_HIT'] = [a_dge_gene in drug_gene_targets for a_dge_gene in significant_dge_df['dge_gene']]
        if sum(significant_dge_df['drug_gene_target_HIT'].values)>0:
            print()
        significant_dge_df['drug_pw_target_HIT'] = ''
        for a_dge_row_idx, a_dge_row in significant_dge_df.iterrows():
            if a_dge_row['dge_gene'] not in gene2geneset.keys():
                continue
            genesets_with_markergene = gene2geneset[a_dge_row['dge_gene']]
            genesets_with_markergene_matches_drugtarget = []
            for a_geneset in genesets_with_markergene:
                if any([a_drug_pw_target.lower() in a_geneset.lower() for a_drug_pw_target in drug_pw_targets]):
                    genesets_with_markergene_matches_drugtarget.append(a_geneset)
        if len(genesets_with_markergene_matches_drugtarget)>0:
            significant_dge_df['drug_pw_target_HIT'] = ','.join(genesets_with_markergene_matches_drugtarget)
        print()

def main():
    #find_cancer_types_matching_oncostratifier_cosmic()
    args = argparser()
    analysis_degenes(args.main_division.lower())
    '''
    
    
    if args.cancer is None:
        folder_name = f'oncostratifier_by_drug_{args.main_division.lower()}'
        res_onco = get_result(folder_name=folder_name, cancer=args.cancer, gene=args.main_gene, return_item_names_only=True)
        multiple_res = []
        for cancer in res_onco['cancers']:
            #find_differential_mutations_per_cancer_gene(main_division=args.main_division.lower(), cancer=cancer, gene=args.main_gene, variant_version=args.variant_version)
            multiple_res.append(analyze_differential_mutation_results(main_division=args.main_division.lower(), cancer=cancer, gene=args.main_gene, variant_version=args.variant_version))#args.cancer
        multiple_res_filtered = [multiple_res_item for multiple_res_item in multiple_res if multiple_res_item is not None and type(multiple_res_item).__name__=='DataFrame' and len(multiple_res_item)>0]
        out_loc_full = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_{args.main_division.lower()}' / 'analysis' / f'differential_mutation' / 'only_significant.csv'
        
        pd.concat(multiple_res).to_csv(out_loc_full)
    elif args.cancer is not None:
        #find_differential_mutations_per_cancer_gene(main_division=args.main_division.lower(), cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
        #analyze_differential_mutation_results(main_division=args.main_division.lower(), cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
        find_mincover(main_division=args.main_division.lower(), main_cancer=args.cancer, main_gene=args.main_gene, variant_version=args.variant_version)
    #analyze_differential_mutation_results(main_division=args.main_division.lower(), cancer='Colorectal Carcinoma', gene=args.main_gene, variant_version=args.variant_version)#args.cancer
    print()
    '''
if __name__ == "__main__":
    main()