#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import sys, os
sys.path.insert(0, os.getcwd())
import warnings
if not sys.warnoptions:
    warnings.simplefilter("ignore")
    os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from joblib import Parallel, delayed
from src.lib.sutils import *
from src.lib import config
from src.process.results_process import previous_results
from src.process.data_process import process_drug
from src.models.oncostratifier_scores import find_entropy
from src.visualization.results_visualization import OncoStratifierVisualizer as onco_visualizer
from src.analysis.results_analysis import oncostratifier_by_drug as onco_analyzer

import argparse

parser = argparse.ArgumentParser(description='Visualization of oncostratifier_by_drugs')
parser.add_argument('--main_gene', '-mg', metavar='the-main-gene', dest='main_gene', type=str,
                    help='Choose the main gene', default='KRAS')
parser.add_argument('--cancer', '-c', metavar='the-cancer', dest='cancer', type=str,
                    help='Choose cancer', default="Colorectal Carcinoma")
parser.add_argument('--main_division', '-md', metavar='the-main-division', dest='main_division', type=str,
                    help='Choose the first level of division: MutDep or Mut', 
                    choices=['MutDep', 'Mut', ''], default='Mut')
parser.add_argument('--stratifier', '-st', metavar='the-stratifier', dest='stratifier', type=str,
                    help='Choose the possible stratifier sources: Dependency or Drug', 
                    choices=['Dependency', 'Drug'], default='Drug')
parser.add_argument('--stratifier_version', '-sv', metavar='the-stratifier-version', dest='stratifier_version', type=str,
                    help='Choose the possible stratifier source version: All, 1, 2 if stratifier is Drug; All if stratifier is Dependency', 
                    choices=['All', '1', '2'], default='All')
parser.add_argument('--minimum_celllines', '-mc', metavar='the-minimum-celllines', dest='min_celllines', type=int,
                    help='Eliminate cases where there are less than min_celllines in any group', default=3)
parser.add_argument('--visual_type', '-vt', metavar='the-visual-type', dest='visual_type', type=str,
                    help='Choose the visualization', 
                    choices=['circular_bar', 'violin', 'violin_facet', 'target_match_distribution','significant_drug_distribution', 'differential_mutation',
                             'cellline_scatter_with_drug_sensitivity', 'drug_cellline_heatmap', 'response_with_mutational_signature'], 
                    default='drug_cellline_heatmap')
parser.add_argument('--version', '-ve', metavar='the-version', dest='visual_version', type=int,
                    help='Choose the version of the visualization', default=2)
parser.add_argument('--seed', '-se', metavar='the-seed', dest='seed', type=int,
                    help='Choose the seed of the visualization', default=123)
parser.add_argument('--variant', '-va', metavar='the-variant-version', dest='variant_version', type=str,
                    help='Choose variant version', default='cmp_2022')
parser.add_argument('--parallel_processing', '-pp', metavar='bool', dest='parallel_process', type=str2bool, default=False,
                    help='Whether to activate parallel processing')


class OncoStratifier(object):
    def __init__(self, args):
        """
        Parameters
        ----------
        args:
            arguments
        """
        self.args = args
        self.main_gene = args.main_gene
        self.cancer = args.cancer
        self.main_genes = None
        self.cancers = None
        self.main_division = args.main_division
        self.stratifier = args.stratifier
        self.stratifier_version = args.stratifier_version
        self.variant_version = args.variant_version
        self.variant_source = self.variant_version.split('_')[0]
        self.variant_year = self.variant_version.split('_')[1]
        self.min_celllines = args.min_celllines
        self.visual_type = args.visual_type
        self.visual_version = args.visual_version
        self.r_state=args.seed
        self.parallel_process=args.parallel_process
        
        #Define folders
        self.main_exp_folder_name = f'Oncostratifier_{self.main_division}Division_{self.stratifier}Stratifier'
        self.detailed_exp_folder_name = f'Variant={self.variant}_Stratifier={self.stratifier_version}'
        self.cancer_gene_exp_folder_name = f'Cancer={self.cancer}_Gene={self.main_gene}_minCell={self.min_celllines}'
        self.results_dir = config.RESULTS_DIR / self.main_exp_folder_name / self.detailed_exp_folder_name / self.cancer_gene_exp_folder_name
        self.cache_folder = self.main_exp_folder_name / self.detailed_exp_folder_name / self.cancer_gene_exp_folder_name

        # this will create folders with all roots
        safe_create_dir(self.result_dir)
        safe_create_dir(self.cache_folder)
        # change log and create log file
        change_log_path(self.result_dir / 'run.log')
        log('exp_data_dir:', self.data_dir)
        
        
    @timeit
    def get_main_division(self):
        screens = previous_results.get_detailed_gene_screens(variant=self.variant_year, cancer=self.cancer, main_gene=self.main_gene, fixed=False)
        if self.cancer is not None:
            screens = screens[screens['cancer_type']==self.cancer]
        if self.main_gene is not None:
            screens = screens[screens['screen_gene']==self.main_gene]
        
        self.cancers = screens['cancer_type'].unique()
        self.main_genes = screens['screen_gene'].unique()
        
        return screens
          
    @timeit      
    def get_stratifier_data(self):
        if self.stratifier.lower() == 'drug':
            gdsc_cache_name = f'gdsc_v={self.stratifier_version}_27Oct23.xz'
            gdsc_cache_dict = config.cache_numpy(cache_name=gdsc_cache_name, is_pickle=True)
            if gdsc_cache_dict is None:
                print(f'GDSC load started!')
                gdsc_list = []
                if self.stratifier_version.lower() = 'all' or self.stratifier_version=='1':
                    gdsc_list.append(process_drug.get_drug(gdsc_version=1))
                if self.stratifier_version.lower() = 'all' or self.stratifier_version=='2':
                    gdsc_list.append(process_drug.get_drug(gdsc_version=2))
                gdsc_cache_dict = {'gdsc': process_drug.clear_combine_sources(gdsc_list)}
                config.cache_numpy(cache_name=gdsc_cache_name, data=gdsc_cache_dict, is_pickle=True)
            gdsc = gdsc_cache_dict['gdsc']
            
        return gdsc
    
    @timeit
    def get_stratifier2ccles_data(self):
        if self.stratifier.lower() == 'drug':
            gdsc_drug2response_cache_name = f'gdsc_{self.stratifier_version}_drug2response_27Oct23.xz'
            gdsc_drug2response = config.cache_numpy(cache_name=gdsc_drug2response_cache_name, is_pickle=True)
            if gdsc_drug2response is None:
                print(f'GDSC gene2response load started!')
                gdsc = self.get_stratifier_data()
                
                gdsc_drug2response = {}
                for drug in gdsc['UNIQUE_DRUG'].unique():
                    drug_df = gdsc[gdsc['UNIQUE_DRUG']==drug]
                    gdsc_drug2response[drug] = {'sensitive':drug_df[drug_df['LN_SENSITIVE']==1]['CELL_LINE_NAME'].values, 'resistant':drug_df[drug_df['LN_SENSITIVE']==0]['CELL_LINE_NAME'].values}
                config.cache_numpy(cache_name=gdsc_drug2response_cache_name, data=gdsc_drug2response, is_pickle=True)
            
        return gdsc_drug2response
        
    
    @timeit
    def get_variant_data(self):
        pass
    
    @timeit
    def get_variant_data(self):
        pass
    
    
    @timeit()
    def process_result_file(self, res):
        gdsc_targets, gdsc_nested_targets = process_drug.get_gdsc_targets(self.get_stratifier_data())
        res = onco_analyzer.categorize_drugs(res, cutoff=0.05, only_significant=True)
        if res is not None:
            res['category'] = res['category'].replace(category_name_fix)
            res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
            res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
            label_add = ''
            if target_included:
                label_add = '_target'
                list_of_labels = ['name_for_cancer', 'name_for_cancer_w_drug_name_version', 'name_for_cancer_w_drug_name', 'name_for_cancer_w_drug_id_name', 
                                'name_for_gene', 'name_for_gene_w_drug_name_version', 'name_for_gene_w_drug_name', 'name_for_gene_w_drug_id_name',
                                'candidateDrug', 'drug_name_version', 'drug_name', 'drug_id_name']
                for name_col in list_of_labels:
                    res[f'{name_col}{label_add}'] = res[name_col] + ' (' + res['targets'] + ')'
        return res


    @timeit
    def call_entropy_per_stratifier(self, a_drug, a_drug_responses, cancer, gene, mutdep, others):
        """
        The function `call_entropy_per_drug` performs an analysis for a specific drug, cancer type, and
        gene, and returns the results in a dataframe.
        
        :param a_drug: The variable `a_drug` represents the drug for which you want to perform the analysis
        :param a_drug_responses: The parameter "a_drug_responses" is a dictionary that contains information
        about the drug responses. It has two keys: "sensitive" and "resistant". The value corresponding to
        each key is a list of cell lines that are either sensitive or resistant to the drug, respectively
        :param cancer: The `cancer` parameter represents the type of cancer being analyzed. It is a string
        that specifies the cancer type, such as "breast cancer" or "lung cancer"
        :param gene: The "gene" parameter represents a specific gene that is being analyzed in the context
        of cancer and drug response. It is used to identify the gene of interest for the analysis
        :param mutdep: The parameter "mutdep" is likely referring to a dataset or information related to the
        mutation dependency of a gene or genes. It could contain information about how mutations in a
        specific gene or genes affect the response to a drug
        :param others: The parameter "others" refers to additional data or features that are not
        specifically related to cancer, gene, or mutation dependency. It could include any other relevant
        information that you want to consider in the analysis
        :return: The function call_entropy_per_drug returns a dataframe (a_res_df) containing the analysis
        results for a specific drug (a_drug) in relation to a specific cancer type (cancer) and gene (gene).
        The dataframe includes columns for the screenGene, cancer, candidateDrug, and label.
        """
        print(f'Analysis for {cancer}, {gene}, {a_drug}')
        sensitive_celllines = a_drug_responses['sensitive']
        resistant_celllines = a_drug_responses['resistant']
        a_res_df =  find_entropy((sensitive_celllines, resistant_celllines), (mutdep, others), display=False, significance=True, n_trials=1000, base=2, n_seed=123)
        if a_res_df is not None:
            a_res_df['screenGene'], a_res_df['cancer'], a_res_df['candidateDrug'], a_res_df['label'] = gene, cancer, a_drug, f'variant2022_gdsc27Oct23'
        return a_res_df

    @timeit
    def calculate_entropy_over_cancers_and_genes(self):
        """
        The function iterates over cancer types and genes, and performs an analysis for each combination if
        there are enough cell lines available.
        
        :param screens: A DataFrame containing information about the screens, such as the cancer type,
        screen gene, and other relevant columns
        :param drug2response: A dictionary that maps drugs to their response data. The response data should
        be in the form of a DataFrame with columns 'sensitive' and 'resistant', where each column contains a
        list of cell lines that are sensitive or resistant to the drug, respectively
        :param main_division: The main_division parameter determines the type of analysis to be performed.
        It can take one of the following values:, defaults to mutdep (optional)
        :param cancers: A list of cancer types to filter the screens dataframe. If None, all cancer types
        will be included
        :param genes: The `genes` parameter is a list of genes. It is used to filter the `screens` dataframe
        to include only rows where the `screen_gene` column matches any of the genes in the list. If `genes`
        is not provided or is set to `None`, no filtering based on genes
        :param min_celllines: The minimum number of cell lines required for both the 'mutdep' and 'others'
        groups in order to perform the analysis. If either group has fewer than this number of cell lines,
        the analysis will not be performed for that gene and cancer combination, defaults to 3 (optional)
        """
        screens = self.get_main_division()
        drug2response = self.get_stratifier2ccles_data()
        all_data_list = []
        for row_id, row in screens.iterrows():
            gene, cancer = row['screen_gene'], row['cancer_type']
            out_loc = self.results_dir / f'entropy_{cancer}_{gene}.csv'
            config.ensure_dir(out_loc)
            if os.path.exists(out_loc):
                all_data_list.append(get_result(folder_name='oncostratifier_by_drug', cancer=cancer, gene=gene, return_item_names_only=False, parallel=False))
                continue
            if 'mutdep' == self.main_division.lower:
                onco_celllines = row['list_cl_with_ess_gene_mut_ess'].split(',') if not pd.isnull(row['list_cl_with_ess_gene_mut_ess']) else np.array([])
                all_celllines = row['list_cl_with_ess'].split(',') if not pd.isnull(row['list_cl_with_ess']) else np.array([])
            elif 'mut' == self.main_division.lower:
                onco_celllines = row['list_cl_gene_mut'].split(',') if not pd.isnull(row['list_cl_gene_mut']) else np.array([])
                all_celllines = row['list_cl'].split(',') if not pd.isnull(row['list_cl']) else np.array([])
            others = np.setdiff1d(all_celllines, onco_celllines)
            
            res_df = None
            if len(onco_celllines)>=self.min_celllines and len(others)>=self.min_celllines:
                print(f'Enough cells for {gene} and {cancer}: {len(onco_celllines)} mutdep and {len(others)} others!')
                if self.parallel_process:
                    results = Parallel(n_jobs=-1)(delayed(call_entropy_per_stratifier)(a_drug, a_drug_responses, cancer, gene, onco_celllines, others) for a_drug, a_drug_responses in drug2response.items())
                else:
                    results = [call_entropy_per_stratifier(a_drug, a_drug_responses, cancer, gene, onco_celllines, others) for a_drug, a_drug_responses in drug2response.items()]
                filtered_results = [a_res for a_res in results if a_res is not None]
                if len(filtered_results)>0:
                    res_df = pd.concat(filtered_results)
                    res_df.to_csv(out_loc, index=False, sep='\t')
                    all_data_list(res_df)
            else:
                print(f'Not enough cells for {gene} and {cancer}: {len(onco_celllines)} mutdep and {len(others)} others!')
        return all_data_list

    @timeit
    def run(self):
        # Calculate entropy scores
        all_res = self.calculate_entropy_over_cancers_and_genes()
        all_res = self.process_result_file(all_res)

        if self.visual_type.lower != 'none':
            call_plot(self.visual_type, folder=self.results_dir, res=all_res, self.cancers, self.genes, target_included=True, version=self.visual_version)



        R_STATE = args.seed
        rng = np.random.default_rng(R_STATE)

        #Set the experiment folder
        if len(args.main_division)>0:
            experiment_folder_name = f'oncostratifier_by_drug_{args.main_division.lower()}'
        else:
            experiment_folder_name = f'oncostratifier_by_drug'
        ONCO_BY_DRUG_FOLDER = config.FHT_RESULTS_DIR / experiment_folder_name

        #Load drug database, check if there is a cached version
        gdsc_cache_name = f'combined_gdsc_27Oct23.xz'
        gdsc_cache_dict = config.cache_numpy(cache_name=gdsc_cache_name, is_pickle=True)
        if gdsc_cache_dict is None:
            print(f'GDSC load started!')
            gdsc1 = process_drug.get_drug(gdsc_version=1)
            gdsc2 = process_drug.get_drug(gdsc_version=2)
            gdsc_cache_dict = {'gdsc': process_drug.clear_combine_sources([gdsc1, gdsc2])}
            config.cache_numpy(cache_name=gdsc_cache_name, data=gdsc_cache_dict, is_pickle=True)
        gdsc = gdsc_cache_dict['gdsc']

        genes, cancers = None, None
        if args.main_gene is not None:
            genes = [args.main_gene]
        if args.cancer is not None:
            cancers = [args.cancer]
            
        #Load screening results and find unique genes and cancer types.
        if cancers is None or genes is None:
            all_res = onco_res.get_result(folder_name=experiment_folder_name, return_item_names_only=True)
        if cancers is None:
            cancers = all_res['cancers']#np.unique(all_res['cancer'].values)
        if genes is None:
            genes = all_res['genes']#np.unique(all_res['screenGene'].values)

        V=args.version
        RE_D = re.compile('\d')
        category_name_fix = {'stratifying essentiality': 'Stratifying Sensitivity', 'stratifying non-essentiality': 'Stratifying Resistance',
                                'co-dependency':'Full Sensitivity', 'exclusive-dependency':'Full Resistance'}

plot_circular_bar_for_all_results(folder, res, cancers, genes, target_included=False, version=1)

        if self.visual_type != 'None':
            call_plot(self.visual_type, **args)

        
        # Rppa Data
        rp_GE, rp_pat_ids, rp_ent_ids = self.read_rppa_data()

        # Somatic mutation data
        som_patients = self.read_som_data()

        # Find intersect
        rs_GE, rs_pat_ids, rp_GE, rp_pat_ids, som_patients = self.find_intersection_patients(rs_GE, rs_pat_ids, rp_GE,
                                                                                             rp_pat_ids, som_patients)

        # Kernel part
        # RnaSeq Data
        rs_GE, rs_uni_ids = self.preprocess_seq_patient_data(rs_GE, rs_ent_ids)
        all_rs_pw_map = self.read_pathways()
        labeled_all_rs_pw_map = self.label_rnaseq_patient_genes(all_rs_pw_map, rs_pat_ids, rs_GE, rs_uni_ids)
        rs_kernels = self.create_seq_kernels(labeled_all_rs_pw_map, rs_pat_ids, 'rnaseq-kms')

        # Rppa Data
        rp_GE, rp_uni_ids = self.preprocess_seq_patient_data(rp_GE, rp_ent_ids)
        all_rp_pw_map = self.read_pathways()
        labeled_all_rp_pw_map = self.label_rppa_patient_genes(all_rp_pw_map, rp_pat_ids, rp_GE, rp_uni_ids)
        rp_kernels = self.create_seq_kernels(labeled_all_rp_pw_map, rp_pat_ids, 'rppa-kms')

        # Somatic mutation data
        som_patients = self.preprocess_som_patient_data(som_patients)
        all_som_pw_map = self.read_pathways()
        labeled_all_som_pw_map = self.label_som_patient_genes(all_som_pw_map, som_patients)
        som_kernels = self.create_som_kernels(labeled_all_som_pw_map, som_patients)

        kernels = np.concatenate((rs_kernels, rp_kernels, som_kernels))

        total = kernels.shape[1] * kernels.shape[2]
        limit = (self.drop_percent * total) / 100.0
        valid_kernels = kernels[np.count_nonzero(kernels, axis=(1, 2)) >= limit]

        log(f'kernel_count={kernels.shape[0]} valid_kernel_count={valid_kernels.shape[0]}')

        cluster_sizes = [2, 3, 4, 5]
        for k in cluster_sizes:
            log(f'Running clustering for k={k}')
            self.cluster(valid_kernels, k)

        self.label_analyzer = LabelAnalysis(results_dir=self.result_dir, methods=['mkkm', 'kmeans'],
                                            cluster_sizes=cluster_sizes, log2_lambdas=self.log2_lambdas)
        self.label_analyzer.run()

def create_experiment(*nargs):
    global args

    if __name__ == '__main__':  # if running directly use command line arguments
        args = parser.parse_args()
    else:  # otherwise use user given arguments
        args = parser.parse_args(nargs)

    print_args(args)

    return Oncostratifier(args)


if __name__ == '__main__':
    create_experiment().run()