import sys, os
sys.path.insert(0, os.getcwd())
import warnings
if not sys.warnoptions:
    warnings.simplefilter("ignore")
    os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from joblib import Parallel, delayed
from src.lib import config
from src.process.results_process import previous_results
from src.process.data_process import process_drug
from src.models.oncostratifier_scores import find_entropy

R_STATE=123
PARALLEL_PROCESS=True

def call_entropy_per_drug(a_drug, a_drug_responses, cancer, gene, mutdep, others):
    """
    The function `call_entropy_per_drug` performs an analysis for a specific drug, cancer type, and
    gene, and returns the results in a dataframe.
    
    :param a_drug: The variable `a_drug` represents the drug for which you want to perform the analysis
    :param a_drug_responses: The parameter "a_drug_responses" is a dictionary that contains information
    about the drug responses. It has two keys: "sensitive" and "resistant". The value corresponding to
    each key is a list of cell lines that are either sensitive or resistant to the drug, respectively
    :param cancer: The `cancer` parameter represents the type of cancer being analyzed. It is a string
    that specifies the cancer type, such as "breast cancer" or "lung cancer"
    :param gene: The "gene" parameter represents a specific gene that is being analyzed in the context
    of cancer and drug response. It is used to identify the gene of interest for the analysis
    :param mutdep: The parameter "mutdep" is likely referring to a dataset or information related to the
    mutation dependency of a gene or genes. It could contain information about how mutations in a
    specific gene or genes affect the response to a drug
    :param others: The parameter "others" refers to additional data or features that are not
    specifically related to cancer, gene, or mutation dependency. It could include any other relevant
    information that you want to consider in the analysis
    :return: The function call_entropy_per_drug returns a dataframe (a_res_df) containing the analysis
    results for a specific drug (a_drug) in relation to a specific cancer type (cancer) and gene (gene).
    The dataframe includes columns for the screenGene, cancer, candidateDrug, and label.
    """
    print(f'Analysis for {cancer}, {gene}, {a_drug}')
    sensitive_celllines = a_drug_responses['sensitive']
    resistant_celllines = a_drug_responses['resistant']
    a_res_df =  find_entropy((sensitive_celllines, resistant_celllines), (mutdep, others), display=False, significance=True, n_trials=1000, base=2, n_seed=123)
    if a_res_df is not None:
        a_res_df['screenGene'], a_res_df['cancer'], a_res_df['candidateDrug'], a_res_df['label'] = gene, cancer, a_drug, f'variant2022_gdsc27Oct23'
    return a_res_df

def iterate_over_cancers_and_genes(screens, drug2response, main_division='mutdep', cancers=None, genes=None, min_celllines = 3):
    """
    The function iterates over cancer types and genes, and performs an analysis for each combination if
    there are enough cell lines available.
    
    :param screens: A DataFrame containing information about the screens, such as the cancer type,
    screen gene, and other relevant columns
    :param drug2response: A dictionary that maps drugs to their response data. The response data should
    be in the form of a DataFrame with columns 'sensitive' and 'resistant', where each column contains a
    list of cell lines that are sensitive or resistant to the drug, respectively
    :param main_division: The main_division parameter determines the type of analysis to be performed.
    It can take one of the following values:, defaults to mutdep (optional)
    :param cancers: A list of cancer types to filter the screens dataframe. If None, all cancer types
    will be included
    :param genes: The `genes` parameter is a list of genes. It is used to filter the `screens` dataframe
    to include only rows where the `screen_gene` column matches any of the genes in the list. If `genes`
    is not provided or is set to `None`, no filtering based on genes
    :param min_celllines: The minimum number of cell lines required for both the 'mutdep' and 'others'
    groups in order to perform the analysis. If either group has fewer than this number of cell lines,
    the analysis will not be performed for that gene and cancer combination, defaults to 3 (optional)
    """
    #drugs = list(drug2response.keys())[:50]
    #drug2response = {drug: drug2response[drug] for drug in drugs}
    if cancers is not None:
        screens = screens[screens['cancer_type'].isin(cancers)]
    if cancers is not None:
        screens = screens[screens['screen_gene'].isin(genes)]
    
    for row_id, row in screens.iterrows():
        gene, cancer = row['screen_gene'], row['cancer_type']
        out_loc = config.RESULTS_DIR / f'oncostratifier_by_drug_{main_division}' / f'entropy_{cancer}_{gene}.csv'
        config.ensure_dir(out_loc)
        if os.path.exists(out_loc):
            continue
        if 'mutdep' in main_division:
            mutdep = row['list_cl_with_ess_gene_mut_ess'].split(',') if not pd.isnull(row['list_cl_with_ess_gene_mut_ess']) else np.array([])
            all_cellines = row['list_cl_with_ess'].split(',') if not pd.isnull(row['list_cl_with_ess']) else np.array([])
        elif 'mut' in main_division:
            mutdep = row['list_cl_gene_mut'].split(',') if not pd.isnull(row['list_cl_gene_mut']) else np.array([])
            all_cellines = row['list_cl'].split(',') if not pd.isnull(row['list_cl']) else np.array([])
        others = np.setdiff1d(all_cellines, mutdep)
        res_df = None
        if len(mutdep)>=min_celllines and len(others)>=min_celllines:
            print(f'Enough cells for {gene} and {cancer}: {len(mutdep)} mutdep and {len(others)} others!')
            if PARALLEL_PROCESS:
                results = Parallel(n_jobs=-1)(delayed(call_entropy_per_drug)(a_drug, a_drug_responses, cancer, gene, mutdep, others) for a_drug, a_drug_responses in drug2response.items())
                #res_df = pd.concat(results)
            else:
                for a_drug, a_drug_responses in drug2response.items():
                    print(f'Analysis for {cancer}, {gene}, {a_drug}')
                    sensitive_celllines = a_drug_responses['sensitive']
                    resistant_celllines = a_drug_responses['resistant']
                    a_res_df = find_entropy((sensitive_celllines, resistant_celllines), (mutdep, others), display=False, significance=True, n_trials=1000, base=2, n_seed=123)
                    if a_res_df is not None:
                        a_res_df['screenGene'], a_res_df['cancer'], a_res_df['candidateDrug'], a_res_df['label'] = gene, cancer, a_drug, f'variant2022_gdsc27Oct23'
                    results.append(a_res_df)
            filtered_results = [a_res for a_res in results if a_res is not None]
            if len(filtered_results)>0:
                res_df = pd.concat(filtered_results)
                res_df.to_csv(out_loc, index=False, sep='\t')
        else:
            print(f'Not enough cells for {gene} and {cancer}: {len(mutdep)} mutdep and {len(others)} others!')

def main():
    """
    The main function loads gene screens and drug response data from GDSC, and then iterates over
    cancers and genes.
    """
    screens = previous_results.get_detailed_gene_screens(variant='2022', fixed=True)
    gdsc_drug2response_cache_name = f'gdsc_drug2response_27Oct23.xz'
    gdsc_drug2response = config.cache_numpy(cache_name=gdsc_drug2response_cache_name, is_pickle=True)
    if gdsc_drug2response is None:
        print(f'GDSC gene2response load started!')
        gdsc_cache_name = f'combined_gdsc_27Oct23.xz'
        gdsc_cache_dict = config.cache_numpy(cache_name=gdsc_cache_name, is_pickle=True)
        if gdsc_cache_dict is None:
            print(f'GDSC load started!')
            gdsc1 = process_drug.get_drug(gdsc_version=1)
            gdsc2 = process_drug.get_drug(gdsc_version=2)
            gdsc_cache_dict = {'gdsc': process_drug.clear_combine_sources([gdsc1, gdsc2])}
            config.cache_numpy(cache_name=gdsc_cache_name, data=gdsc_cache_dict, is_pickle=True)
        gdsc = gdsc_cache_dict['gdsc']
        
        gdsc_drug2response = {}
        for drug in gdsc['UNIQUE_DRUG'].unique():
            drug_df = gdsc[gdsc['UNIQUE_DRUG']==drug]
            gdsc_drug2response[drug] = {'sensitive':drug_df[drug_df['LN_SENSITIVE']==1]['CELL_LINE_NAME'].values, 'resistant':drug_df[drug_df['LN_SENSITIVE']==0]['CELL_LINE_NAME'].values}
        config.cache_numpy(cache_name=gdsc_drug2response_cache_name, data=gdsc_drug2response, is_pickle=True)
    
    iterate_over_cancers_and_genes(screens, gdsc_drug2response, main_division='mut')

if __name__ == "__main__":
    main()