import sys, os
sys.path.insert(0, os.getcwd())
import warnings
if not sys.warnoptions:
    warnings.simplefilter("ignore")
    os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses
from src.lib.sutils import *
from src.lib import config
from src.models import select_model
from src.process.results_process import previous_results
from src.process.data_process import extract_features

import numpy as np
import pandas as pd
from sklearn.model_selection import learning_curve, StratifiedKFold, KFold
from sklearn.preprocessing import StandardScaler
from sklearn.metrics import accuracy_score, f1_score, average_precision_score, roc_auc_score, matthews_corrcoef, balanced_accuracy_score
import matplotlib.pyplot as plt
import seaborn as sns
import argparse

R_STATE=123

parser = argparse.ArgumentParser(description='Prediction of entropy scores (geneXcancer) using linear model and AA sequence features')
parser.add_argument('--main_gene', '-mg', metavar='the-main-gene', dest='main_gene', type=str,
                    help='Choose the main gene', default='KRAS')
parser.add_argument('--cancer', '-c', metavar='the-cancer', dest='cancer', type=str,
                    help='Choose cancer', default='Colorectal Carcinoma')
parser.add_argument('--variant', '-v', metavar='the-variant-version', dest='variant', type=str,
                    help='Choose variant version', default='2022')
parser.add_argument('--model', '-m', metavar='the-model', dest='model', type=str,
                    help='Choose the ML model', default='LRCV_v0')
parser.add_argument('--feature', '-f', metavar='the-feature', dest='feature', type=str,
                    help='Choose the features for genes', default='gene_sequence')
parser.add_argument('--prediction_class', '-pc', metavar='the-prediction-class', dest='prediction_class', type=str,
                    help='Choose what to predict', default='2category')
args = parser.parse_args()
print(f'Running args:{args}')

def report(res_df, pdf_loc, scoring = ['accuracy', 'f1_score', 'AUROC', 'AUPRC', 'MCC', 'balanced_accuracy']):
    '''
    res_df.groupby(['set'])
    print('Training scores:\n\n', res_df[res_df['set']='train'])
    print('\n', '-' * 70) # separator to make the output easy to read
    print('\nValidation scores:\n\n', validation_scores)
    train_scores_mean = train_scores.mean(axis = 1)
    validation_scores_mean = validation_scores.mean(axis = 1)
    if 'neg' in scoring:
        train_scores_mean = -train_scores_mean
        validation_scores_mean = -validation_scores
    print('Mean training scores\n\n', pd.Series(train_scores_mean, index = train_sizes))
    print('\n', '-' * 20) # separator
    print('\nMean validation scores\n\n',pd.Series(validation_scores_mean, index = train_sizes))
    '''
    ids = ['cancer', 'variant', 'main_gene', 'model_name', 'fold_id', 'set']
    melted_df = res_df.melt(id_vars=ids, value_vars=scoring).rename(columns={'variable': 'Metric', 'value': 'Score'})
    melted_df['Metric'] = melted_df['Metric'].str.replace('_','\n').str.capitalize()
    sns.boxplot(melted_df, x='Metric', y='Score', hue='set', fill=False, gap=.1)
    #plt.plot(train_sizes, train_scores_mean, label = 'Training')
    #plt.plot(train_sizes, validation_scores_mean, label = 'Validation')
    #plt.ylabel('Sc', fontsize = 14)
    #plt.xlabel('Training set size', fontsize = 14)
    #plt.title('Performance', fontsize = 18, y = 1.03)
    plt.legend(bbox_to_anchor=(1, 1), loc='upper left', borderaxespad=0, frameon=False)
    plt.savefig(pdf_loc, dpi=300, bbox_inches='tight')

def calculate_scores(actual, preds, probs):
    scores = {}
    scores['accuracy'] = accuracy_score(actual, preds)
    scores['f1_score'] = f1_score(actual, preds)
    scores['AUPRC'] = average_precision_score(actual, probs)
    scores['AUROC'] = roc_auc_score(actual, probs)
    scores['MCC'] = matthews_corrcoef(actual, preds)
    scores['bal_accuracy'] = balanced_accuracy_score(actual, preds)
    return scores

def fit_predict_per_gene():
    main_gene = args.main_gene
    cancer = args.cancer.replace(' ', '_')
    variant=args.variant
    model_name=args.model
    feature_type=args.feature
    prediction_class=args.prediction_class
    main_cv_fold = 5
    filename_replacements = {'gene_sequence': 'seq', 'diffEntropy':'entropy', '3category': '3category', '2category': '2category', 'gene_expression': 'exp'}

    add_categories=False
    only_significant=False
    scoring = 'neg_mean_squared_error'
    if 'category' in prediction_class:
        add_categories=True
        scoring = 'f1_weighted'
    elif 'Entropy' in prediction_class:
        scoring = 'neg_mean_squared_error'
    
    param_str = f'{main_gene}_{cancer}_{variant}_{model_name}_{feature_type}'
    res_name = config.str2path(f'linear_{filename_replacements[feature_type]}2{filename_replacements[prediction_class]}') / param_str / f'cv{main_cv_fold}.npz'
    pdf_loc = config.RESULTS_DIR / f'linear_{filename_replacements[feature_type]}2{filename_replacements[prediction_class]}' / param_str / f'cv{main_cv_fold}.pdf' 
    config.ensure_dir(pdf_loc)
    res_dict = config.cache_numpy(cache_name=res_name, is_pickle=True)
    if res_dict is not None:
        report(res_dict['res_df'], pdf_loc)
    else:
        entropy_df = previous_results.get_entropy_scores(gene=main_gene, cancer=cancer, variant=variant, add_categories=add_categories, only_significant=only_significant)
        entropy_df  = entropy_df.set_index('candidateGene')
        genes = entropy_df.index.values
        
        cache_name = f'{filename_replacements[feature_type]}2entropy_{main_gene}_{cancer}_{variant}_{feature_type}_{len(genes)}genes_prott5.npz'
        genes_features_dict = config.cache_numpy(cache_name=cache_name)
        if genes_features_dict is None:
            print(f'Sequence feature generation started! {len(genes)}')
            genes_features_dict = extract_features.call_feature_extraction(args.feature, sample_list = genes)
            config.cache_numpy(cache_name=cache_name, data=genes_features_dict)
            
        X, gene_list = genes_features_dict['features'], genes_features_dict['genes']
        print(f'Sequence feature generation ended! {len(gene_list)}')
        featurized_entropy_df = entropy_df.loc[gene_list]
        y = featurized_entropy_df[prediction_class].values
        
        #X = StandardScaler().fit_transform(X.copy())

        skf = StratifiedKFold(n_splits=main_cv_fold, random_state=R_STATE, shuffle=True)
        results_title = ['cancer', 'variant', 'main_gene', 'model_name', 'fold_id', 'set', 'accuracy', 'f1_score', 'AUROC', 'AUPRC', 'MCC', 'balanced_accuracy']
        results = []
        models = []
        splits = []
        for i, (train_index, test_index) in enumerate(skf.split(X, y)):
            splits.append(((train_index, test_index)))
            X_train, y_train = X[train_index], y[train_index]
            X_val, y_val = X[test_index], y[test_index]
            standard_scaler = StandardScaler()
            X_train = standard_scaler.fit_transform(X_train.copy())
            X_val = standard_scaler.transform(X_val.copy())
            
            model = select_model.return_model(model_name)
            model.fit(X_train, y_train)
            models.append(model)
            preds_train = model.predict(X_train)
            preds_val = model.predict(X_val)
            probs_train = model.predict_proba(X_train)[:,1]
            probs_val = model.predict_proba(X_val)[:,1]
            
            tr_scores = calculate_scores(y_train, preds_train, probs_train)
            val_scores = calculate_scores(y_val, preds_val, probs_val)
            
            fold_res_train = [cancer, variant, main_gene, model_name, f'{i}', 'train', tr_scores['accuracy'], tr_scores['f1_score'], tr_scores['AUROC'], tr_scores['AUPRC'], tr_scores['MCC'], tr_scores['bal_accuracy']]# , , , , ]
            fold_res_val = [cancer, variant, main_gene, model_name, f'{i}', 'val', val_scores['accuracy'], val_scores['f1_score'], val_scores['AUROC'], val_scores['AUPRC'], val_scores['MCC'], val_scores['bal_accuracy']]
            results.append(fold_res_train)
            results.append(fold_res_val)
            
        df = pd.DataFrame(results, columns=results_title)
        res_dict = {'res_df': df, 'models': models, 'genes': genes, 'split_indices': splits}
        config.cache_numpy(cache_name=res_name, data=res_dict, is_pickle=True)
        report(res_dict['res_df'], pdf_loc)


def main():
    fit_predict_per_gene()

if __name__ == "__main__":
    main()


