import sys, os
sys.path.insert(0, os.getcwd())
import warnings
if not sys.warnoptions:
    warnings.simplefilter("ignore")
    os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses
from src.lib import config
from src.models import select_model
from src.process.results_process import previous_results
from src.process.data_process import extract_features

import numpy as np
import pandas as pd
from sklearn.model_selection import learning_curve, KFold
from sklearn.preprocessing import StandardScaler
import matplotlib.pyplot as plt
import argparse

R_STATE=123

parser = argparse.ArgumentParser(description='Prediction of entropy scores (geneXcancer) using linear model and AA sequence features')
parser.add_argument('--main_gene', '-mg', metavar='the-main-gene', dest='main_gene', type=str,
                    help='Choose the main gene', default='KRAS')
parser.add_argument('--cancer', '-c', metavar='the-cancer', dest='cancer', type=str,
                    help='Choose cancer', default='Colorectal Carcinoma')
parser.add_argument('--variant', '-v', metavar='the-variant-version', dest='variant', type=str,
                    help='Choose variant version', default='2022')
parser.add_argument('--model', '-m', metavar='the-model', dest='model', type=str,
                    help='Choose the ML model', default='LRCV_v0')
parser.add_argument('--feature', '-f', metavar='the-feature', dest='feature', type=str,
                    help='Choose the features for genes', default='gene_sequence')
parser.add_argument('--prediction_class', '-pc', metavar='the-prediction-class', dest='prediction_class', type=str,
                    help='Choose what to predict', default='2category')
args = parser.parse_args()
print(f'Running args:{args}')

def report(res_dict, pdf_loc, scoring):
    train_sizes, train_scores, validation_scores = res_dict['train_sizes'], res_dict['train_scores'], res_dict['validation_scores']
    print('Training scores:\n\n', train_scores)
    print('\n', '-' * 70) # separator to make the output easy to read
    print('\nValidation scores:\n\n', validation_scores)
    train_scores_mean = train_scores.mean(axis = 1)
    validation_scores_mean = validation_scores.mean(axis = 1)
    if 'neg' in scoring:
        train_scores_mean = -train_scores_mean
        validation_scores_mean = -validation_scores
    print('Mean training scores\n\n', pd.Series(train_scores_mean, index = train_sizes))
    print('\n', '-' * 20) # separator
    print('\nMean validation scores\n\n',pd.Series(validation_scores_mean, index = train_sizes))

    plt.plot(train_sizes, train_scores_mean, label = 'Training')
    plt.plot(train_sizes, validation_scores_mean, label = 'Validation')
    plt.ylabel(scoring, fontsize = 14)
    plt.xlabel('Training set size', fontsize = 14)
    plt.title('Learning curves for model', fontsize = 18, y = 1.03)
    plt.legend()
    plt.savefig(pdf_loc)

def fit_predict_per_gene():
    main_gene = args.main_gene
    cancer = args.cancer.replace(' ', '_')
    variant=args.variant
    model_name=args.model
    feature_type=args.feature
    prediction_class=args.prediction_class
    learning_curve_cv = 5
    train_sizes = [50, 500, 1000, 1500, 2000]
    filename_replacements = {'gene_sequence': 'seq', 'diffEntropy':'entropy', '3category': '3category', '2category': '2category', 'gene_expression': 'exp'}

    add_categories=False
    only_significant=False
    scoring = 'neg_mean_squared_error'
    if 'category' in prediction_class:
        add_categories=True
        scoring = 'f1_weighted'
    elif 'Entropy' in prediction_class:
        scoring = 'neg_mean_squared_error'
    
    param_str = f'{main_gene}_{cancer}_{variant}_{model_name}_{feature_type}'
    res_name = config.str2path(f'linear_{filename_replacements[feature_type]}2{filename_replacements[prediction_class]}') / param_str / f'curve_cv{learning_curve_cv}_sizes{"|".join(map(str, train_sizes))}.npz'
    pdf_loc = config.RESULTS_DIR / f'linear_{filename_replacements[feature_type]}2{filename_replacements[prediction_class]}' / param_str / f'curve_cv{learning_curve_cv}_sizes{"|".join(map(str, train_sizes))}.pdf' 
    config.ensure_dir(pdf_loc)
    res_dict = config.cache_numpy(cache_name=res_name)
    if res_dict is not None:
        report(res_dict, pdf_loc, scoring=scoring)
    else:
        model = select_model.return_model(model_name)
        entropy_df = previous_results.get_entropy_scores(gene=main_gene, cancer=cancer, variant=variant, add_categories=add_categories, only_significant=only_significant)
        entropy_df  = entropy_df.set_index('candidateGene')
        genes = entropy_df.index.values
        
        cache_name = f'{filename_replacements[feature_type]}2entropy_{main_gene}_{cancer}_{variant}_{feature_type}_{len(genes)}genes_prott5.npz'
        genes_features_dict = config.cache_numpy(cache_name=cache_name)
        if genes_features_dict is None:
            print(f'Sequence feature generation started! {len(genes)}')
            genes_features_dict = extract_features.call_feature_extraction(args.feature, sample_list = genes)
            config.cache_numpy(cache_name=cache_name, data=genes_features_dict)
            
        X, gene_list = genes_features_dict['features'], genes_features_dict['genes']
        print(f'Sequence feature generation ended! {len(gene_list)}')
        featurized_entropy_df = entropy_df.loc[gene_list]
        y = featurized_entropy_df[prediction_class].values
        
        X = StandardScaler().fit_transform(X.copy())

        
        
        train_sizes, train_scores, validation_scores = learning_curve(
            estimator = model, X = X, y = y, train_sizes = train_sizes, verbose = 2, 
            cv = KFold(n_splits=learning_curve_cv, random_state=R_STATE, shuffle=True), 
            scoring = scoring)
        
        res_dict = {'train_sizes': train_sizes, 'train_scores': train_scores, 'validation_scores': validation_scores}
        config.cache_numpy(cache_name=res_name, data=res_dict)
        report(res_dict, pdf_loc, scoring=scoring)


def main():
    fit_predict()

if __name__ == "__main__":
    main()


