import sys, os
sys.path.insert(0, os.getcwd())
import warnings
if not sys.warnoptions:
    warnings.simplefilter("ignore")
    os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses
from src.lib import config
from src.models import select_model
from src.process.results_process import previous_results
from src.process.data_process import extract_features

import numpy as np
import pandas as pd
from sklearn.model_selection import learning_curve, KFold
from sklearn.preprocessing import StandardScaler
import matplotlib.pyplot as plt
import argparse

R_STATE=123

parser = argparse.ArgumentParser(description='Prediction of entropy scores (geneXcancer) using nonlinear model and AA sequence features')
parser.add_argument('--gene', '-g', metavar='the-main-gene', dest='main_gene', type=str,
                    help='Choose the main gene', default='KRAS')
parser.add_argument('--cancer', '-c', metavar='the-cancer', dest='cancer', type=str,
                    help='Choose cancer', default='Colorectal Carcinoma')
parser.add_argument('--variant', '-v', metavar='the-variant-version', dest='variant', type=str,
                    help='Choose variant version', default='2022')
parser.add_argument('--model', '-m', metavar='the-model', dest='model', type=str,
                    help='Choose the ML model', default='RFR_v0')
parser.add_argument('--feature', '-f', metavar='the-feature', dest='feature', type=str,
                    help='Choose the features for genes', default='gene_sequence')
args = parser.parse_args()
print(f'Running args:{args}')

main_gene = args.main_gene
cancer = args.cancer.replace(' ', '_')
variant=args.variant
model_name=args.model
feature_type=args.feature
learning_curve_cv = 5
train_sizes = [50, 500, 1000, 1500, 2000]

param_str = f'{main_gene}_{cancer}_{variant}_{model_name}_{feature_type}'
res_name = config.str2path('nonlinear_seq2entropy') / param_str / f'curve_cv{learning_curve_cv}_sizes{"|".join(map(str, train_sizes))}.npz'
pdf_loc = config.RESULTS_DIR / 'nonlinear_seq2entropy' / param_str / f'curve_cv{learning_curve_cv}_sizes{"|".join(map(str, train_sizes))}.pdf' 
config.ensure_dir(pdf_loc)
res_dict = config.cache_numpy(cache_name=res_name)
if res_dict is not None:
    train_sizes, train_scores, validation_scores = res_dict['train_sizes'], res_dict['train_scores'], res_dict['validation_scores']
else:
    model = select_model.return_model(model_name)
    entropy_df = previous_results.get_entropy_scores(gene=main_gene, cancer=cancer, variant=variant)
    entropy_df  = entropy_df.set_index('candidateGene')
    genes = entropy_df.index.values
    
    cache_name = f'seq2entropy_{main_gene}_{cancer}_{variant}_{feature_type}_{len(genes)}genes_prott5.npz'
    genes_features_dict = config.cache_numpy(cache_name=cache_name)
    if genes_features_dict is None:
        print(f'Sequence feature generation started! {len(genes)}')
        genes_features_dict = extract_features.call_feature_extraction(args.feature, sample_list = genes)
        config.cache_numpy(cache_name=cache_name, data=genes_features_dict)
        
    X, gene_list = genes_features_dict['features'], genes_features_dict['genes']
    print(f'Sequence feature generation ended! {len(gene_list)}')
    featurized_entropy_df = entropy_df.loc[gene_list]
    diff_entropies = featurized_entropy_df['diffEntropy'].values
    p_values = featurized_entropy_df['p'].values
    
    X = StandardScaler().fit_transform(X.copy())

    train_sizes, train_scores, validation_scores = learning_curve(
        estimator = model, X = X, y = diff_entropies, train_sizes = train_sizes, verbose = 2, 
        cv = KFold(n_splits=learning_curve_cv, random_state=R_STATE, shuffle=True), 
        scoring = 'neg_mean_squared_error')
    res_dict = {'train_sizes': train_sizes, 'train_scores': train_scores, 'validation_scores': validation_scores}
    config.cache_numpy(cache_name=res_name, data=res_dict)


print('Training scores:\n\n', train_scores)
print('\n', '-' * 70) # separator to make the output easy to read
print('\nValidation scores:\n\n', validation_scores)


train_scores_mean = -train_scores.mean(axis = 1)
validation_scores_mean = -validation_scores.mean(axis = 1)
print('Mean training scores\n\n', pd.Series(train_scores_mean, index = train_sizes))
print('\n', '-' * 20) # separator
print('\nMean validation scores\n\n',pd.Series(validation_scores_mean, index = train_sizes))

plt.plot(train_sizes, train_scores_mean, label = 'Training error')
plt.plot(train_sizes, validation_scores_mean, label = 'Validation error')
plt.ylabel('MSE', fontsize = 14)
plt.xlabel('Training set size', fontsize = 14)
plt.title('Learning curves for a nonlinear regression model', fontsize = 18, y = 1.03)
plt.legend()
#plt.ylim(0,40)
plt.savefig(pdf_loc)

#model.fit(X, diff_entropies)
#print(model.alpha_)
#print(model.intercept_)


