import pandas as pd
from scipy.stats import chisquare, chi2_contingency, fisher_exact, entropy, barnard_exact, boschloo_exact
from statsmodels.stats.multitest import multipletests
import numpy as np


def find_mutational_significance(mutation, partitioned_cls, display=False, min_cells=3):
    """
    The function `find_mutational_significance` takes in mutation data and partitioned cell lines, and
    calculates various statistical tests to determine the mutational significance of the mutation.
    
    :param mutation: The "mutation" parameter is a tuple containing two lists. The first list represents
    the cells that have undergone mutation, and the second list represents the cells that have not
    undergone mutation
    :param partitioned_cls: The parameter "partitioned_cls" is a tuple containing four arrays:
    mutDep_sensitive, mutDep_resistant, others_sensitive, and others_resistant. These arrays represent
    different groups or classes of data. "mutDep_sensitive" contains the data for the mutation-dependent
    sensitive group, "mutDep_res
    :param display: The `display` parameter is a boolean value that determines whether or not to display
    the results. If set to `True`, the function will print the results. If set to `False`, the function
    will only return the results as a DataFrame, defaults to False (optional)
    :param min_cells: The parameter "min_cells" is the minimum number of cells required in each group
    (mutated and not mutated) for the analysis to be performed. If either the mutated or not mutated
    group has fewer than "min_cells" cells, the function will return None, defaults to 3 (optional)
    :return: a pandas DataFrame containing various statistics and information related to the mutational
    significance of a given mutation.
    """
    mutated, not_mutated = mutation
    all_mutdatas = np.union1d(mutated, not_mutated)
    mutDep_sensitive, mutDep_resistant, others_sensitive, others_resistant = partitioned_cls
    #mutDep, others = partitioned_cls
    mutDep = np.union1d(mutDep_sensitive, mutDep_resistant)
    others = np.union1d(others_sensitive, others_resistant)
    all_cls = np.union1d(mutDep, others)
    all_cls_with_mutdata = np.intersect1d(all_cls, all_mutdatas)
    all_cls_without_mutdata = np.setdiff1d(all_cls, all_mutdatas)
    all_cls_with_mutdata_mutated = np.intersect1d(all_cls, mutated)
    all_cls_with_mutdata_not_mutated = np.intersect1d(all_cls, not_mutated)
    mutDep_with_mutdata = np.intersect1d(mutDep, all_mutdatas)
    others_with_mutdata = np.intersect1d(others, all_mutdatas)
    if len(mutDep_with_mutdata)<min_cells or len(others_with_mutdata)<min_cells:
        return None
    mutDep_with_mutdata_mutated = np.intersect1d(mutDep, mutated)
    mutDep_with_mutdata_not_mutated = np.intersect1d(mutDep, not_mutated)
    mutDep_sensitive_with_mutdata = np.intersect1d(mutDep_sensitive, all_mutdatas)
    mutDep_sensitive_with_mutdata_mutated = np.intersect1d(mutDep_sensitive, mutated)
    mutDep_sensitive_with_mutdata_not_mutated = np.intersect1d(mutDep_sensitive, not_mutated)
    mutDep_resistant_with_mutdata = np.intersect1d(mutDep_resistant, all_mutdatas)
    mutDep_resistant_with_mutdata_mutated = np.intersect1d(mutDep_resistant, mutated)
    mutDep_resistant_with_mutdata_not_mutated = np.intersect1d(mutDep_resistant, not_mutated)
    others_with_mutdata_mutated = np.intersect1d(others, mutated)
    others_with_mutdata_not_mutated = np.intersect1d(others, not_mutated)
    others_sensitive_with_mutdata = np.intersect1d(others_sensitive, all_mutdatas)
    others_sensitive_with_mutdata_mutated = np.intersect1d(others_sensitive, mutated)
    others_sensitive_with_mutdata_not_mutated = np.intersect1d(others_sensitive, not_mutated)
    others_resistant_with_mutdata = np.intersect1d(others_resistant, all_mutdatas)
    others_resistant_with_mutdata_mutated = np.intersect1d(others_resistant, mutated)
    others_resistant_with_mutdata_not_mutated = np.intersect1d(others_resistant, not_mutated)
    
    #Compare within mutdep
    contingency_for_mutdep = [[len(mutDep_sensitive_with_mutdata_mutated), len(mutDep_sensitive_with_mutdata_not_mutated)], 
                       [len(mutDep_resistant_with_mutdata_mutated), len(mutDep_resistant_with_mutdata_not_mutated)]]
    mutdep_fisher_exact_res = fisher_exact(contingency_for_mutdep) #statistic, pvalue
    mutdep_barnard_exact_res = barnard_exact(contingency_for_mutdep) #statistic, pvalue
    mutdep_boschloo_exact_res = boschloo_exact(contingency_for_mutdep) #statistic, pvalue
    
    #Compare within others
    contingency_for_others = [[len(others_sensitive_with_mutdata_mutated), len(others_sensitive_with_mutdata_not_mutated)], 
                       [len(others_resistant_with_mutdata_mutated), len(others_resistant_with_mutdata_not_mutated)]]
    others_fisher_exact_res = fisher_exact(contingency_for_others) #statistic, pvalue
    others_barnard_exact_res = barnard_exact(contingency_for_others) #statistic, pvalue
    others_boschloo_exact_res = boschloo_exact(contingency_for_others) #statistic, pvalue
    
    #Compare mutdep sensitive vs others sensitive
    contingency_for_mutdepSens_vs_othersSens = [[len(mutDep_sensitive_with_mutdata_mutated), len(mutDep_sensitive_with_mutdata_not_mutated)], 
                       [len(others_sensitive_with_mutdata_mutated), len(others_sensitive_with_mutdata_not_mutated)]]
    mutdepSens_vs_othersSens_fisher_exact_res = fisher_exact(contingency_for_mutdepSens_vs_othersSens) #statistic, pvalue
    mutdepSens_vs_othersSens_barnard_exact_res = barnard_exact(contingency_for_mutdepSens_vs_othersSens) #statistic, pvalue
    mutdepSens_vs_othersSens_boschloo_exact_res = boschloo_exact(contingency_for_mutdepSens_vs_othersSens) #statistic, pvalue
    
    #Compare mutdep sensitive vs others resistant
    contingency_for_mutdepSens_vs_othersResist = [[len(mutDep_sensitive_with_mutdata_mutated), len(mutDep_sensitive_with_mutdata_not_mutated)], 
                       [len(others_resistant_with_mutdata_mutated), len(others_resistant_with_mutdata_not_mutated)]]
    mutdepSens_vs_othersResist_fisher_exact_res = fisher_exact(contingency_for_mutdepSens_vs_othersResist) #statistic, pvalue
    mutdepSens_vs_othersResist_barnard_exact_res = barnard_exact(contingency_for_mutdepSens_vs_othersResist) #statistic, pvalue
    mutdepSens_vs_othersResist_boschloo_exact_res = boschloo_exact(contingency_for_mutdepSens_vs_othersResist) #statistic, pvalue
    
    #Compare mutdep resistant vs others sensitive
    contingency_for_mutdepResist_vs_othersSens = [[len(mutDep_resistant_with_mutdata_mutated), len(mutDep_resistant_with_mutdata_not_mutated)], 
                       [len(others_sensitive_with_mutdata_mutated), len(others_sensitive_with_mutdata_not_mutated)]]
    mutdepResist_vs_othersSens_fisher_exact_res = fisher_exact(contingency_for_mutdepResist_vs_othersSens) #statistic, pvalue
    mutdepResist_vs_othersSens_barnard_exact_res = barnard_exact(contingency_for_mutdepResist_vs_othersSens) #statistic, pvalue
    mutdepResist_vs_othersSens_boschloo_exact_res = boschloo_exact(contingency_for_mutdepResist_vs_othersSens) #statistic, pvalue
    
    #Compare mutdep resistant vs others resistant
    contingency_for_mutdepResist_vs_othersResist = [[len(mutDep_resistant_with_mutdata_mutated), len(mutDep_resistant_with_mutdata_not_mutated)], 
                       [len(others_resistant_with_mutdata_mutated), len(others_resistant_with_mutdata_not_mutated)]]
    mutdepResist_vs_othersResist_fisher_exact_res = fisher_exact(contingency_for_mutdepResist_vs_othersResist) #statistic, pvalue
    mutdepResist_vs_othersResist_barnard_exact_res = barnard_exact(contingency_for_mutdepResist_vs_othersResist) #statistic, pvalue
    mutdepResist_vs_othersResist_boschloo_exact_res = boschloo_exact(contingency_for_mutdepResist_vs_othersResist) #statistic, pvalue
    
    data_res = pd.DataFrame({
        'screenGene': ['TBF'],
        'cancer': ['TBF'],
        'candidateDrug': ['TBF'],
        'label': ['TBF'],
        'mutdep_fisher_stat': [mutdep_fisher_exact_res.statistic],
        'mutdep_fisher_p': [mutdep_fisher_exact_res.pvalue],
        'mutdep_barnard_stats': [mutdep_barnard_exact_res.statistic],
        'mutdep_barnard_p': [mutdep_barnard_exact_res.pvalue],
        'mutdep_boschloo_stats': [mutdep_boschloo_exact_res.statistic],
        'mutdep_boschloo_p': [mutdep_boschloo_exact_res.pvalue],
        'others_fisher_stat': [others_fisher_exact_res.statistic],
        'others_fisher_p': [others_fisher_exact_res.pvalue],
        'others_barnard_stats': [others_barnard_exact_res.statistic],
        'others_barnard_p': [others_barnard_exact_res.pvalue],
        'others_boschloo_stats': [others_boschloo_exact_res.statistic],
        'others_boschloo_p': [others_boschloo_exact_res.pvalue],
        'mutdepSens_vs_othersSens_fisher_stat': [mutdepSens_vs_othersSens_fisher_exact_res.statistic],
        'mutdepSens_vs_othersSens_fisher_p': [mutdepSens_vs_othersSens_fisher_exact_res.pvalue],
        'mutdepSens_vs_othersSens_barnard_stats': [mutdepSens_vs_othersSens_barnard_exact_res.statistic],
        'mutdepSens_vs_othersSens_barnard_p': [mutdepSens_vs_othersSens_barnard_exact_res.pvalue],
        'mutdepSens_vs_othersSens_boschloo_stats': [mutdepSens_vs_othersSens_boschloo_exact_res.statistic],
        'mutdepSens_vs_othersSens_boschloo_p': [mutdepSens_vs_othersSens_boschloo_exact_res.pvalue],
        'mutdepSens_vs_othersResist_fisher_stat': [mutdepSens_vs_othersResist_fisher_exact_res.statistic],
        'mutdepSens_vs_othersResist_fisher_p': [mutdepSens_vs_othersResist_fisher_exact_res.pvalue],
        'mutdepSens_vs_othersResist_barnard_stats': [mutdepSens_vs_othersResist_barnard_exact_res.statistic],
        'mutdepSens_vs_othersResist_barnard_p': [mutdepSens_vs_othersResist_barnard_exact_res.pvalue],
        'mutdepSens_vs_othersResist_boschloo_stats': [mutdepSens_vs_othersResist_boschloo_exact_res.statistic],
        'mutdepSens_vs_othersResist_boschloo_p': [mutdepSens_vs_othersResist_boschloo_exact_res.pvalue],
        'mutdepResist_vs_othersSens_fisher_stat': [mutdepResist_vs_othersSens_fisher_exact_res.statistic],
        'mutdepResist_vs_othersSens_fisher_p': [mutdepResist_vs_othersSens_fisher_exact_res.pvalue],
        'mutdepResist_vs_othersSens_barnard_stats': [mutdepResist_vs_othersSens_barnard_exact_res.statistic],
        'mutdepResist_vs_othersSens_barnard_p': [mutdepResist_vs_othersSens_barnard_exact_res.pvalue],
        'mutdepResist_vs_othersSens_boschloo_stats': [mutdepResist_vs_othersSens_boschloo_exact_res.statistic],
        'mutdepResist_vs_othersSens_boschloo_p': [mutdepResist_vs_othersSens_boschloo_exact_res.pvalue],
        'mutdepResist_vs_othersResist_fisher_stat': [mutdepResist_vs_othersResist_fisher_exact_res.statistic],
        'mutdepResist_vs_othersResist_fisher_p': [mutdepResist_vs_othersResist_fisher_exact_res.pvalue],
        'mutdepResist_vs_othersResist_barnard_stats': [mutdepResist_vs_othersResist_barnard_exact_res.statistic],
        'mutdepResist_vs_othersResist_barnard_p': [mutdepResist_vs_othersResist_barnard_exact_res.pvalue],
        'mutdepResist_vs_othersResist_boschloo_stats': [mutdepResist_vs_othersResist_boschloo_exact_res.statistic],
        'mutdepResist_vs_othersResist_boschloo_p': [mutdepResist_vs_othersResist_boschloo_exact_res.pvalue],
        
        'mutDep_n': [len(mutDep)], #mutDep sizes
        'mutDep_with_mutdata_n': [len(mutDep_with_mutdata)],
        'mutDep_mutated_n': [len(mutDep_with_mutdata_mutated)],
        'mutDep_not_mutated_n': [len(mutDep_with_mutdata_not_mutated)],
        'others_n': [len(others)], #Others sizes
        'others_with_mutdata_n': [len(others_with_mutdata)],
        'others_mutated_n': [len(others_with_mutdata_mutated)],
        'others_not_mutated_n': [len(others_with_mutdata_not_mutated)],
        'mutDep_sensitive_n': [len(mutDep_sensitive)], #mutDep sensitive sizes
        'mutDep_sensitive_with_mutdata_n': [len(mutDep_sensitive_with_mutdata)],
        'mutDep_sensitive_mutated_n': [len(mutDep_sensitive_with_mutdata_mutated)],
        'mutDep_sensitive_not_mutated_n': [len(mutDep_sensitive_with_mutdata_not_mutated)],
        'mutDep_resistant_n': [len(mutDep_resistant)], #mutDep resistant sizes
        'mutDep_resistant_with_mutdata_n': [len(mutDep_resistant_with_mutdata)],
        'mutDep_resistant_mutated_n': [len(mutDep_resistant_with_mutdata_mutated)],
        'mutDep_resistant_not_mutated_n': [len(mutDep_resistant_with_mutdata_not_mutated)],
        'others_sensitive_n': [len(others_sensitive)], #others sensitive sizes
        'others_sensitive_with_mutdata_n': [len(others_sensitive_with_mutdata)],
        'others_sensitive_mutated_n': [len(others_sensitive_with_mutdata_mutated)],
        'others_sensitive_not_mutated_n': [len(others_sensitive_with_mutdata_not_mutated)],
        'others_resistant_n': [len(others_resistant)], #others resistant sizes
        'others_resistant_with_mutdata_n': [len(others_resistant_with_mutdata)],
        'others_resistant_mutated_n': [len(others_resistant_with_mutdata_mutated)],
        'others_resistant_not_mutated_n': [len(others_resistant_with_mutdata_not_mutated)],
        'all_n': [len(all_cls)], #All sizes
        'all_with_mutdata_n': [len(all_cls_with_mutdata)],
        'all_mutated_n': [len(all_cls_with_mutdata_mutated)],
        'all_not_mutated_n': [len(all_cls_with_mutdata_not_mutated)],
        
        'mutDep_list': [','.join(mutDep)], #mutDep list
        'mutDep_with_mutdata_list': [','.join(mutDep_with_mutdata)],
        'mutDep_mutated_list': [','.join(mutDep_with_mutdata_mutated)],
        'mutDep_not_mutated_list': [','.join(mutDep_with_mutdata_not_mutated)],
        'others_list': [','.join(others)], #Others list
        'others_with_mutdata_list': [','.join(others_with_mutdata)],
        'others_mutated_list': [','.join(others_with_mutdata_mutated)],
        'others_not_mutated_list': [','.join(others_with_mutdata_not_mutated)],
        'mutDep_sensitive_list': [','.join(mutDep_sensitive)], #mutDep sensitive list
        'mutDep_sensitive_with_mutdata_list': [','.join(mutDep_sensitive_with_mutdata)],
        'mutDep_sensitive_mutated_list': [','.join(mutDep_sensitive_with_mutdata_mutated)],
        'mutDep_sensitive_not_mutated_list': [','.join(mutDep_sensitive_with_mutdata_not_mutated)],
        'mutDep_resistant_list': [','.join(mutDep_resistant)], #mutDep resistant list
        'mutDep_resistant_with_mutdata_list': [','.join(mutDep_resistant_with_mutdata)],
        'mutDep_resistant_mutated_list': [','.join(mutDep_resistant_with_mutdata_mutated)],
        'mutDep_resistant_not_mutated_list': [','.join(mutDep_resistant_with_mutdata_not_mutated)],
        'others_sensitive_list': [','.join(others_sensitive)], #others sensitive list
        'others_sensitive_with_mutdata_list': [','.join(others_sensitive_with_mutdata)],
        'others_sensitive_mutated_list': [','.join(others_sensitive_with_mutdata_mutated)],
        'others_sensitive_not_mutated_list': [','.join(others_sensitive_with_mutdata_not_mutated)],
        'others_resistant_list': [','.join(others_resistant)], #others resistant list
        'others_resistant_with_mutdata_list': [','.join(others_resistant_with_mutdata)],
        'others_resistant_mutated_list': [','.join(others_resistant_with_mutdata_mutated)],
        'others_resistant_not_mutated_list': [','.join(others_resistant_with_mutdata_not_mutated)],
        'all_list': [','.join(all_cls)], #All list
        'all_with_mutdata_list': [','.join(all_cls_with_mutdata)],
        'all_mutated_list': [','.join(all_cls_with_mutdata_mutated)],
        'all_not_mutated_list': [','.join(all_cls_with_mutdata_not_mutated)],
    })
    return data_res


def find_entropy(response, partitioned_cls, display=False, significance=False, n_trials=1000, base=2, n_seed=None, excess=0.1):
    """
    The `find_entropy` function calculates the entropy and difference in entropy between two groups of
    responses, and optionally performs significance testing.
    
    :param response: The `response` parameter is a tuple containing two arrays: `sensitive` and
    `resistant`. These arrays represent the response of a system to a certain condition or treatment
    :param partitioned_cls: The `partitioned_cls` parameter is a tuple containing two arrays. The first
    array, `mutDep`, represents a set of classes that are mutation-dependent. The second array,
    `others`, represents a set of classes that are not mutation-dependent. These arrays are used to
    partition the classes based on
    :param display: The `display` parameter is a boolean value that determines whether or not to display
    additional information during the execution of the function. If set to `True`, it will display
    additional information. If set to `False`, it will not display any additional information, defaults
    to False (optional)
    :param significance: The `significance` parameter determines whether or not to perform a
    significance analysis. If set to `True`, the function will estimate a p-value using bootstrap
    resampling to determine the significance of the calculated difference in entropy. If set to `False`,
    no significance analysis will be performed, defaults to False (optional)
    :param n_trials: The parameter `n_trials` specifies the number of trials to perform in the
    significance analysis. It determines how many times the bootstrap process will be repeated to
    estimate the p-value, defaults to 1000 (optional)
    :param base: The base parameter determines the base of the logarithm used in the entropy
    calculation. By default, it is set to 2, which corresponds to calculating entropy in bits. However,
    you can change the base to any positive number to calculate entropy in a different unit, such as
    nats (base e, defaults to 2 (optional)
    :param n_seed: The `n_seed` parameter is used to set the seed for the random number generator when
    generating random samples for the bootstrap analysis. It is an optional parameter that allows you to
    control the randomness of the samples generated. By setting a specific value for `n_seed`, you can
    reproduce the same random samples
    :param excess: The `excess` parameter is a small value added to the entropy calculations to avoid
    division by zero errors. It is typically set to a small positive value, such as 0.1
    :return: a pandas DataFrame object named `data_res`.
    """
    sensitive, resistant = response
    all_responses = np.union1d(sensitive, resistant)
    mutDep, others = partitioned_cls
    all_cls = np.union1d(mutDep, others)
    all_cls_with_response = np.intersect1d(all_cls, all_responses)
    all_cls_without_response = np.setdiff1d(all_cls, all_responses)
    all_cls_with_response_sensitive = np.intersect1d(all_cls, sensitive)
    all_cls_with_response_resistant = np.intersect1d(all_cls, resistant)
    mutDep_with_response = np.intersect1d(mutDep, all_responses)
    others_with_response = np.intersect1d(others, all_responses)
    if len(mutDep_with_response)<3 or len(others_with_response)<3:
        return None
    mutDep_with_response_sensitive = np.intersect1d(mutDep, sensitive)
    mutDep_with_response_resistant = np.intersect1d(mutDep, resistant)
    others_with_response_sensitive = np.intersect1d(others, sensitive)
    others_with_response_resistant = np.intersect1d(others, resistant)
    entropy_mutDep = entropy([len(mutDep_with_response_sensitive)/len(mutDep_with_response), len(mutDep_with_response_resistant)/len(mutDep_with_response)], base=base)
    entropy_others = entropy([len(others_with_response_sensitive)/len(others_with_response), len(others_with_response_resistant)/len(others_with_response)], base=base)
    
    diff_entropy = np.log10((entropy_mutDep+excess) / (entropy_others+excess))
    
    #P value analysis
    if significance:
        np.random.seed(n_seed)

        # Create emp dist to estimate bootstrap p.val
        emp_dist = np.zeros(n_trials)
        for i in range(n_trials):
            pos_cls = np.random.default_rng(seed=n_seed+i).choice(all_cls_with_response, size=len(mutDep_with_response), replace=False)
            neg_cls = np.setdiff1d(all_cls_with_response, pos_cls)

            entropy_pos = entropy([len(np.intersect1d(pos_cls, sensitive))/len(pos_cls), len(np.intersect1d(pos_cls, resistant))/len(pos_cls)], base=base)
            entropy_neg = entropy([len(np.intersect1d(neg_cls, sensitive))/len(neg_cls), len(np.intersect1d(neg_cls, resistant))/len(neg_cls)], base=base)

            random_diff_entropy = np.log10((entropy_pos+excess) / (entropy_neg+excess))
            emp_dist[i] = random_diff_entropy

        # Estimate bootstrap p.value
        if diff_entropy < 0:
            p = np.sum(emp_dist <= diff_entropy) / n_trials
        else:
            p = np.sum(emp_dist >= diff_entropy) / n_trials
    
    data_res = pd.DataFrame({
        'screenGene': ['TBF'],
        'cancer': ['TBF'],
        'candidateDrug': ['TBF'],
        'label': ['TBF'],
        'diffEntropy': [diff_entropy],
        'p': [p],
        'mutDep_entropy': [entropy_mutDep],
        'others_entropy': [entropy_others],
        'mutDep_n': [len(mutDep)],
        'mutDep_with_response_n': [len(mutDep_with_response)],
        'mutDep_sensitive_n': [len(mutDep_with_response_sensitive)],
        'mutDep_resistant_n': [len(mutDep_with_response_resistant)],
        'others_n': [len(others)],
        'others_with_response_n': [len(others_with_response)],
        'others_sensitive_n': [len(others_with_response_sensitive)],
        'others_resistant_n': [len(others_with_response_resistant)],
        'all_with_response_n': [len(all_cls_with_response)],
        'all_sensitive_n': [len(all_cls_with_response_sensitive)],
        'all_resistant_n': [len(all_cls_with_response_resistant)],
        'mutDep_list': [','.join(mutDep)],
        'mutDep_with_response_list': [','.join(mutDep_with_response)],
        'mutDep_sensitive_list': [','.join(mutDep_with_response_sensitive)],
        'mutDep_resistant_list': [','.join(mutDep_with_response_resistant)],
        'others_list': [','.join(others)],
        'others_with_response_list': [','.join(others_with_response)],
        'others_sensitive_list': [','.join(others_with_response_sensitive)],
        'others_resistant_list': [','.join(others_with_response_resistant)],
        'all_with_response_list': [','.join(all_cls_with_response)],
        'all_sensitive_list': [','.join(all_cls_with_response_sensitive)],
        'all_resistant_list': [','.join(all_cls_with_response_resistant)]
    })
    return data_res

def main():
    # Run variants frequency test
    sTest_possibilities = ["test_candidates", "test_screen"]
    if sTest in ['vardrv', 'varall']:
        varType = 'drv' if sTest == 'vardrv' else 'all'
        res = varFreq_test(
            screenGene=f"{screen_gene}+{candidateGene}",
            label=f"{run_label}{candidateGene}{CLsubGroup}",
            partitionedCLs=candidateCLs[CLsubGroup],
            ncells=threshold_ncells,
            varType='all' #vardrv or all
        )

if __name__ == "__main__":
    main()


