import os
import pandas as pd
import numpy as np
from src.lib import config
from src.mapping import gene_mappings


def extract_seq_features_per_gene(gene_list=['KRAS']):
    """
    The function `extract_seq_features_per_gene` extracts sequence features for a list of genes using
    protein embeddings and returns the feature matrix and the list of found genes.
    
    :param gene_list: The `gene_list` parameter is a list of gene symbols. In the given code, the
    default value for `gene_list` is `['KRAS']`, which means that if no value is provided when calling
    the function, it will extract sequence features for the gene symbol 'KRAS'
    :return: The function `extract_seq_features_per_gene` returns a dictionary with two keys: 'features'
    and 'genes'. The value associated with the 'features' key is a numpy array containing the sequence
    features extracted from the genes in the input gene list. The value associated with the 'genes' key
    is a numpy array containing the names of the genes that were found and processed successfully.
    """
    embedding_folder = config.PERSONAL_DATA_DIR / 'protT5'
    gene_embeddings = gene_mappings.get_prottrans_embeddings(embedding_folder, 'per_protein_embeddings_k*.h5')
    symbol2entrez = gene_mappings.get_ncbi_mapping('symbol', 'entrez')
    embedding_list = []
    found_gene_list = []
    for gene in gene_list:
        try:
            embedding_list.append(gene_embeddings[symbol2entrez[gene]])
            found_gene_list.append(gene)
        except Exception as e:
            print(f'{gene} is not found: {e}')
    feature_matrix = np.array(embedding_list)
    found_gene_list = np.array(found_gene_list)
    return {'features': feature_matrix, 'genes': found_gene_list}

def extract_abs_seq_features_between_genes(gene_tuple_list=[('KRAS','KRAS')]):
    pass

def extract_abs_seq_features_between_genes(gene_tuple_list=[('KRAS','KRAS')]):
    pass

def call_feature_extraction(strategy='gene_sequence', sample_list=[]):
    """
    The function `call_feature_extraction` is used to extract sequence features per gene based on a
    given strategy.
    
    :param strategy: The strategy parameter determines the type of feature extraction to be performed.
    In this case, the available strategy is 'gene_sequence', which suggests that the feature extraction
    will be based on gene sequences, defaults to gene_sequence (optional)
    :param sample_list: The sample_list parameter is a list of gene sequences that you want to extract
    features from
    :return: the result of calling the function `extract_seq_features_per_gene` with the argument
    `gene_list` set to the value of `sample_list`.
    """
    if strategy=='gene_sequence':
        return extract_seq_features_per_gene(gene_list = sample_list)

def main():
    """
    The main function extracts sequence features per gene and saves the gene IDs in a numpy array.
    """
    #extract_seq_features_per_gene(gene_list=None)
    from src.process.data_process import process_oncostratifier
    from src.mapping import gene_mappings
    binarized_dependency = process_oncostratifier.get_dependency(binarized=True, processed=True)
    symbol2entrez = gene_mappings.get_ncbi_mapping('symbol', 'entrez')
    geneids = np.array([symbol2entrez[gene_symbol] for gene_symbol in binarized_dependency.index.values])
    np.save('all_dependency_gene_ids.npy', geneids)
    print()
    
if __name__ == "__main__":
    main()