import os
project_path = '.'
path2this = os.path.dirname(os.path.abspath(__file__)).split('/')
for i, folder in enumerate(path2this):
    if folder.lower() == 'oncostratifierml':
        project_path = '/'.join(path2this[:i + 1])

import numpy as np
import pandas as pd
from src.lib import config

BCAPE_FOLDER = config.RAW_DATA_DIR / 'BCaPE'

def get_drug_response(source='Models', score='iC50', return_matrix=True):
    """_summary_
    Args:
        source (str, optional): Models or Samples. Defaults to 'model'.
        score (str, optional): AUC or iC50. Defaults to 'iC50'.
        return_matrix (boolean, optional): Converting the matrix to drugXmodel matrix with "score" values. Defaults to 'True'.

    Returns:
        dataframe: Loads the dataframe
    """
    in_loc = BCAPE_FOLDER / f'DrugResponsesAUC{source}.txt'
    df = pd.read_csv(in_loc, sep='\t')
    if return_matrix:
        return df.pivot(columns= df.columns[0], index= 'Drug', values=score)
    return df

def get_mutation(source='Models', remove_silent=True, return_matrix=True):
    """_summary_
    Args:
        source (str, optional): Models or Samples. Defaults to 'model'.
        score (str, optional): AUC or iC50. Defaults to 'iC50'.
        return_matrix (boolean, optional): Converting the matrix to drugXmodel matrix with "score" values. Defaults to 'True'.

    Returns:
        dataframe: Loads the dataframe
    """
    in_loc = BCAPE_FOLDER / f'SNVs{source}.txt'
    df = pd.read_csv(in_loc, sep='\t')
    if source == 'Models':
        df = (df.set_index('Symbol') !='NO').astype(int)
    elif source == 'Samples':
        df['Mutation'] = 1
        return df.drop_duplicates(subset=['ID', 'Symbol'], keep='first').pivot(columns= 'ID', index= 'Symbol', values='Mutation').fillna(0).astype(int)
    return df
    
def main():
    """
        The main function in this code retrieves drug response data and mutation data, and then performs
        analysis to identify matching genes and corresponding drug responses.
        
        :param source: The `source` parameter determines whether to load data from the "Models" or "Samples"
        source, defaults to Models (optional)
        :param score: The `score` parameter in the `get_drug_response` and `get_mutation` functions
        determines which score to use when loading the data. It can be set to either 'AUC' or 'iC50',
        defaults to iC50 (optional)
        :param return_matrix: The `return_matrix` parameter determines whether the function should return a
        matrix or a dataframe. If `return_matrix` is set to `True`, the function will convert the dataframe
        into a drugXmodel matrix with "score" values. If `return_matrix` is set to `False`, the function,
        defaults to True (optional)
        :return: The main function in the code is being executed, which performs several operations
        including reading data from files, filtering and manipulating dataframes, and printing the results.
        The code does not explicitly return any values.
    """
    screen_gene, marker_gene = 'BRD4', 'PIK3R1'
    df_response_ic50 = get_drug_response(source='Samples', score='iC50', return_matrix=True)
    df_response_auc = get_drug_response(source='Samples', score='AUC', return_matrix=True)
    df_mut = get_mutation(source='Samples', remove_silent=True, return_matrix=True)
    match_list = []
    for firstgene in [agene1 for agene1 in df_mut.index[df_mut.index.notnull()] if screen_gene in agene1]:
        for secondgene in [agene2 for agene2 in df_mut.index[df_mut.index.notnull()] if marker_gene in agene2]:
            match_cols = df_mut.columns[(df_mut.loc[firstgene] & df_mut.loc[secondgene])==1].values
            if len(match_cols)>0:
                print(f'{firstgene}, {secondgene}: {match_cols}')
                match_list += list(match_cols)
    unique_pdxs = np.unique(match_list)
    print(unique_pdxs)
    for aimpdx in unique_pdxs:
        aimpdx = aimpdx.split('-')[0]
        match_pdxs = [apdx for apdx in df_response_ic50.columns if aimpdx in apdx]
        for match_pdx in match_pdxs:
            top5drugs_auc = df_response_auc[match_pdx].sort_values(ascending=False).iloc[:5].index
            print(f'AUC-{aimpdx}-{match_pdx}\n{pd.concat((df_response_auc.loc[top5drugs_auc, match_pdx], df_response_ic50.loc[top5drugs_auc, match_pdx]), axis=1)}')
            print(f'AUC-{aimpdx}-{match_pdx}\n{df_response_auc.drop(columns=[match_pdx]).loc[top5drugs_auc].mean(axis=1)}')
            #print()
            #top5drugs_ic50 = df_response_ic50[match_pdx].sort_values(ascending=True).iloc[:5].index
            #print(f'IC50-{aimpdx}-{match_pdx}\n{pd.concat((df_response_ic50.loc[top5drugs_ic50, match_pdx], df_response_auc.loc[top5drugs_ic50, match_pdx]), axis=1)}')
            #print(f'IC50-{aimpdx}-{match_pdx}\n{df_response_ic50.drop(columns=[match_pdx]).loc[top5drugs_ic50].mean(axis=1)}')
            print('\n------------------------------------------------------\n')
        
    
if __name__ == "__main__":
    main()