import os
project_path = '.'
path2this = os.path.dirname(os.path.abspath(__file__)).split('/')
for i, folder in enumerate(path2this):
    if folder.lower() == 'oncostratifierml':
        project_path = '/'.join(path2this[:i + 1])

import numpy as np
import pandas as pd
from src.lib import config
from src.mapping import ccle_mappings

def process_ccles(col_from, col_to, from_list=None, to_list=None):
    """
    The function `process_ccles` takes in two column names and optional lists of values, and returns a
    dictionary mapping values from the first column to values from the second column in a DataFrame.
    
    :param col_from: The parameter `col_from` is the name of the column from which you want to convert
    values. It can be either 'TCGA' or 'ModelID'
    :param col_to: The `col_to` parameter is the name of the column in the DataFrame that you want to
    convert the values to
    :param from_list: The `from_list` parameter is a list of values that you want to filter the
    `col_from` column by. It will only include rows in the DataFrame where the value in the `col_from`
    column is in the `from_list`
    :param to_list: The `to_list` parameter is a list of values that you want to filter the `col_to`
    column by. It is used to select specific values from the `col_to` column in the DataFrame
    :return: The function `process_ccles` returns different values based on the conditions specified in
    the code.
    """
    in_loc = config.DATA_DIR / 'depmap_23Q2/Model.csv'
    df = pd.read_csv(in_loc, sep=',')
    if col_from.lower() == 'tcga' or col_to.lower() == 'tcga':
        modelid2tcga, tcga2modelid = ccle_mappings.map_tcga_oncotreeprimary(df)
        df['TCGA'] = df['ModelID'].map(modelid2tcga)
    if (col_from.lower() == 'tcga' and col_to =='ModelID'):
        return tcga2modelid
    elif (col_to.lower() == 'tcga' and col_from =='ModelID'):
        return modelid2tcga
    if col_from.lower() == 'tcga':
        df['TCGA'] = df['ModelID'].map(modelid2tcga)
    if from_list is not None:
        df[df[col_from].isin(from_list)]
    if to_list is not None:
        df[df[col_to].isin(to_list)]
    if col_from == 'ModelID':
        return df.set_index(col_from)[col_to].to_dict()
    elif col_from == 'TCGA':
        return {unique_group: df[df[col_from]==unique_group][col_to].values for unique_group in df[col_from].unique()}

def process_dependency(genes=None, ccles=None, gene_identifier=None, binarize=False):
    """
    The function `process_dependency` reads a CSV file containing gene dependency data, renames the
    columns based on a specified gene identifier, and filters the data based on specified genes and cell
    lines.
    
    :param genes: The "genes" parameter is a list of gene names or gene identifiers. It specifies which
    genes' dependency data should be included in the output dataframe. If this parameter is not provided
    or set to None, all genes will be included in the output dataframe
    :param ccles: ccles is a list of cell line identifiers. It is used to filter the dataframe based on
    specific cell lines
    :param gene_identifier: The `gene_identifier` parameter is used to specify how the gene identifiers
    in the dataframe should be interpreted. It can take one of three values:
    :param binarize: The `binarize` parameter is a boolean flag that determines whether the gene
    dependency values should be binarized or not. If `binarize` is set to `True`, the gene dependency
    values will be converted to binary values (0 or 1) based on a threshold. If, defaults to False
    (optional)
    :return: a pandas DataFrame object.
    """
    in_loc = config.DATA_DIR / 'depmap_23Q2/CRISPRGeneDependency.csv'
    df = pd.read_csv(in_loc, sep=',').set_index('ModelID')
    if gene_identifier=='id':
        col_rename = {col: col.split(' (')[-1][:-1] for col in df.columns}
    elif gene_identifier=='name':
        col_rename = {col: col.split(' (')[0] for col in df.columns}
    elif '2' in gene_identifier:
        from_, to_ = gene_identifier.split('2')
        gene_identifier_list = {'id': [col.split(' (')[-1][:-1] for col in df.columns], 'name': [col.split(' (')[0] for col in df.columns]}
        return dict(zip(gene_identifier_list[from_], gene_identifier_list[to_]))
    df = df.rename(columns=col_rename, inplace=False)
    if genes:
        df = df[genes]
    if ccles:
        df = df.loc[ccles]
    
    return df 

def process_mutation(filtering={}, silent_included=False):
    """
    The function `process_mutation` reads a CSV file containing somatic mutation data, filters the data
    based on specified criteria, and returns the filtered dataframe.
    
    :param filtering: The `filtering` parameter is a dictionary where the keys are column names in the
    dataframe `df` and the values are lists of values to filter on for each column. This allows you to
    specify specific values to include or exclude from the dataframe based on the values in those
    columns
    :param silent_included: The `silent_included` parameter is a boolean flag that determines whether or
    not to include silent mutations in the resulting dataframe. If `silent_included` is set to `True`,
    then the dataframe will include all mutations, including silent ones. If `silent_included` is set to
    `False, defaults to False (optional)
    :return: a pandas DataFrame object.
    """
    in_loc = config.DATA_DIR / 'depmap_23Q2/OmicsSomaticMutations.csv'
    df = pd.read_csv(in_loc, sep=',')
    if not silent_included:
        df = df[df['VariantInfo'] != 'SILENT']
    for col, filterlist in filtering.items():
        df = df[df[col].isin(filterlist)]
    return df

def process_expression():
    return 0

def process_cnv():
    return 0


def define_model_set(gene, setting_dict):
    """
    The function `define_model_set` takes a gene and a dictionary of settings as input, and returns a
    dictionary of model IDs based on the given gene and settings.
    
    :param gene: The gene parameter is a string that represents the name of a gene
    :param setting_dict: The `setting_dict` parameter is a dictionary that contains different types of
    omics data and their corresponding settings. Each key-value pair in the dictionary represents an
    omic type and its settings. The key is the name of the omic type (e.g., "mutation", "dependency"),
    and the
    :return: a dictionary called `models_dict`.
    """
    models_dict = {}
    for omic_name, omic in setting_dict.items():
        if len(omic)>1:
            arg = omic[1]
        omic=omic[0]
        if omic_name=='mutation':
            model_list = omic[omic['HugoSymbol'] == gene]['ModelID'].values
        elif omic_name=='dependency':
            model_list = omic[omic[gene]>=arg].index.values
        models_dict[omic_name] = model_list
    return models_dict

def find_more_dependency():
    """
    The function `find_more_dependency()` retrieves the model IDs for a list of cell lines from a
    mapping dictionary.
    :return: a list of model IDs that correspond to the cell lines in the `needed_list` variable.
    """
    stripped2modelid = ccle_mappings.depmap2any(aim='StrippedCellLineName')
    needed_list = ['CL-14', 'CL-34', 'CL-40', 'COLO-320-HSR', 'CW-2', 'Geo',
       'Hs-255-T', 'Hs-675-T', 'LS-123', 'NCI-H508', 'NCI-H684',
       'SK-CO-1', 'SNU-1040', 'SNU-1197', 'SNU-175', 'SNU-283', 'SNU-407',
       'SNU-C2A', 'SNU-C2B', 'SW1116', 'SW1417', 'SW948']
    needed_modelids = []
    for a_cl in needed_list:
        found = stripped2modelid.get(a_cl.strip().replace("-", "").upper())
        print(f'{a_cl} ({a_cl.strip().replace("-", "").upper()}): {found}')
        if found is not None:
            needed_modelids.append(found)
    return needed_modelids

def main():
    """
    The main function processes gene dependencies and finds additional model IDs.
    """
    genes = ['KRAS']
    #ccle_dict = process_ccles('TCGA', 'ModelID')
    #gene_name2id = process_dependency(gene_identifier='name2id')
    #mut_df = process_mutation(filtering = {'HugoSymbol': genes, 'ModelID': ccle_dict['COAD']})
    needed_modelids  = find_more_dependency()
    dep_df = process_dependency(gene_identifier='name')
    #models_settings = {'mutation': [mut_df], 'dependency': [dep_df, 0.9]}
    #models_dict = define_model_set(genes[0], models_settings)
    print()
    
if __name__ == "__main__":
    main()