import os,sys
project_path = '.'
path2this = os.path.dirname(os.path.abspath(__file__)).split('/')
for i, folder in enumerate(path2this):
    if folder.lower() == 'oncostratifierml':
        project_path = '/'.join(path2this[:i + 1])
sys.path.insert(0, project_path)
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from pandas.plotting import table
from src.lib import config
from src.mapping import ccle_mappings

COSMIC_FOLDER = config.RAW_DATA_DIR / 'COSMIC'

def get_cancer_gene_census(tiers=[1], roles=['oncogene'], somatic=True, mutation_types=[]):
    """
    The function `get_cancer_gene_census` retrieves cancer genes from a dataset based on specified
    criteria such as tiers, roles, somatic status, and mutation types.
    
    :param tiers: The `tiers` parameter is a list of integers representing the tiers of cancer genes.
    The function will filter the genes based on these tiers. For example, if `tiers=[1, 2]`, the
    function will only return genes that are in tier 1 or tier 2
    :param roles: The 'roles' parameter is a list of roles that the genes can have in cancer. The
    default value is ['oncogene'], which means that the function will return genes that are classified
    as oncogenes in the cancer gene census. However, you can provide a different list of roles to get
    :param somatic: The "somatic" parameter is a boolean value that determines whether the genes
    returned should be somatic or not. If somatic is set to True, only somatic genes will be returned.
    If somatic is set to False, both somatic and non-somatic genes will be returned, defaults to True
    (optional)
    :param mutation_types: The `mutation_types` parameter is a list that specifies the types of
    mutations you are interested in. It filters the results based on the specified mutation types
    :return: a pandas DataFrame containing information about cancer genes from the COSMIC database. The
    returned DataFrame will have filters applied based on the input parameters: tiers, roles, somatic,
    and mutation_types.
    """
    file_loc = COSMIC_FOLDER / 'Census_allFri Nov 10 10_56_25 2023.tsv'
    df = pd.read_csv(file_loc, sep='\t')
    if len(tiers)>0:
        df = df[df['Tier'].isin(tiers)]
    for role in roles:
        df = df[df['Role in Cancer'].str.contains(role)]
    for mut_type in mutation_types:
        df = df[df['Mutation Types'].str.contains(mut_type)]
    return df

def report_genes(df, mutation_type='Tumour Types(Somatic)', synonyms=False, as_png=False):
    """
    The `report_genes` function generates a report of gene mutations based on a given dataframe,
    mutation type, and optional parameters for including synonyms and saving the report as a PNG file.
    
    :param df: The `df` parameter is a pandas DataFrame that contains the data you want to report on
    :param mutation_type: The `mutation_type` parameter specifies the column in the dataframe that
    contains the mutation type information. By default, it is set to 'Tumour Types(Somatic)', defaults
    to Tumour Types(Somatic) (optional)
    :param synonyms: The `synonyms` parameter is a boolean value that determines whether or not to
    include the column for gene synonyms in the output. If `synonyms` is set to `True`, the column will
    be included; if `synonyms` is set to `False`, the column will be excluded, defaults to False
    (optional)
    :param as_png: The `as_png` parameter determines whether the table should be saved as a PNG image
    file. If `as_png` is set to `True`, the table will be saved as a PNG image file named "table.png".
    If `as_png` is set to `False`, the table will be, defaults to False (optional)
    """
    accepted_columns = ['Gene Symbol', mutation_type]
    if synonyms:
        accepted_columns.append('Synonyms')
    df = df[accepted_columns]
    print(df.to_markdown()) 
    if as_png:
        fig, ax = plt.subplots(figsize=(10, len(df)*0.17)) # set size frame
        ax.xaxis.set_visible(False)  # hide the x axis
        ax.yaxis.set_visible(False)  # hide the y axis
        ax.set_frame_on(False)  # no visible frame, uncomment if size is ok
        col_widths = [0.1, 0.9]
        tabla = ax.table(cellText=df.values, colLabels=df.keys(), loc='upper left', colWidths=col_widths, cellLoc='left')  # where df is your data frame
        tabla.auto_set_font_size(False) # Activate set fontsize manually
        tabla.set_fontsize(9) # if ++fontsize is necessary ++colWidths
        #tabla.scale(1.2, 1.2) # change size table
        plt.savefig('table.png', transparent=True, dpi=300, bbox_inches='tight')
        
def main():
    df = get_cancer_gene_census(tiers=[1], roles=['oncogene'], somatic=True, mutation_types=[])
    report_genes(df, mutation_type='Tumour Types(Somatic)', as_png=True)
    
if __name__ == "__main__":
    main()