import os,sys
project_path = '.'
path2this = os.path.dirname(os.path.abspath(__file__)).split('/')
for i, folder in enumerate(path2this):
    if folder.lower() == 'oncostratifierml':
        project_path = '/'.join(path2this[:i + 1])
sys.path.insert(0, project_path)
import numpy as np
import pandas as pd
from src.lib import config

CRx_FOLDER = config.RAW_DATA_DIR / 'CRx'

def get_drug(gdsc_version=1, drug=None):
    """
    The function `get_drug` reads IC50 data from a file, calculates additional columns, and returns
    either the entire dataset or a subset based on a specified drug.
    
    :param gdsc_version: The `gdsc_version` parameter specifies the version of the GDSC dataset to use.
    The default value is 1, defaults to 1 (optional)
    :param drug: The `drug` parameter is used to filter the data based on a specific drug name. If a
    drug name is provided, the function will return a subset of the data containing only the rows where
    the drug name matches the provided value. If no drug name is provided, the function will return the
    entire
    :return: If the `drug` parameter is provided, the function will return a subset of the `ic50`
    dataframe that contains only the rows where the `DRUG_NAME` column matches the provided `drug`
    value. If the `drug` parameter is not provided, the function will return the entire `ic50`
    dataframe.
    """
    ic50_loc = CRx_FOLDER / f'GDSC{gdsc_version}ic50' / f'GDSC{gdsc_version}_fitted_dose_response_27Oct23.xlsx'
    ic50 = pd.read_excel(ic50_loc) 
    ic50['IC50'] = np.exp(ic50['LN_IC50'])
    ic50['LN_MAX_CONC'] = np.log(ic50['MAX_CONC'])
    ic50['SENSITIVE'] = (ic50['IC50'] < ic50['MAX_CONC']).replace({True: 1, False: 0})
    ic50['LN_SENSITIVE'] = (ic50['LN_IC50'] < ic50['LN_MAX_CONC']).replace({True: 1, False: 0})
    if drug is not None:
        if drug.lower() == 'max':
            drugs, drug_counts = np.unique(ic50['DRUG_NAME'].values, return_counts=True)
            drugs, drug_counts = drugs[np.argsort(-drug_counts)], drug_counts[np.argsort(-drug_counts)]   
            drug = drugs[0]
        ic50_adrug = ic50[ic50['DRUG_NAME']==drug]
        #ic50_adrug[['LN_IC50', 'AUC', 'Z_SCORE']].hist()
        return ic50_adrug
    return ic50


def clear_combine_sources(list_of_dfs = [], verbose=0):
    """
    The function `clear_combine_sources` combines a list of dataframes, performs some data analysis, and
    returns the combined dataframe.
    
    :param list_of_dfs: The `list_of_dfs` parameter is a list of pandas DataFrames that you want to
    combine into a single DataFrame. Each DataFrame in the list represents a source of data
    :param verbose: The `verbose` parameter is used to control the level of detail in the output. If
    `verbose` is set to 0, no additional information will be printed. If `verbose` is set to 1,
    additional information about the unique values and duplicates in the dataframes will be printed,
    defaults to 0 (optional)
    :return: a concatenated dataframe `gdsc` with an additional column `UNIQUE_DRUG`.
    """
    if verbose:
        for df in list_of_dfs:
            unique_celllines = len(df['CELL_LINE_NAME'].unique())
            unique_drug_ids = len(df['DRUG_ID'].unique())
            unique_drug_names = len(df['DRUG_NAME'].unique())
            grouped_id = df.groupby(['CELL_LINE_NAME', 'DRUG_ID']).agg(
                SENSITIVITY_AGREEMENT=pd.NamedAgg(column="LN_SENSITIVE", aggfunc=lambda x: len(set(x)) == 1),
                COUNT=('LN_SENSITIVE','count')).reset_index()
            grouped_name = df.groupby(['CELL_LINE_NAME', 'DRUG_NAME']).agg(
                SENSITIVITY_AGREEMENT=pd.NamedAgg(column="LN_SENSITIVE", aggfunc=lambda x: len(set(x)) == 1),
                COUNT=('LN_SENSITIVE','count')).reset_index()
            unique_cellline_drugid_combs = grouped_id.shape[0]
            unique_cellline_drugname_combs = grouped_name.shape[0]
            grouped_name_dups = grouped_name[grouped_name['COUNT']>1]
            grouped_name_agree = grouped_name_dups[grouped_name_dups['SENSITIVITY_AGREEMENT']==1]
            grouped_name_disagree = grouped_name_dups[grouped_name_dups['SENSITIVITY_AGREEMENT']==0]
            
            grouped_id_dups = grouped_id[grouped_id['COUNT']>1]
            grouped_id_agree = grouped_id_dups[grouped_id_dups['SENSITIVITY_AGREEMENT']==1]
            grouped_id_disagree = grouped_id_dups[grouped_id_dups['SENSITIVITY_AGREEMENT']==0]
            
            print(f'Uniques-- CellLines: {unique_celllines},\tDrugNames: {unique_drug_names},\tDrugIds: {unique_drug_ids},\tCellLineDrugName: {unique_cellline_drugname_combs},\tCellLineDrugId: {unique_cellline_drugid_combs}')
            print(f'Duplicates (Name)-- CellLineDrug_Total: {grouped_name_dups.shape[0]}, \tCellLineDrug_Agree: {grouped_name_agree.shape[0]}, \tCellLineDrug_Disagree: {grouped_name_disagree.shape[0]}')
            print(f'Duplicates (Id)-- CellLineDrug_Total: {grouped_id_dups.shape[0]}, \tCellLineDrug_Agree: {grouped_id_agree.shape[0]}, \tCellLineDrug_Disagree: {grouped_id_disagree.shape[0]}')
    if len(list_of_dfs)>1:
        gdsc = pd.concat(list_of_dfs)
    elif len(list_of_dfs)==1:
        gdsc = list_of_dfs[0]
    gdsc['UNIQUE_DRUG'] = gdsc['DRUG_ID'].astype('str')+'_'+gdsc['DRUG_NAME']+'_'+gdsc['DATASET']
    return gdsc

def extract_drug_matrix(gdsc_version=1, drug=None):
    """
    The function `extract_drug_matrix` extracts drug data from the GDSC dataset, including IC50 values,
    maximum concentration, sensitivity, and other related information.
    
    :param gdsc_version: The `gdsc_version` parameter specifies the version of the GDSC dataset to use.
    It is set to 1 by default, defaults to 1 (optional)
    :param drug: The "drug" parameter is used to specify a particular drug for which you want to extract
    the drug matrix. If a specific drug is provided, the function will filter the drug matrix to include
    only data related to that drug. If no drug is provided, the function will return the entire drug
    matrix
    :return: The function `extract_drug_matrix` returns a pandas DataFrame object. If the `drug`
    parameter is provided, it returns a subset of the DataFrame containing data for the specified drug.
    If the `drug` parameter is not provided, it returns the entire DataFrame.
    """
    ic50_loc = CRx_FOLDER / f'GDSC{gdsc_version}ic50' / f'GDSC{gdsc_version}_fitted_dose_response_25Feb20.xlsx'
    ic50 = pd.read_excel(ic50_loc) 
    ic50['VERSION'] = gdsc_version
    ic50['IC50'] = np.exp(ic50['LN_IC50'])
    ic50['LN_MAX_CONC'] = np.log(ic50['MAX_CONC'])
    ic50['SENSITIVE'] = (ic50['IC50'] < ic50['MAX_CONC']).replace({True: 1, False: 0})
    ic50['LN_SENSITIVE'] = (ic50['LN_IC50'] < ic50['LN_MAX_CONC']).replace({True: 1, False: 0})
    if drug is not None:
        if drug.lower() == 'max':
            drugs, drug_counts = np.unique(ic50['DRUG_NAME'].values, return_counts=True)
            drugs, drug_counts = drugs[np.argsort(-drug_counts)], drug_counts[np.argsort(-drug_counts)]   
            drug = drugs[0]
        ic50_adrug = ic50[ic50['DRUG_NAME']==drug]
        #ic50_adrug[['LN_IC50', 'AUC', 'Z_SCORE']].hist()
        return ic50_adrug
    return ic50

def visualize_distribution(df, drug='Cisplatin'):
    """
    The function `visualize_distribution` visualizes the distribution of different scores for a given
    drug or for all drugs in a dataframe.
    
    :param df: The `df` parameter is a pandas DataFrame that contains the data you want to visualize. It
    should have columns named 'UNIQUE_DRUG', 'LN_IC50', 'IC50', 'AUC', 'Z_SCORE', and 'DRUG_NAME'
    :param drug: The `drug` parameter is a string that specifies the name of a drug. By default, it is
    set to 'Cisplatin', defaults to Cisplatin (optional)
    """
    import matplotlib.pyplot as plt
    import seaborn as sns
    if drug is None:
        df = df.set_index('UNIQUE_DRUG')
        for drug in np.unique(df.index):
            fig, axes = plt.subplots(2,2, layout='constrained')
            fig.suptitle(f'{drug}')
            for ax, score in zip(axes.flatten(),['LN_IC50', 'IC50', 'AUC', 'Z_SCORE']):
                ax.hist(df.loc[drug, score].values, bins=30)   
                ax.set_xlabel(score)
            out_loc = config.FHT_RESULTS_DIR / 'images' / 'drug' / f'{drug}.png'
            config.ensure_dir(out_loc)
            plt.savefig(out_loc, dpi=100, bbox_inches='tight')
            plt.clf()
            plt.close('all')
    else:
        if drug.lower() == 'max':
            drugs, drug_counts = np.unique(df['DRUG_NAME'].values, return_counts=True)
            drugs, drug_counts = drugs[np.argsort(-drug_counts)], drug_counts[np.argsort(-drug_counts)]   
            drug = drugs[0]
        df_adrug = df[df['DRUG_NAME']==drug]
        df_adrug[['LN_IC50', 'IC50', 'AUC', 'Z_SCORE']].hist(layout=(2,2), figsize=(10,10))
        out_loc = config.FHT_RESULTS_DIR / 'images' / 'drug' / f'{drug}.png'
        config.ensure_dir(out_loc)
        plt.savefig(out_loc, dpi=300, bbox_inches='tight')


def get_gdsc_targets(gdsc):
    curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
    gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
    gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
    gdsc_nested_targets = collections.defaultdict(dict)
    for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
    return gdsc_targets, gdsc_nested_targets

def get_compound_annotation(dict_key = 'DRUGID_DRUGNAME'):
    annotation_loc = CRx_FOLDER / 'screened_compounds_rel_8.5.csv'
    annot = pd.read_csv(annotation_loc, index_col=False)
    annot[['TARGET', 'TARGET_PATHWAY']] = annot[['TARGET', 'TARGET_PATHWAY']].fillna('')
    annot['DRUGID_DRUGNAME'] = annot['DRUG_ID'].astype('str') + '_' + annot['DRUG_NAME']
    if dict_key is not None:
        drug2gene = annot.set_index(dict_key)['TARGET'].to_dict()
        drug2pw = annot.set_index(dict_key)['TARGET_PATHWAY'].to_dict()
        return drug2gene, drug2pw
    else:
        return annot

def main():
    """
    The main function loads or creates a cache of GDSC data, combines two versions of the data, and then
    visualizes the distribution of the data.
    """
    gdsc_cache_name = f'combined_gdsc_27Oct23.xz'
    gdsc_cache_dict = config.cache_numpy(cache_name=gdsc_cache_name, is_pickle=True)
    if gdsc_cache_dict is None:
        print(f'GDSC load started!')
        gdsc1 = get_drug(gdsc_version=1)
        gdsc2 = get_drug(gdsc_version=2)
        gdsc_cache_dict = {'gdsc': clear_combine_sources([gdsc1, gdsc2])}
        config.cache_numpy(cache_name=gdsc_cache_name, data=gdsc_cache_dict, is_pickle=True)
    gdsc = gdsc_cache_dict['gdsc']
    visualize_distribution(gdsc, drug=None)
    print()
    
if __name__ == "__main__":
    main()