import os
project_path = '.'
path2this = os.path.dirname(os.path.abspath(__file__)).split('/')
for i, folder in enumerate(path2this):
    if folder.lower() == 'oncostratifierml':
        project_path = '/'.join(path2this[:i + 1])

import numpy as np
import pandas as pd
from src.lib import config
from src.mapping import ccle_mappings
from src.process.results_process import previous_results
import pyreadr

DEPMAP_FOLDER = config.RAW_DATA_DIR / 'DepMap'
CMP_FOLDER = config.RAW_DATA_DIR / 'CMP'
CORRECTED_ESS22Q2_FOLDER = DEPMAP_FOLDER / 'essentiality20Q2'
CORRECTED_ESS22Q2_RAW_FOLDER = CORRECTED_ESS22Q2_FOLDER / 'Project_score_combined_Sanger_v1_Broad_20Q2_20210311'

def get_dependency(binarized=True, processed=True):
    """
    The function `get_dependency` returns a dataframe based on the specified parameters for binarized
    and processed data.
    
    :param binarized: A boolean parameter indicating whether the data should be binarized or not. If
    True, the binarized data will be returned. If False, the scaled data will be returned, defaults to
    True (optional)
    :param processed: The "processed" parameter indicates whether the data is already processed or not.
    If set to True, it means that the data has already been processed and the function will return the
    processed data. If set to False, it means that the data is in its raw form and the function will
    return the raw, defaults to True (optional)
    :return: a DataFrame object.
    """
    if binarized:
        if processed:
            binary_processed_loc = CORRECTED_ESS22Q2_FOLDER / 'Sanger_Broad_higQ_bdep.RData'
            df = pyreadr.read_r(binary_processed_loc)['bdep']
        else:
            binary_fitness_loc = CORRECTED_ESS22Q2_RAW_FOLDER / 'Project_score_combined_Sanger_v1_Broad_20Q2_fitness_scores_binary_matrix_20210311.tsv'
            df = pd.read_csv(binary_fitness_loc, sep='\t')
    else:
        if processed:
            scaled_processed_loc = CORRECTED_ESS22Q2_FOLDER / 'Sanger_Broad_higQ_scaled_depFC.RData'
            df = pyreadr.read_r(scaled_processed_loc)['scaled_depFC']
        else:
            scaled_fitness_loc = CORRECTED_ESS22Q2_RAW_FOLDER / 'Project_score_combined_Sanger_v1_Broad_20Q2_fitness_scores_scaled_bayesian_factors_20210311.tsv'
            df = pd.read_csv(scaled_fitness_loc, sep='\t')
    return df
    

def get_expression(source='cmp'):
    """
    The function `get_expression` reads and returns a dataframe containing expression data based on the
    specified source.
    
    :param source: The source parameter is a string that specifies the source of the expression data. It
    can be either 'cmp' or 'depmap', defaults to cmp (optional)
    :return: a pandas DataFrame object containing expression data.
    """
    if 'cmp' in source.lower():
        expression_loc = CMP_FOLDER / 'expression' / 'rnaseq_latest.csv.gz'
        exp_df = pd.read_csv(expression_loc)
    elif 'depmap' in source.lower():
        expression_loc = DEPMAP_FOLDER / 'CCLE_expression.csv'
        exp_df = pd.read_csv(expression_loc)
    else:
        raise SystemExit(f'Source for expression data is not recognized! source: {source}')
    return exp_df


def get_mutation(source='cmp', version='2022'):
    """
    The function `get_mutation` retrieves mutation data from a specified source and version.
    
    :param source: The source parameter is used to specify the source of the mutation data. It can be
    either 'cmp' or 'depmap', defaults to cmp (optional)
    :param version: The `version` parameter specifies the version of the mutation data to retrieve. The
    available options are '2022', '2019', and '2018', defaults to 2022 (optional)
    :return: a DataFrame containing mutation data.
    """
    variant_loc_dict = {'2022': 'mutations_all_20220510.csv', 
                        '2019': 'mutations_20191101.csv', 
                        '2018': 'mutations_2018-08-01_1640.csv'}
    if 'cmp' in source.lower():
        mutation_loc = CMP_FOLDER / 'variants' / variant_loc_dict[version]
    elif 'depmap' in source.lower():
        mutation_loc = DEPMAP_FOLDER / 'CCLE_mutations.csv'
    else:
        raise SystemExit(f'Source for expression data is not recognized! source: {source}')
    cache_name = f'oncostratifier_raw/mutation_{source}_{version}.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is None:
        cache_dict = {'data': pd.read_csv(mutation_loc)}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)
    return cache_dict['data']

def depmatrix2genedict(bdep):
    """
    The function `depmatrix2genedict` takes a binary dependency matrix as input and returns a dictionary
    where the keys are the row indices and the values are lists of column indices where the value is 1.
    
    :param bdep: bdep is a pandas DataFrame that represents a binary dependency matrix. Each row in the
    DataFrame represents a gene, and each column represents a dependency relationship with another gene.
    The values in the DataFrame are binary, where 1 indicates a dependency and 0 indicates no dependency
    :return: a dictionary where the keys are the row indices of the input DataFrame `bdep`, and the
    values are lists of column indices where the corresponding row value is 1.
    """
    result_dict = {}
    # Iterate over rows
    for row_idx, row in bdep.iterrows():
        row_values = [col_idx for col_idx, value in row.items() if value == 1]
        result_dict[row_idx] = row_values
    return result_dict

def get_annotation(source='cmp|depmap'):
    """
    The function `get_annotation` merges data from two different data sources, CMP and DepMap, based on
    predefined fields, and returns a DataFrame with the merged data.
    
    :param source: The `source` parameter is a string that specifies the source of the annotation data.
    The default value is `'cmp|depmap'`, which indicates that the annotation data should be obtained
    from both the CMP and DepMap sources, defaults to cmp|depmap (optional)
    :return: a pandas DataFrame containing the final annotation data.
    """
    cmp_loc = CMP_FOLDER / 'models/model_list_latest.csv'
    cmp_fixed_loc = CMP_FOLDER / 'models/model_name_fixes.txt'
    depmap_loc = DEPMAP_FOLDER / 'models/sample_info.csv'
    cmp_df = pd.read_csv(cmp_loc)
    cmp_fix_df = pd.read_csv(cmp_fixed_loc, sep='\t')
    depmap_df = pd.read_csv(depmap_loc)
    depmap_df.rename(columns={'RRID': 'RRID_DepMap', 'patient_id': 'patient_id_DepMap'}, inplace=True)
    cmp_fix_df.rename(columns={'Source': 'model_name_fix', 'CMP': 'model_name'}, inplace=True)

    # Merge DataFrames
    cmp_df = pd.merge(cmp_fix_df, cmp_df, how='right')

    # Update specific columns
    cmp_df.loc[cmp_df['model_name_fix'].isna(), 'model_name_fix'] = cmp_df.loc[cmp_df['model_name_fix'].isna(), 'model_name']

    # Assigning values from one column to another in a DataFrame
    cmp_df['stripped_model_name_1'] = cmp_df['model_name']
    cmp_df['stripped_model_name_2'] = cmp_df['model_name_fix']
    
    # Add normalized cell name fields
    columns_to_strip = ['stripped_model_name_1', 'stripped_model_name_2']
    for sFieldStripped in columns_to_strip:
        cmp_df[sFieldStripped] = cmp_df[sFieldStripped].str.replace('-', '').str.replace('_', '') \
            .str.replace('.', '').str.replace('/', '').str.replace(':', '') \
            .str.replace('(', '').str.replace(')', '') \
            .str.replace('[', '').str.replace(']', '')

    # Create final annotation DataFrame
    df = pd.DataFrame()

    # Set list of fields to merge data between CMP and DepMap
    lFieldsMerge = {
        'model_name': 'cell_line_name',
        'model_name_fix': 'cell_line_name',
        'stripped_model_name_1': 'stripped_cell_line_name',
        'stripped_model_name_1': 'stripped_cell_line_name',
        'model_id': 'Sanger_Model_ID',
        'BROAD_ID': 'DepMap_ID',
        'RRID': 'RRID_DepMap',
        'CCLE_ID': 'CCLE_Name'
    }

    # Merge data between CMP and DepMap using the predefined fields
    for sFieldMergeCMP, sFieldMergeDepMap in lFieldsMerge.items():
        df = pd.concat([
            df,
            pd.merge(
                cmp_df[~cmp_df[sFieldMergeCMP].isna()],
                depmap_df[~depmap_df[sFieldMergeDepMap].isna()],
                left_on=sFieldMergeCMP, right_on=sFieldMergeDepMap
            )
        ])
    df = df.loc[~df.duplicated(), :]

    # Remove cell lines with misassigned RRID
    df = df[
        (~df['model_id'].isin(df['model_id'].value_counts()[df['model_id'].value_counts() > 1].index)) |
        (df['RRID'].isna()) |
        (df['RRID_DepMap'].isna()) |
        (df['RRID'] == df['RRID_DepMap'])
    ]

    # Remove duplicated cell line
    df = df[
        ~((df['model_name'] == 'CHL-1') & (df['cell_line_name'] != 'CHL-1'))
    ]

    # Add CMP cell lines that are not in DepMap
    df = pd.concat([
        df,
        cmp_df[~cmp_df['model_id'].isin(df['model_id'])]
    ])

    # Remove cell lines with issues
    df.loc[df['Cellosaurus_issues'].isna(), 'Cellosaurus_issues'] = ''
    df = df[df['Cellosaurus_issues'] == '']

    # Remove cell lines with duplicated names
    df = df[
        df['model_name'].isin(df['model_name'].value_counts()[df['model_name'].value_counts() == 1].index)
    ]
    df.set_index('model_name', inplace=True)
    return df

def extract_essentiality_mutations_cell_lines(bin_dep, mut, cancers=[], genes=[]):
    """
    The function `extract_essentiality_mutations_cell_lines` extracts essentiality mutations in cell
    lines based on the provided parameters.
    
    :param bin_dep: The variable "bin_dep" is not defined in the code snippet you provided. Could you
    please provide the definition or explanation of what "bin_dep" represents?
    :param mut: The "mut" parameter is a variable that represents a mutation. It is used in the function
    to filter the data based on mutations
    :param cancers: A list of cancer types. It specifies the cancer types for which you want to extract
    essentiality mutations in cell lines
    :param genes: The `genes` parameter is a list of genes for which you want to extract essentiality
    mutations in cell lines
    """
    res_dict = {}
    #for cancer in cancers:
    #    for gene in genes:
    df_sel = df[df['cancer_type']==cancer]
    df_sel = df_sel[df_sel['screen_gene']==main_gene]
    all_cls = df_sel['list_cl'].values[0].split(',')
    cls_with_ess = df_sel['list_cl_with_ess'].values[0].split(',')
    cls_with_drv = df_sel['list_cl_gene_drv'].values[0].split(',')
    cls_with_mut = df_sel['list_cl_gene_mut'].values[0].split(',')
    cls_with_ess_drv = df_sel['list_cl_with_ess_gene_drv'].values[0].split(',')
    cls_with_ess_mut = df_sel['list_cl_with_ess_gene_mut'].values[0].split(',')


def extract_first_level_stratification(gene='KRAS'):
    """
    The function `extract_first_level_stratification` extracts various levels of stratification for a
    given gene in different cancer types.
    
    :param gene: The gene parameter is a string that specifies the gene for which you want to extract
    the first-level stratification. In this case, the default gene is set to 'KRAS', defaults to KRAS
    (optional)
    :return: The function `extract_first_level_stratification` returns a dictionary containing
    information about the first level stratification of a given gene (default is 'KRAS'). The dictionary
    has the following structure:
    """
    prev_df = previous_results.get_details_per_gene(gene)
    res_dict = {}
    for cancer_type in prev_df['cancer_type'].unique():
        res_dict[cancer_type] = {}
        cancer_df = prev_df[prev_df['cancer_type']==cancer_type]
        res_dict[cancer_type]['all_cls'] = cancer_df['model_name'].unique()
        res_dict[cancer_type]['cls_with_ess'] = cancer_df[cancer_df['cellline_with_essientiality']==True]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_ess'] = cancer_df[cancer_df['gene_is_essential']==1]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_not_ess'] = cancer_df[cancer_df['gene_is_essential']==0]['model_name'].values
        res_dict[cancer_type]['cls_are_drv'] = cancer_df[cancer_df['screen_gene_drv']==True]['model_name'].values
        res_dict[cancer_type]['cls_are_mut'] = cancer_df[cancer_df['screen_gene_mut']==True]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_drv'] = cancer_df[(cancer_df['screen_gene_drv']==True)&(cancer_df['cellline_with_essientiality']==True)]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_mut'] = cancer_df[(cancer_df['screen_gene_mut']==True)&(cancer_df['cellline_with_essientiality']==True)]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_drv_are_ess'] = cancer_df[(cancer_df['screen_gene_drv']==True)&(cancer_df['gene_is_essential']==1)]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_mut_are_ess'] = cancer_df[(cancer_df['screen_gene_mut']==True)&(cancer_df['gene_is_essential']==1)]['model_name'].values
    return res_dict


def extract_entropy_scores(gene='KRAS', cancer='Colorectal_Carcinoma'):
    """
    The function `extract_entropy_scores` extracts various information related to entropy scores for a
    given gene and cancer type.
    
    :param gene: The gene parameter is the name of the gene for which you want to extract entropy
    scores. In this case, the default gene is set to 'KRAS'. You can change the value of the gene
    parameter to extract entropy scores for a different gene, defaults to KRAS (optional)
    :param cancer: The `cancer` parameter refers to the type of cancer for which you want to extract
    entropy scores. In this case, the default value is set to 'Colorectal_Carcinoma'. You can change it
    to any other cancer type for which you have data available, defaults to Colorectal_Carcinoma
    (optional)
    :return: The function `extract_entropy_scores` returns a dictionary containing information about the
    entropy scores for a given gene and cancer type. The dictionary has the following structure:
    """
    prev_df = previous_results.get_details_per_gene(gene)
    res_dict = {}
    for cancer_type in prev_df['cancer_type'].unique():
        res_dict[cancer_type] = {}
        cancer_df = prev_df[prev_df['cancer_type']==cancer_type]
        res_dict[cancer_type]['all_cls'] = cancer_df['model_name'].unique()
        res_dict[cancer_type]['cls_with_ess'] = cancer_df[cancer_df['cellline_with_essientiality']==True]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_ess'] = cancer_df[cancer_df['gene_is_essential']==1]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_not_ess'] = cancer_df[cancer_df['gene_is_essential']==0]['model_name'].values
        res_dict[cancer_type]['cls_are_drv'] = cancer_df[cancer_df['screen_gene_drv']==True]['model_name'].values
        res_dict[cancer_type]['cls_are_mut'] = cancer_df[cancer_df['screen_gene_mut']==True]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_drv'] = cancer_df[(cancer_df['screen_gene_drv']==True)&(cancer_df['cellline_with_essientiality']==True)]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_mut'] = cancer_df[(cancer_df['screen_gene_mut']==True)&(cancer_df['cellline_with_essientiality']==True)]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_drv_are_ess'] = cancer_df[(cancer_df['screen_gene_drv']==True)&(cancer_df['gene_is_essential']==1)]['model_name'].values
        res_dict[cancer_type]['cls_with_ess_are_mut_are_ess'] = cancer_df[(cancer_df['screen_gene_mut']==True)&(cancer_df['gene_is_essential']==1)]['model_name'].values
    return res_dict

def main():
    """
    The main function performs clustering analysis on gene expression data for the KRAS gene in
    colorectal carcinoma, and visualizes the results using UMAP.
    """
    
    expression = get_expression(source='cmp')
    
    
    #genes = ['KRAS']
    #binarized_dependency = get_dependency(binarized=True, processed=True)
    #mut = get_mutation(source='cmp', version='2022')
    kras_dict = extract_first_level_stratification()
    for cancer, cancer_dict in kras_dict.items():
        mutess_size = len(cancer_dict['cls_with_ess_are_mut_are_ess'])
        if mutess_size>3:
            print(f'{cancer}: {mutess_size}')
    scaled_dependency = get_dependency(binarized=False, processed=True)
    X = scaled_dependency[kras_dict['Colorectal Carcinoma']['cls_with_ess_are_mut_are_ess']].T
    
    #expression = get_expression(source='cmp')
    from sklearn.cluster import KMeans
    import umap
    import matplotlib.pyplot as plt
    clustered_celllines = KMeans(n_clusters=2, random_state=0, n_init="auto").fit(X.values)
    clusters = zip(X.index, clustered_celllines.labels_)
    cluster_labels = clustered_celllines.labels_
    label2color = {0: 'blue', 1:'red'}
    cluster_colors = [label2color[label] for label in cluster_labels]
    embedding = umap.UMAP().fit_transform(X.values)
    plt.clf()
    fig, ax = plt.subplots(figsize=(12, 10))
    plt.scatter(
        embedding[:, 0], embedding[:, 1], cmap="Spectral", s=5, c=cluster_colors
    )
    plt.show()
    print()
    #KRAS_entropy_2022_Colorectal_Carcinoma_entropy KRAS_entropy_2022_Colorectal_Carcinoma_entropy.tsv
    
if __name__ == "__main__":
    main()