import numpy as np
import pandas as pd
from src.lib import config
from src.process.data_process import process_oncostratifier

def get_candidate_main_genes(cancer=None, min_mutdep=5, min_rest=5):
    """
    The function `get_candidate_main_genes` reads a file, filters the data based on the specified cancer
    type, and assigns the filtered data to the variable `df`.
    
    :param cancer: The `cancer` parameter is used to specify the type of cancer for which you want to
    retrieve the main genes. It is an optional parameter and if not provided, all cancer types will be
    considered
    :param min_mutdep: The parameter "min_mutdep" represents the minimum mutation depth required for a
    gene to be considered a candidate main gene. It specifies the minimum number of mutations that must
    be observed in a gene for it to be included in the list of candidate main genes, defaults to 5
    (optional)
    :param min_rest: The `min_rest` parameter is the minimum number of samples in which a gene must be
    expressed in order to be considered a candidate main gene, defaults to 5 (optional)
    """
    inloc = config.R_SCREEN_DIR / '2022_tumor_stats.txt'
    df = pd.read_csv(inloc, sep='\t')
    df = df[df['cancer_type']==cancer]
    
    pass

def get_candidate_secondary_genes(cancer=None, main_gene=None):
    pass


def fix_detailed_gene_screens(gene_screen, bdep_dict):
    """
    The function `fix_detailed_gene_screens` takes a gene screen dataframe and a dictionary of gene
    dependencies as input, and updates the dataframe with additional information based on the gene
    dependencies. It then saves the updated dataframe to a file.
    
    :param gene_screen: The `gene_screen` parameter is a pandas DataFrame that contains information
    about gene screens. It has the following columns:
    :param bdep_dict: The `bdep_dict` parameter is a dictionary that contains information about
    essential genes. It is used to determine which genes are essential in a given context
    """
    gene_screen['list_cl_with_ess_gene_ess'] = np.nan
    for row_id, arow in gene_screen.iterrows():
        gene, cancer = arow['screen_gene'], arow['cancer_type']
        all_cls = arow['list_cl'].split(',') if not pd.isnull(arow['list_cl']) else np.array([])
        cls_with_ess = arow['list_cl_with_ess'].split(',') if not pd.isnull(arow['list_cl_with_ess']) else np.array([])
        cls_are_drv = arow['list_cl_gene_drv'].split(',') if not pd.isnull(arow['list_cl_gene_drv']) else np.array([])
        cls_are_mut = arow['list_cl_gene_mut'].split(',') if not pd.isnull(arow['list_cl_gene_mut']) else np.array([])
        cls_with_ess_are_ess = np.intersect1d(cls_with_ess, np.array(bdep_dict[gene]))
        if len(cls_with_ess_are_ess)>0:
            gene_screen.loc[row_id, 'list_cl_with_ess_gene_ess'] = ','.join(cls_with_ess_are_ess)
        cls_with_ess_are_drv = arow['list_cl_with_ess_gene_drv'].split(',') if not pd.isnull(arow['list_cl_with_ess_gene_drv']) else np.array([])
        cls_with_ess_are_mut = np.intersect1d(cls_with_ess, cls_are_mut)
        if len(cls_with_ess_are_mut)>0:
            gene_screen.loc[row_id, 'list_cl_with_ess_gene_mut'] = ','.join(cls_with_ess_are_mut)
        cls_with_ess_are_drv_ess = arow['list_cl_with_ess_gene_drv_ess'].split(',') if not pd.isnull(arow['list_cl_with_ess_gene_drv_ess']) else np.array([])
        cls_with_ess_are_mut_ess = np.intersect1d(cls_with_ess_are_mut, cls_with_ess_are_ess)
        if len(cls_with_ess_are_mut_ess)>0:
            gene_screen.loc[row_id, 'list_cl_with_ess_gene_mut_ess'] = ','.join(cls_with_ess_are_mut_ess)
        if arow['count_cl_with_ess_gene_mut']!=len(cls_with_ess_are_mut):
            raise Exception(f"Sorry, numbers for mutated with ess does not match: {arow['count_cl_with_ess_gene_mut']} vs {len(cls_with_ess_are_mut)} for {row_id} ")
        if arow['count_cl_with_ess_gene_mut_ess']!=len(cls_with_ess_are_mut_ess):
            raise Exception(f"Sorry, numbers for mutated with ess does not match: {arow['count_cl_with_ess_gene_mut_ess']} vs {len(cls_with_ess_are_mut_ess)} for {row_id} ")
    populated =  gene_screen[gene_screen['count_cl_with_ess_gene_mut_ess']>0]
    outloc = config.R2_SCREEN_DIR / '2022_tumor_stats_listfix.txt'
    gene_screen.to_csv(outloc, sep='\t', index=False)
    
def get_detailed_gene_screens(variant='2022', cancer=None, main_gene=None, fixed=True):
    """
    The function `get_detailed_gene_screens` retrieves detailed gene screens based on the specified
    variant, cancer type, and main gene, with an option to use fixed or non-fixed data.
    
    :param variant: The variant parameter is a string that represents the specific variant or version of
    the gene screens. It is set to '2022' by default, defaults to 2022 (optional)
    :param cancer: The "cancer" parameter is used to filter the data by a specific cancer type. If you
    pass a value to the "cancer" parameter, the function will only return rows where the "cancer_type"
    column matches the specified value. If you don't pass a value to the "
    :param main_gene: The main_gene parameter is used to filter the dataframe by a specific gene. Only
    rows where the 'screen_gene' column matches the value of main_gene will be included in the returned
    dataframe
    :param fixed: The "fixed" parameter is a boolean value that determines whether to use the fixed
    version of the gene screens file or not. If set to True, it will use the fixed version, otherwise it
    will use the regular version, defaults to True (optional)
    :return: a pandas DataFrame object.
    """
    if fixed:
        in_loc = config.R2_SCREEN_DIR / f'{variant}_tumor_stats_listfix.txt'
    else:
        in_loc = config.R_SCREEN_DIR / f'{variant}_tumor_stats.txt'
    df = pd.read_csv(in_loc, sep='\t')
    if cancer is not None:
        df = df[df['cancer_type']==cancer]
    if main_gene is not None:
        df = df[df['screen_gene']==main_gene]
    return df 


def get_details_per_gene(gene='KRAS', cancer='Colorectal_Carcinoma', variant='2022'): 
    """
    The function `get_details_per_gene` returns a DataFrame containing details for a specific gene,
    cancer type, and variant.
    
    :param gene: The gene parameter specifies the gene for which you want to retrieve details. In this
    case, the default value is 'KRAS', which means that if no gene is specified when calling the
    function, it will retrieve details for the KRAS gene, defaults to KRAS (optional)
    :param cancer: The 'cancer' parameter refers to the type of cancer for which you want to retrieve
    the details per gene. In this case, the default value is 'Colorectal_Carcinoma', but you can change
    it to any other type of cancer for which you have the corresponding data, defaults to
    Colorectal_Carcinoma (optional)
    :param variant: The 'variant' parameter refers to a specific variant or mutation of the gene. In
    this case, it is used to specify the specific variant of the gene 'KRAS' that you are interested in,
    defaults to 2022 (optional)
    :return: a pandas DataFrame containing the details of a specific gene, cancer type, and variant.
    """
    res_dict = {}
    in_loc = config.R_RESULTS_DIR / f'{gene}_entropy_{variant}_{cancer}_tumor_stats_details.txt'
    df = pd.read_csv(in_loc, sep='\t')
    return df


def get_entropy_scores(gene='KRAS', cancer='Colorectal_Carcinoma', variant='2022', add_categories=False, only_significant=False): 
    """
    The function `get_entropy_scores` retrieves entropy scores for a specific gene, cancer type, and
    variant, and optionally adds categories based on significance and other criteria.
    
    :param gene: The gene parameter specifies the gene for which you want to retrieve entropy scores. In
    this case, the default value is 'KRAS', defaults to KRAS (optional)
    :param cancer: The 'cancer' parameter specifies the type of cancer for which you want to get the
    entropy scores. In this case, the default value is 'Colorectal_Carcinoma', defaults to
    Colorectal_Carcinoma (optional)
    :param variant: The 'variant' parameter specifies the specific variant of the gene that you want to
    analyze, defaults to 2022 (optional)
    :param add_categories: The `add_categories` parameter is a boolean flag that determines whether
    additional categories should be added to the dataframe. If `add_categories` is set to `True`, the
    function will add the following columns to the dataframe:, defaults to False (optional)
    :param only_significant: The parameter "only_significant" is a boolean flag that determines whether
    to include only significant results in the output. If set to True, only rows with a p-value less
    than the specified cutoff (0.05) will be included in the output dataframe. If set to False, all rows
    will, defaults to False (optional)
    :return: a pandas DataFrame containing entropy scores for a specific gene, cancer type, and variant.
    If the 'add_categories' parameter is set to True, additional columns 'category', '2category', and
    '3category' are added to the DataFrame. If the 'only_significant' parameter is set to True, only
    rows with a p-value less than a specified cutoff are included in
    """
    in_loc = config.R_RESULTS_DIR / f'{gene}_entropy_{variant}_{cancer}_entropy' / f'{gene}_entropy_{variant}_{cancer}_entropy.tsv'
    df = pd.read_csv(in_loc, sep='\t')
    if add_categories:
        df['category'] = 'NotFound' 
        df['2category'] = 0
        cutoff=0.05
        if only_significant:
            df = df[df['p']<cutoff]
        if len(df) == 0:
            return None
        df.loc[(df['p']<cutoff)&(df['diffEntropy']<0)&(df['mutDep_candidateGene_n']>df['mutDep_notEssential_n']), 'category']='co-dependency'
        df.loc[(df['p']<cutoff)&(df['diffEntropy']<0)&(df['mutDep_candidateGene_n']<=df['mutDep_notEssential_n']), 'category']='exclusive-dependency'
        df.loc[(df['p']<cutoff)&(df['diffEntropy']>0)&(df['Others_candidateGene_n']>df['Others_notEssential_n']), 'category']='stratifying non-essentiality'
        df.loc[(df['p']<cutoff)&(df['diffEntropy']>0)&(df['Others_candidateGene_n']<=df['Others_notEssential_n']), 'category']='stratifying essentiality'
        df.loc[df['category']=='stratifying essentiality', '2category']=1
        df.loc[df['category']=='stratifying non-essentiality', '2category']=1
        df['3category']=df['category']
        
    return df
    

def main():
    #get_candidate_main_genes(cancer='Colorectal Carcinoma')
    #get_detailed_gene_screens(variant='2022', cancer='Colorectal Carcinoma', main_gene='KRAS')
    #get_details_per_gene(gene='KRAS', cancer='Colorectal_Carcinoma', variant='2022')
    bdep = process_oncostratifier.get_dependency(binarized=True, processed=True)
    bdep_dict = process_oncostratifier.depmatrix2genedict(bdep)
    df = get_detailed_gene_screens(variant='2022')
    fix_detailed_gene_screens(df, bdep_dict)
    print()
    
if __name__ == "__main__":
    main()