import sys, os
sys.path.insert(0, os.getcwd())
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
from matplotlib.colors import to_rgba, LinearSegmentedColormap
from matplotlib.patches import Patch
import collections
import re
import warnings
if not sys.warnoptions:
warnings.simplefilter("ignore")
os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses
from src.lib import config
from src.analysis.results_analysis import oncostratifier_by_drug as onco_res
from src.process.data_process import process_drug

def add_target_columns(df, only_targets=False):
    #Load targets into the dataframe
    if only_targets:
        df = df[df['target_hit']==True]
    label_add = '_target'
    list_of_labels = ['name_for_cancer', 'name_for_cancer_w_drug_name_version', 'name_for_cancer_w_drug_name', 'name_for_cancer_w_drug_id_name', 
                    'name_for_gene', 'name_for_gene_w_drug_name_version', 'name_for_gene_w_drug_name', 'name_for_gene_w_drug_id_name',
                    'candidateDrug', 'drug_name_version', 'drug_name', 'drug_id_name', 'name', 'name_w_drug_name', 'name_for_cancer_w_drug_id_name',
                    'name_for_cancer_w_drug_name_version']
    for name_col in list_of_labels:
        df[f'{name_col}{label_add}'] = df[name_col] + ' (' + df['targets'] + ')'
    return df

def create_drug_df_for_violin(res, main_division):
    #Create a new dataframe for the violin plots
    res_drug_list = []
    #Loop over results (Per drug)
    for row_id, row in res.iterrows():
        #If drug version is used remove it from the drug name on the plot.
        drug = f"{row['drug_id']}\n{row['drug_name']}\n{row['drug_version']}"
        
        #Annotate
        full_drug = row['candidateDrug']
        gdsc_drug_cellline = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])
        mutdep, others = row['mutDep_with_response_list'].split(','), row['others_with_response_list'].split(',')
        all_cell_lines = np.concatenate([mutdep, others], axis=0)
        mutdep_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in mutdep])
        others_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in others])
        all_lnmaxconc = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_MAX_CONC'] for celline in all_cell_lines])
        res_drug_list.append(pd.DataFrame({
            'Cancer': row['cancer'], 'ScreenGene': row['screenGene'], 'Cell Lines': np.concatenate([mutdep, others],axis=0),
            'LN_IC50': np.concatenate([mutdep_lnic50, others_lnic50],axis=0),
            'Candidate Gene Status': [main_division]*len(mutdep)+['Others']*len(others),
            'Drug': [drug]*len(all_cell_lines), 'Threshold': all_lnmaxconc, 'DiffEntropy': row['diffEntropy'],
            'P Value': row['p'], 'MutDep Entropy': row['mutDep_entropy'], 'Others Entropy': row['others_entropy'],
            'Category': row['category'],
        }))
    if len(res_drug_list)>0:
        return pd.concat(res_drug_list)
    else:
        return None


def plot_circular_bar_for_all_results(folder, res, cancers, genes, target_included=False, version=1):
    a_res_df['screenGene'], a_res_df['cancer']
    label_add = ''
    if target_included:
        label_add = '_target'
    
    for cancer in cancers:
        out_loc = folder / 'images' / 'by_cancer' / f'{cancer}_drugidname_v{version}.pdf'
        if out_loc.exists():
            print(f'{cancer} cancer is already processed!')
            continue
        print(f'For cancer {cancer}')
        res_cancer = res[res['cancer']==cancer]
        if len(res_cancer)>0:
            out_loc = folder / 'images' / 'by_cancer' / f'{cancer}_drugidname_v{version}.pdf'
            config.ensure_dir(out_loc)
            circular_bar(res_cancer.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'name_for_cancer{label_add}', center_label = f'{cancer}', out_loc=out_loc)
            
            out_loc = folder / 'images' / 'by_cancer' / f'{cancer}_drugnameversion_v{version}.pdf'
            circular_bar(res_cancer.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'name_for_cancer_w_drug_name_version{label_add}', center_label = f'{cancer}', out_loc=out_loc)
            
            out_loc = folder / 'images' / 'by_cancer' / f'{cancer}_drugname_v{version}.pdf'
            circular_bar(res_cancer.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'name_for_cancer_w_drug_name{label_add}', center_label = f'{cancer}', out_loc=out_loc)
        del res_cancer
                
    
    for gene in genes:
        out_loc = folder / 'images' / 'by_gene' / f'{gene}_drugidname_v{version}.pdf'
        if out_loc.exists():
            print(f'{gene} gene is already processed!')
            continue
        print(f'For gene {gene}')
        res_gene = res[res['screenGene']==gene]
        if len(res_gene)>0:
            out_loc = folder / 'images' / 'by_gene' / f'{gene}_drugidname_v{version}.pdf'
            config.ensure_dir(out_loc)
            circular_bar(res_gene.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'name_for_gene{label_add}', center_label = f'{gene}', out_loc=out_loc)
            
            out_loc = folder / 'images' / 'by_gene' / f'{gene}_drugnameversion_v{version}.pdf'
            circular_bar(res_gene.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'name_for_gene_w_drug_name_version{label_add}', center_label = f'{gene}', out_loc=out_loc)
            
            out_loc = folder / 'images' / 'by_gene' / f'{gene}_drugname_v{version}.pdf'
            circular_bar(res_gene.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'name_for_gene_w_drug_name{label_add}', center_label = f'{gene}', out_loc=out_loc)
        del res_gene
                    
    for cancer in cancers:
        res_cancer = res[res['cancer']==cancer]
        for gene in genes:
            out_loc = folder / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugidname_v{version}.pdf'
            if out_loc.exists():
                print(f'{cancer}-{gene} gene is already processed!')
                continue
            print(f'For cancer-gene: {cancer}-{gene}')
            res_cancer_gene = res_cancer[res_cancer['screenGene']==gene]
            if res_cancer_gene is not None:
                out_loc = folder / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugidname_v{version}.pdf'
                config.ensure_dir(out_loc)
                circular_bar(res_cancer_gene.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'candidateDrug{label_add}', center_label = f'{cancer}_{gene}', out_loc=out_loc)
                
                out_loc = folder / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugnameversion_v{version}.pdf'
                circular_bar(res_cancer_gene.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'drug_name_version{label_add}', center_label = f'{cancer}_{gene}', out_loc=out_loc)
                
                out_loc = folder / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugname_v{version}.pdf'
                circular_bar(res_cancer_gene.copy(), value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=True, labels_name=f'drug_name{label_add}', center_label = f'{cancer}_{gene}', out_loc=out_loc)
            del res_cancer_gene

def plot_ic50_violin_per_cancer_gene_drug(drug_version=None, main_division='mutdep', verbose=0):
    #Loop over cancers and genes:
    for cancer in cancers:
        for gene in genes:
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugidname_v{self.visual_version}.pdf'
            if out_loc.exists():
                print(f'{cancer}-{gene} gene is already processed!')
                continue
            print(f'For cancer-gene: {cancer}-{gene}')
            #Get result per cancer-gene pair, categorize the results and use 0.05 as p-value threshold.
            res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer)
            res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
            
            if res is not None:                
                #Divide the set according to the GDSC version if drug_version is given.
                if drug_version is not None: 
                    res = res[res['drug_version']==drug_version]
                res_drug_list = []
                
                #Loop over results (Per drug)
                for row_id, row in res.iterrows():
                    if 'stratifying' in row['category'].lower():
                        if verbose==1:
                            print(f'For cancer-gene-drug: {cancer} - {gene} - {drug}')
                        #If drug version is used remove it from the drug name on the plot.
                        if drug_version is None:
                            drug = f"{row['drug_id']}\n{row['drug_name']}\n{row['drug_version']}"
                        else:
                            drug = f"{row['drug_id']}\n{row['drug_name']}"
                        
                        #Annotate
                        full_drug = row['candidateDrug']
                        gdsc_drug_cellline = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])
                        mutdep, others = row['mutDep_with_response_list'].split(','), row['others_with_response_list'].split(',')
                        all_cell_lines = np.concatenate([mutdep, others], axis=0)
                        mutdep_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in mutdep])
                        others_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in others])
                        all_lnmaxconc = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_MAX_CONC'] for celline in all_cell_lines])
                        res_drug_list.append(pd.DataFrame({
                            'Cell Lines': np.concatenate([mutdep, others],axis=0),
                            'LN_IC50': np.concatenate([mutdep_lnic50, others_lnic50],axis=0),
                            'Candidate Gene Status': ['MutDep']*len(mutdep)+['Others']*len(others),
                            'Drug': [drug]*len(all_cell_lines),
                            'Threshold': all_lnmaxconc,
                            'DiffEntropy': row['diffEntropy'],
                            'P Value': row['p'],
                            'MutDep Entropy': row['mutDep_entropy'],
                            'Others Entropy': row['others_entropy'],
                            'Category': row['category'],
                        }))
                if len(res_drug_list)>0:
                    res_drugs = pd.concat(res_drug_list)
                    if drug_version is None:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene' / f'{cancer}_{gene}_v3.pdf'
                        title_label = f'{cancer}\n{gene}'
                    else:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene_gdsc_version' / f'{cancer}_{gene}_{drug_version}_v3.pdf'
                        title_label = f'{cancer}\n{gene}\n{drug_version}'
                    config.ensure_dir(out_loc)
                    violin_distribution(res_drugs, label=title_label, out_loc=out_loc)
                    
                    

def plot_ic50_violin_per_cancer_gene_drug_facet(drug_version=None, main_division='mutdep', verbose=0):
    print(f'Violin_facet started!')
    #Loop over cancers and genes:
    for cancer in cancers:
        for gene in genes:
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene' / f'{cancer}_{gene}_facet_v1.pdf'
            if out_loc.exists():
                print(f'{cancer}-{gene} gene is already processed!')
                continue
            print(f'For cancer-gene: {cancer}-{gene}')
            #Get result per cancer-gene pair, categorize the results and use 0.05 as p-value threshold.
            res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer)
            res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
            
            if res is not None:                
                #Divide the set according to the GDSC version if drug_version is given.
                if drug_version is not None: 
                    res = res[res['drug_version']==drug_version]
                res_drug_list = []
                res['category'] = res['category'].replace(category_name_fix)
                #Loop over results (Per drug)
                for row_id, row in res.iterrows():
                    #If drug version is used remove it from the drug name on the plot.
                    if drug_version is None:
                        drug = f"{row['drug_id']}\n{row['drug_name']}\n{row['drug_version']}"
                    else:
                        drug = f"{row['drug_id']}\n{row['drug_name']}"
                    if verbose==1:
                        print(f'For cancer-gene-drug: {cancer} - {gene} - {drug}')
                    
                    #Annotate
                    full_drug = row['candidateDrug']
                    gdsc_drug_cellline = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])
                    mutdep, others = row['mutDep_with_response_list'].split(','), row['others_with_response_list'].split(',')
                    all_cell_lines = np.concatenate([mutdep, others], axis=0)
                    mutdep_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in mutdep])
                    others_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in others])
                    all_lnmaxconc = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_MAX_CONC'] for celline in all_cell_lines])
                    res_drug_list.append(pd.DataFrame({
                        'Cell Lines': np.concatenate([mutdep, others],axis=0),
                        'LN_IC50': np.concatenate([mutdep_lnic50, others_lnic50],axis=0),
                        'Candidate Gene Status': [main_division]*len(mutdep)+['Others']*len(others),
                        'Drug': [drug]*len(all_cell_lines),
                        'Threshold': all_lnmaxconc,
                        'DiffEntropy': row['diffEntropy'],
                        'P Value': row['p'],
                        'MutDep Entropy': row['mutDep_entropy'],
                        'Others Entropy': row['others_entropy'],
                        'Category': row['category'],
                    }))
                if len(res_drug_list)>0:
                    res_drugs = pd.concat(res_drug_list)
                    if drug_version is None:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene' / f'{cancer}_{gene}_facet_v1.pdf'
                        title_label = f'{cancer}\n{gene}'
                    else:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene_gdsc_version' / f'{cancer}_{gene}_{drug_version}_facet_v1.pdf'
                        title_label = f'{cancer}\n{gene}\n{drug_version}'
                    config.ensure_dir(out_loc)
                    violin_distribution_facet(res_drugs, label=title_label, out_loc=out_loc)
                        

def plot_all_for_drug_target_match_distribution(main_division = ''):
    cache_name = f'{experiment_folder_name}/combined_{args.cancer}_{args.main_gene}_results_with_targetinfo_tmp.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        res = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=args.main_gene, cancer=args.cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)

        cache_dict = {'data': res}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)
    
    
    #Load targets into the dataframe
    res = add_target_columns(res, only_targets=True)
    
    #Plot for all cancer, gene pairs
    #Cicular plot of target matching distribution
    out_circular_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{args.cancer}_{args.main_gene}_circbar_drugname_v{self.visual_version}.pdf'
    config.ensure_dir(out_circular_loc)
    if not out_circular_loc.exists():
        circular_bar(res.copy(), sorted=True, labels_name=f'name_w_drug_name{label_add}', center_label = f'All', out_loc=out_circular_loc)
    #Histogram Bar of target matching distribution
    out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{args.cancer}_{args.main_gene}_hist_v{self.visual_version}.pdf'
    config.ensure_dir(out_loc)
    if not out_loc.exists():
        title = "Distribution of drugs that match with its target into categories "
        histogram_bar(res, label=title, out_loc=out_loc)
    
    #Plot for each cancer
    for cancer in cancers:
        res_cancer = res[res['cancer']==cancer]
        if len(res_cancer)==0:
            continue
        #Cicular plot of target matching distribution
        out_circular_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{cancer}_circbar_drugname_v{self.visual_version}.pdf'
        if not out_circular_loc.exists():
            circular_bar(res_cancer.copy(), sorted=True, labels_name=f'name_for_cancer_w_drug_name{label_add}', center_label = f'{cancer}', out_loc=out_circular_loc)
        
        #Histogram Bar of target matching distribution
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{cancer}_hist_v{self.visual_version}.pdf'
        if not out_loc.exists():
            title = f"Distribution of drugs that match with its target into categories in {cancer}"
            histogram_bar(res_cancer.copy(), label=title, out_loc=out_loc)
        
        #Violin plots
        res_drugs_cancer = create_drug_df_for_violin(res_cancer, main_division)
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{cancer}_facet_violin_v{self.visual_version}.pdf'
        if len(res_drugs_cancer)>0 and not out_loc.exists():
            violin_distribution_facet(res_drugs_cancer, label=f'{cancer}', out_loc=out_loc)
        
        del res_cancer, res_drugs_cancer
    
    #Plot for each gene
    for gene in genes:
        res_gene = res[res['screenGene']==gene]
        if len(res_gene)==0:
            continue
        #Cicular plot of target matching distribution
        out_circular_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{gene}_circbar_drugname_v{self.visual_version}.pdf'
        if not out_circular_loc.exists():
            circular_bar(res_gene.copy(), sorted=True, labels_name=f'name_for_gene_w_drug_name{label_add}', center_label = f'{gene}', out_loc=out_circular_loc)
        
        #Histogram Bar of target matching distribution
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{gene}_hist_v{self.visual_version}.pdf'
        if not out_loc.exists():
            title = f"Distribution of drugs that match with its target into categories in {gene}"
            histogram_bar(res_gene.copy(), label=title, out_loc=out_loc)
        
        #Violin plots
        res_drugs_gene = create_drug_df_for_violin(res_gene, main_division)
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{gene}_facet_violin_v{self.visual_version}.pdf'
        if len(res_drugs_gene)>0 and not out_loc.exists():
            violin_distribution_facet(res_drugs_gene, label=f'{gene}', out_loc=out_loc)
            
        del res_gene, res_drugs_gene
        

def plot_histogram_of_significant_drugs(main_division = '', drug_label=''):
    cache_name = f'{experiment_folder_name}/combined_{args.cancer}_{args.main_gene}_results_with_targetinfo_tmp.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        res = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=args.main_gene, cancer=args.cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)

        cache_dict = {'data': res}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)

    res = add_target_columns(res, only_targets=False)
    #Histogram Bar of target matching distribution
    for acol in [col for col in res.columns if '_target' in col]:
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'drug_distribution' / f'{args.cancer}_{args.main_gene}_{acol}_hist_v{self.visual_version}.pdf'
        config.ensure_dir(out_loc)
        if True:#not out_loc.exists():
            title = "Histogram of drugs into categories"
            facet_histogram_bar(res, y=acol, label=title, out_loc=out_loc)


def plot_differential_distribution(main_division = '', cancer=None, gene=None, variant_version='cmp_2022'):
    cache_name = f'{experiment_folder_name}/combined_{args.cancer}_{args.main_gene}_results_with_targetinfo_tmp.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        res = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=args.main_gene, cancer=args.cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)

        cache_dict = {'data': res}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)

    res = add_target_columns(res, only_targets=False)
    #Histogram Bar of target matching distribution
    for acol in [col for col in res.columns if '_target' in col]:
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'drug_distribution' / f'{args.cancer}_{args.main_gene}_{acol}_hist_v{self.visual_version}.pdf'
        config.ensure_dir(out_loc)
        if True:#not out_loc.exists():
            title = "Histogram of drugs into categories"
            facet_histogram_bar(res, y=acol, label=title, out_loc=out_loc)
            


def plot_celllines_scatter_or_stackedbar_with_drug(main_division = '', cancer=None, gene=None, variant_version='cmp_2022', plot_type='scatter_pie'):
    cache_name = f'{experiment_folder_name}/combined_{args.cancer}_{args.main_gene}_stratifying_results_with_targetinfo_ordered_tmp.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        res = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True, returned_categories=['stratifying essentiality', 'stratifying non-essentiality'])
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
        res = add_target_columns(res, only_targets=False)
        sign_markers = onco_res.analyze_differential_mutation_results(main_division=main_division, cancer=cancer, gene=gene, variant_version=variant_version)
        potential_drugs = sign_markers['candidateDrug'].unique()
        res['z_order']=1
        res_drug_index = res.set_index('candidateDrug')
        res_drug_index.loc[potential_drugs, 'z_order']=0
        res = res_drug_index.sort_values(by='z_order').reset_index()
        cache_dict = {'data': res}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)

    out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'sensitivity_scatter' / f'{args.cancer}' / f'{args.main_gene}' / f'stratifying_v{self.visual_version}_horizontal_res.pdf'
    config.ensure_dir(out_loc)
    title = f'Drugs on {gene}-Mut vs {gene}-WT {cancer} Cell Lines'
    if plot_type == 'scatter_pie':
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'sensitivity_scatter' / f'{args.cancer}' / f'{args.main_gene}' / f'stratifying_v{self.visual_version}_horizontal_scatter_pie_res.pdf'
        scatter_pie(res, xs=['mutDep', 'others'], y='with_response_list', hue='drug_name_target',  label=title, gene=gene, include_resistance=True, out_loc=out_loc)
    if plot_type == 'stacked_bar':
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'sensitivity_scatter' / f'{args.cancer}' / f'{args.main_gene}' / f'stratifying_v{self.visual_version}_horizontal_stacked_bar_res.pdf'
        stacked_bar(res, xs=['mutDep', 'others'], y='with_response_list', hue='drug_name_target',  label=title, gene=gene, include_resistance=True, out_loc=out_loc)


def plot_drug_cellline_heatmap(main_division = '', cancer=None, gene=None, variant_version='cmp_2022'):
    cache_name = f'{experiment_folder_name}/combined_{cancer}_{gene}_cellline_based_df.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        cellline_drug_df = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True, returned_categories=['stratifying essentiality', 'stratifying non-essentiality'])
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
        res = add_target_columns(res, only_targets=False)
        sign_markers = onco_res.analyze_differential_mutation_results(main_division=main_division, cancer=cancer, gene=gene, variant_version=variant_version)
        potential_drugs = sign_markers['candidateDrug'].unique()
        res['z_order']=1
        res_drug_index = res.set_index('candidateDrug')
        res_drug_index.loc[potential_drugs, 'z_order']=0
        res = res_drug_index.sort_values(by='z_order')#.reset_index()
        
        res = res.reset_index().set_index('candidateDrug_target')
        all_target_ccles, mutDep_target_ccles, others_target_ccles = [], [], []
        for with_resp_list in res['all_with_response_list'].values: all_target_ccles+=with_resp_list.split(',')
        res_list = []
        unique_drugs = np.unique(res.index.values)
        for a_ccle in np.unique(all_target_ccles):
            for a_drug in unique_drugs:
                response_in_mutdep = 'Sensitive' if a_ccle in str(res.loc[a_drug, 'mutDep_sensitive_list']) else 'Resistant' if a_ccle in str(res.loc[a_drug, 'mutDep_resistant_list']) else 'No Response' #x = 2 if i>100 else 1 if i<100 else 0
                response_in_others = 'Sensitive' if a_ccle in str(res.loc[a_drug, 'others_sensitive_list']) else 'Resistant' if a_ccle in str(res.loc[a_drug, 'others_resistant_list']) else 'No Response'
                one_res = [a_ccle, a_drug, response_in_mutdep, response_in_others]
                for other_column in res.columns:
                    if '_n' != other_column[-2:] and '_list' not in other_column:
                        one_res.append(res.loc[a_drug, other_column])
                res_list.append(one_res)
        new_columns = ['Cell Line', 'Drug (Target)', 'Mut Response', 'WT Response']
        for other_column in res.columns:
            if '_n' != other_column[-2:] and '_list' not in other_column:
                new_columns.append(other_column)
        cellline_drug_df = pd.DataFrame(res_list, columns=new_columns)
        
        cache_dict = {'data': cellline_drug_df}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)
    for cancer_loop in cellline_drug_df['cancer'].unique():
        cancer_cellline_drug_df = cellline_drug_df[cellline_drug_df['cancer']==cancer_loop]
        for gene_loop in cancer_cellline_drug_df['screenGene'].unique():
            gene_cancer_cellline_drug_df = cancer_cellline_drug_df[cancer_cellline_drug_df['screenGene']==gene_loop]
            title = f'Drugs on {gene_loop}-Mut vs {gene_loop}-WT {cancer_loop} Cell Lines'
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'drug_cellline_heatmap' / f'{cancer_loop}' / f'{gene_loop}' / f'heatmap_{self.visual_version}.pdf'
            if True:#not out_loc.exists():
                heatmap(gene_cancer_cellline_drug_df, title=title, out_loc=out_loc)
                


def plot_response_with_mutational_signature(main_division = '', variant_version='cmp_2022', drug_version=None, verbose=0):
    for cancer in cancers:
        for gene in genes:
            print(f'For cancer-gene: {cancer}-{gene}')
            #Get result per cancer-gene pair, categorize the results and use 0.05 as p-value threshold.
            res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer)
            res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True, returned_categories=['stratifying essentiality', 'stratifying non-essentiality'])
            
            curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
            gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
            gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
            gdsc_nested_targets = collections.defaultdict(dict)
            for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
            
            if res is None:
                continue
            res['category'] = res['category'].replace(category_name_fix)
            res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
            res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
            res = add_target_columns(res, only_targets=False)
            sign_markers = onco_res.analyze_differential_mutation_results(main_division=main_division, cancer=cancer, gene=gene, variant_version=variant_version)
            potential_drugs = sign_markers['candidateDrug'].unique()
            res['z_order']=1
            res_drug_index = res.set_index('candidateDrug')
            res_drug_index.loc[potential_drugs, 'z_order']=0
            res = res_drug_index.sort_values(by='z_order')#.reset_index()
            res = res[res['z_order']==0]
            res = pd.merge(sign_markers.reset_index(), res, how='left', on=['screenGene', 'cancer', 'candidateDrug', 'label'])
            
            if res is not None:                
                #Divide the set according to the GDSC version if drug_version is given.
                if drug_version is not None: 
                    res = res[res['drug_version']==drug_version]
                    
                #Loop over results (Per drug)
                for row_id, row in res.iterrows():
                    #If drug version is used remove it from the drug name on the plot.
                    if drug_version is None:
                        drug = row['drug_name_target']#f"{row['drug_id']}\n{row['drug_name']}\n{row['drug_version']}"
                    else:
                        drug = f"{row['drug_id']}\n{row['drug_name']}"
                    marker =row['differentialGene']
                    if verbose==1:
                        print(f'For cancer-gene-drug: {cancer} - {gene} - {drug} - {marker}')
                    out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'response_with_mut_marker' / f'{cancer}_{gene}_{drug}_{marker}_v{self.visual_version}.pdf'
                    if False:#out_loc.exists():
                        print(f'{cancer}-{gene}-{drug}-{marker} is already processed!')
                        continue
                    
                    #Annotate
                    full_drug = row['candidateDrug']
                    gdsc_drug_cellline = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])
                    mutdep, others = row['mutDep_with_response_list'].split(','), row['others_with_response_list'].split(',')
                    mutdep_mutated, others_mutated = row['mutDep_mutated_list'].split(','), row['others_mutated_list'].split(',')
                    mutdep_notmutated, others_notmutated = row['mutDep_not_mutated_list'].split(','), row['others_not_mutated_list'].split(',')
                    mutdep_lnic50 = np.column_stack(([f'{gene}-Mut']*len(mutdep), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in mutdep]))
                    others_lnic50 = np.column_stack(([f'{gene}-WT']*len(others), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in others]))
                    mutdep_mutated_lnic50 = np.column_stack(([f'{gene}-Mut\n{marker}-Mut']*len(mutdep_mutated), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in mutdep_mutated]))
                    others_mutated_lnic50 = np.column_stack(([f'{gene}-WT\n{marker}-Mut']*len(others_mutated), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in others_mutated]))
                    mutdep_notmutated_lnic50 = np.column_stack(([f'{gene}-Mut\n{marker}-WT']*len(mutdep_notmutated), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in mutdep_notmutated]))
                    others_notmutated_lnic50 = np.column_stack(([f'{gene}-WT\n{marker}-WT']*len(others_notmutated), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in others_notmutated]))
                    all_lnic50 = np.concatenate([mutdep_lnic50, others_lnic50, mutdep_mutated_lnic50, others_mutated_lnic50, mutdep_notmutated_lnic50, others_notmutated_lnic50], axis=0)
                    if self.visual_version==1:
                        order= [f'{gene}-WT', f'{gene}-Mut', f'{gene}-Mut\n{marker}-Mut']
                    if self.visual_version==2:
                        order= [f'{gene}-WT\n{marker}-WT', f'{gene}-WT\n{marker}-Mut', f'{gene}-Mut\n{marker}-WT', f'{gene}-Mut\n{marker}-Mut']
                    
                    plot_df = pd.DataFrame(all_lnic50, columns=['Group', 'LN(IC50)', 'LN_SENSITIVE'])
                    plot_df['ScreenGene'] = gene
                    plot_df['Cancer'] = cancer
                    plot_df['Drug'] = drug
                    plot_df['Marker'] = marker
                    plot_df['Status'] = plot_df['LN_SENSITIVE'].astype(float).astype(int).replace({0:'Resistant', 1:'Sensitive'})
                    plot_df['LN(IC50)'] = plot_df['LN(IC50)'].astype(float)
                    plot_df = plot_df[plot_df['Group'].isin(order)]
                    title_label = f'{cancer}\n{gene}\n{drug}\n{marker}'
                    config.ensure_dir(out_loc)
                    swarm_per_group(plot_df, order=order, label=title_label, out_loc=out_loc)


def call_plot(visual_type, **args):
    if visual_type == 'circular_bar':
        plot_circular_bar_for_all_results(args)
    elif visual_type == 'violin':
        plot_ic50_violin_per_cancer_gene_drug(drug_version='GDSC2', main_division='Mut')
    elif visual_type == 'violin_facet':
        plot_ic50_violin_per_cancer_gene_drug_facet(drug_version=None, main_division=args.main_division)
    elif visual_type == 'target_match_distribution':
        plot_all_for_drug_target_match_distribution(main_division=args.main_division)
    elif visual_type == 'significant_drug_distribution':
        plot_histogram_of_significant_drugs(main_division=args.main_division, drug_label='')
    elif visual_type == 'differential_mutation': 
        plot_differential_distribution(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
    elif visual_type == 'cellline_scatter_with_drug_sensitivity': 
        plot_celllines_scatter_or_stackedbar_with_drug(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version, plot_type='stacked_bar')
    elif visual_type == 'drug_cellline_heatmap': 
        plot_drug_cellline_heatmap(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
    elif visual_type == 'response_with_mutational_signature': 
        plot_response_with_mutational_signature(main_division=args.main_division, variant_version=args.variant_version)
    elif visual_type == 'drug_coverage': 
        plot_drug_coverage(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)


def main():
if args.visual_type == 'circular_bar':
    plot_circular_bar_for_all_results(target_included=True, main_division=args.main_division)
elif args.visual_type == 'violin':
    plot_ic50_violin_per_cancer_gene_drug(drug_version='GDSC2', main_division='Mut')
elif args.visual_type == 'violin_facet':
    plot_ic50_violin_per_cancer_gene_drug_facet(drug_version=None, main_division=args.main_division)
elif args.visual_type == 'target_match_distribution':
    plot_all_for_drug_target_match_distribution(main_division=args.main_division)
elif args.visual_type == 'significant_drug_distribution':
    plot_histogram_of_significant_drugs(main_division=args.main_division, drug_label='')
elif args.visual_type == 'differential_mutation': 
    plot_differential_distribution(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
elif args.visual_type == 'cellline_scatter_with_drug_sensitivity': 
    plot_celllines_scatter_or_stackedbar_with_drug(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version, plot_type='stacked_bar')
elif args.visual_type == 'drug_cellline_heatmap': 
    plot_drug_cellline_heatmap(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
elif args.visual_type == 'response_with_mutational_signature': 
    plot_response_with_mutational_signature(main_division=args.main_division, variant_version=args.variant_version)
elif args.visual_type == 'drug_coverage': 
    plot_drug_coverage(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)

#analyze_differential_mutation_results(main_division=args.main_division.lower(), cancer=cancer, gene=args.main_gene, variant_version=args.variant_version)#args.cancer
if __name__ == "__main__":
main()