import sys, os
sys.path.insert(0, os.getcwd())
#Packages
import numpy as np
import pandas as pd
import seaborn as sns
import collections
import re
from scipy.stats import mannwhitneyu

#Systemwide
import warnings
if not sys.warnoptions:
    warnings.simplefilter("ignore")
    os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses
from src.lib import config
from src.analysis.results_analysis import oncostratifier_by_drug as onco_res
from src.process.data_process import process_drug

#Matplotlib
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
from matplotlib.colors import to_rgba, LinearSegmentedColormap
from matplotlib.patches import Patch
from matplotlib import rcParams
rcParams['pdf.fonttype'] = 42
rcParams['ps.fonttype'] = 42
rcParams['font.sans-serif'] = "Arial"
rcParams['font.family'] = "Arial"

import argparse

parser = argparse.ArgumentParser(description='Visualization of oncostratifier_by_drugs')
parser.add_argument('--main_gene', '-mg', metavar='the-main-gene', dest='main_gene', type=str,
                    help='Choose the main gene', default=None)
parser.add_argument('--cancer', '-c', metavar='the-cancer', dest='cancer', type=str,
                    help='Choose cancer', default=None)
parser.add_argument('--main_division', '-md', metavar='the-main-division', dest='main_division', type=str,
                    help='Choose the first level of division: MutDep or Mut', 
                    choices=['MutDep', 'Mut', ''], default='Mut')
parser.add_argument('--visual_type', '-vt', metavar='the-visual-type', dest='visual_type', type=str,
                    help='Choose the visualization', 
                    choices=['circular_bar', 'violin', 'violin_facet', 'target_match_distribution','significant_drug_distribution', 'differential_mutation',
                             'cellline_scatter_with_drug_sensitivity', 'drug_cellline_heatmap', 'response_with_mutational_signature'], 
                    default='target_match_distribution')
parser.add_argument('--version', '-ve', metavar='the-version', dest='version', type=int,
                    help='Choose the version of the visualization', default=6)
parser.add_argument('--seed', '-s', metavar='the-seed', dest='seed', type=int,
                    help='Choose the seed of the visualization', default=123)
parser.add_argument('--variant', '-va', metavar='the-variant-version', dest='variant_version', type=str,
                    help='Choose variant version', default='cmp_2022')
#parser.add_argument('--variant', '-v', metavar='the-variant-version', dest='variant', type=str,
#                    help='Choose variant version', default='2022')
args = parser.parse_args()
print(f'Running args:{args}')

R_STATE = args.seed
rng = np.random.default_rng(R_STATE)

#COLORS
GROUP_NAMES = ['Stratifying Sensitivity', 'Stratifying Resistance', 'Full Sensitivity', 'Full Resistance']
RESPONSE_NAMES = ['Sensitive', 'Resistant']
ONCOGENE_STATUS = ['Mut', 'WT']
VIBRANT_4_COLORS = ['#EE7733', '#33BBEE', '#EE3377', '#CC3311']
MAT_4_COLORS = ['#332288', '#117733', '#999933', '#882255']
VIBRANT_2_COLORS = ['#009988', '#EE7733']
VIBRANT_BR_COLORS = ['#004488', '#CC3311']
GROUP2VIBRANTCOLOR = dict(zip(GROUP_NAMES,VIBRANT_4_COLORS))
GROUP2MATCOLOR = dict(zip(GROUP_NAMES,MAT_4_COLORS))
RESPONSE2VIBRANTCOLOR = dict(zip(RESPONSE_NAMES,VIBRANT_2_COLORS))
ONCOGENESTATUS2COLOR = dict(zip(ONCOGENE_STATUS,VIBRANT_BR_COLORS))
#RESPONSE2MATCOLOR = dict(zip(GROUP_NAMES,MAT_4_COLORS))

#Set the experiment folder
if len(args.main_division)>0:
    experiment_folder_name = f'oncostratifier_by_drug_{args.main_division.lower()}'
else:
    experiment_folder_name = f'oncostratifier_by_drug'
ONCO_BY_DRUG_FOLDER = config.FHT_RESULTS_DIR / experiment_folder_name

#Load drug database, check if there is a cached version
gdsc_cache_name = f'combined_gdsc_27Oct23.xz'
gdsc_cache_dict = config.cache_numpy(cache_name=gdsc_cache_name, is_pickle=True)
if gdsc_cache_dict is None:
    print(f'GDSC load started!')
    gdsc1 = process_drug.get_drug(gdsc_version=1)
    gdsc2 = process_drug.get_drug(gdsc_version=2)
    gdsc_cache_dict = {'gdsc': process_drug.clear_combine_sources([gdsc1, gdsc2])}
    config.cache_numpy(cache_name=gdsc_cache_name, data=gdsc_cache_dict, is_pickle=True)
gdsc = gdsc_cache_dict['gdsc']

genes, cancers = None, None
if args.main_gene is not None:
    genes = [args.main_gene]
if args.cancer is not None:
    cancers = [args.cancer]
    
#Load screening results and find unique genes and cancer types.
if cancers is None or genes is None:
    all_res = onco_res.get_result(folder_name=experiment_folder_name, return_item_names_only=True)
if cancers is None:
    cancers = all_res['cancers']#np.unique(all_res['cancer'].values)
if genes is None:
    genes = all_res['genes']#np.unique(all_res['screenGene'].values)

V=args.version
RE_D = re.compile('\d')
category_name_fix = {'stratifying essentiality': 'Stratifying Sensitivity', 'stratifying non-essentiality': 'Stratifying Resistance',
                        'co-dependency':'Full Sensitivity', 'exclusive-dependency':'Full Resistance'}


def add_target_columns(df, only_targets=False):
    #Load targets into the dataframe
    if only_targets:
        df = df[df['target_hit']==True]
    label_add = '_target'
    list_of_labels = ['name_for_cancer', 'name_for_cancer_w_drug_name_version', 'name_for_cancer_w_drug_name', 'name_for_cancer_w_drug_id_name', 
                    'name_for_gene', 'name_for_gene_w_drug_name_version', 'name_for_gene_w_drug_name', 'name_for_gene_w_drug_id_name',
                    'candidateDrug', 'drug_name_version', 'drug_name', 'drug_id_name', 'name', 'name_w_drug_name', 'name_for_cancer_w_drug_id_name',
                    'name_for_cancer_w_drug_name_version']
    for name_col in list_of_labels:
        df[f'{name_col}{label_add}'] = df[name_col] + ' (' + df['targets'] + ')'
    return df

def create_drug_df_for_violin(res, main_division):
    #Create a new dataframe for the violin plots
    res_drug_list = []
    #Loop over results (Per drug)
    for row_id, row in res.iterrows():
        #If drug version is used remove it from the drug name on the plot.
        drug = f"{row['drug_id']}\n{row['drug_name']}\n{row['drug_version']}"
        
        #Annotate
        full_drug = row['candidateDrug']
        gdsc_drug_cellline = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])
        mutdep, others = row['mutDep_with_response_list'].split(','), row['others_with_response_list'].split(',')
        all_cell_lines = np.concatenate([mutdep, others], axis=0)
        mutdep_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in mutdep])
        others_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in others])
        all_lnmaxconc = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_MAX_CONC'] for celline in all_cell_lines])
        res_drug_list.append(pd.DataFrame({
            'Cancer': row['cancer'], 'ScreenGene': row['screenGene'], 'Cell Lines': np.concatenate([mutdep, others],axis=0),
            'LN_IC50': np.concatenate([mutdep_lnic50, others_lnic50],axis=0),
            'Candidate Gene Status': [main_division]*len(mutdep)+['Others']*len(others),
            'Drug': [drug]*len(all_cell_lines), 'Threshold': all_lnmaxconc, 'DiffEntropy': row['diffEntropy'],
            'P Value': row['p'], 'MutDep Entropy': row['mutDep_entropy'], 'Others Entropy': row['others_entropy'],
            'Category': row['category'],
        }))
    if len(res_drug_list)>0:
        return pd.concat(res_drug_list)
    else:
        return None

def get_label_rotation(angle, offset):
    # Rotation must be specified in degrees :(
    rotation = np.rad2deg(angle + offset)
    if rotation%360<270 and rotation%360>90:
        alignment = "right"
        rotation = rotation +180
    else: 
        alignment = "left"
        rotation = rotation
    return rotation, alignment

def add_labels(angles, values, labels, colors, offset, ax):
    # This is the space between the end of the bar and the label
    padding = 0.05
    
    # Iterate over angles, values, and labels, to add all of them.
    for angle, value, label, color in zip(angles, values, labels, colors):
        angle = angle
        
        # Obtain text rotation and alignment
        rotation, alignment = get_label_rotation(angle, offset)

        # And finally add the text
        if value>=0:
            ax.text(
                x=angle, 
                y=value + padding, 
                s=f'{label}', 
                ha=alignment, 
                va="center", 
                rotation=rotation, 
                rotation_mode="anchor",
                fontsize=10, 
                color=color
            )
        else:
            ax.text(
                x=angle, 
                y=padding, 
                s=f'{label}', 
                ha=alignment, 
                va="center", 
                rotation=rotation, 
                rotation_mode="anchor",
                fontsize=10, 
                color=color
            )

def circular_bar(df, sorted=False, labels_name='name', center_label = '', out_loc='trial.pdf'):
    if out_loc.exists():
        print('Recreating')
    plt.clf()
    # Reorder the dataframe
    if sorted:
        df = (
            df.groupby(["category"]) 
            .apply(lambda x: x.sort_values(["diffEntropy"], ascending = False, key=abs)).reset_index(drop=True)
        )

    VALUES = df["diffEntropy"].values
    LABELS = df[labels_name].values
    LABEL_COLORS = df["target_hit"].map({False:'black', True:'red'}).values
    GROUP = df["category"].values

    PAD = 2
    ANGLES_N = len(VALUES) + PAD * len(np.unique(GROUP))
    ANGLES = np.linspace(0, 2 * np.pi, num=ANGLES_N, endpoint=False)
    WIDTH = (2 * np.pi) / len(ANGLES)

    GROUPS_SIZE = [len(i[1]) for i in df.groupby("category")]
    GROUPS_NAME = [i[0] for i in df.groupby("category")]

    offset = 0
    IDXS = []
    for size in GROUPS_SIZE:
        IDXS += list(range(offset + PAD, offset + size + PAD))
        offset += size + PAD

    fig, ax = plt.subplots(figsize=(10, 10), subplot_kw={"projection": "polar"})
    ax.set_theta_offset(offset)
    ax.set_ylim(-1.8, 1.5)
    ax.set_frame_on(False)
    ax.xaxis.grid(False)
    ax.yaxis.grid(False)
    ax.set_xticks([])
    ax.set_yticks([])
    ax.tick_params(grid_color="palegoldenrod")

    GROUPS_SIZE = [len(i[1]) for i in df.groupby("category")]
    COLORS = [GROUP2MATCOLOR[name] for i, (size, name) in enumerate(zip(GROUPS_SIZE, GROUPS_NAME)) for _ in range(size)]
    #COLORS = [GROUP2MATCOLOR[whichgroup] for whichgroup in GROUPS_NAME]

    ax.bar(
        ANGLES[IDXS], VALUES, width=WIDTH, color=COLORS, 
        edgecolor="white", linewidth=2
    )

    add_labels(ANGLES[IDXS], VALUES, LABELS, LABEL_COLORS, offset, ax)

    # Extra customization below here --------------------

    # This iterates over the sizes of the groups adding reference
    # lines and annotations.

    offset = 0 
    ANGLE_BETWEEN_SIZE = (ANGLES[1]-ANGLES[0])/3
    for group_id, (group, size) in enumerate(zip(GROUPS_NAME, GROUPS_SIZE)):
        group_mask = GROUP==group
        sorting_order_for_group = np.argsort(VALUES[group_mask])
        if size%2==0:
            medians = sorting_order_for_group[[int(size/2-1),int(size/2)]]
        else:
            medians = sorting_order_for_group[[int((size-1)/2)]]
        max_size_string = max(len(ele) for ele in LABELS[group_mask][medians])
        max_val = max(0, max(ele for ele in VALUES[group_mask][medians]))
        # Add line below bars
        x1 = np.linspace(ANGLES[offset + PAD]-ANGLE_BETWEEN_SIZE, ANGLES[offset + size + PAD - 1]+ANGLE_BETWEEN_SIZE, num=50)
        ax.plot(x1, [0] * 50, color="#333333")
        
        # Add text to indicate group and reference lines
        #x2 = np.linspace(ANGLES[offset]-ANGLE_BETWEEN_SIZE, ANGLES[offset + PAD - 1]+ANGLE_BETWEEN_SIZE, num=50)
        if 'stratifying' in group.lower():
            ax.text(
                np.mean(x1), max_val*0.9+max_size_string*0.08, group.replace(' ','\n'), color="#333333", fontsize=12, 
                fontweight="bold", ha='center', va="center", rotation=0#group_label_rotation+90
            )
        else:
            ax.text(
                np.mean(x1), max_val+max_size_string*0.08, group.replace(' ','\n'), color="#333333", fontsize=12, 
                fontweight="bold", ha='center', va="center", rotation=0#group_label_rotation+90
            )
        x2 = np.linspace(ANGLES[offset+PAD]+ANGLE_BETWEEN_SIZE, ANGLES[offset]-ANGLE_BETWEEN_SIZE, num=50)
        if 'stratifying' in GROUPS_NAME[group_id].lower():
            for ref_value in np.arange(0.0, 1.1, 0.2):
                #ax.plot(x2[:23], [ref_value] * 23, color="#bebebe", lw=0.8)
                #ax.plot(x2[:], [ref_value] * 50, color="#bebebe", lw=0.8)
                ax.text(ANGLES[offset+PAD-1]+ANGLE_BETWEEN_SIZE, ref_value, f'{ref_value:.1f}', color="black", ha='center', va="center", 
                        rotation=0, fontsize=10)
        else:
            for ref_value in np.arange(-1.0, 0.1, 0.2):
                #ax.plot(x2[:23], [ref_value] * 23, color="#bebebe", lw=0.8)
                #ax.plot(x2[:], [ref_value] * 50, color="#bebebe", lw=0.8)
                ax.text(ANGLES[offset+PAD-1]+ANGLE_BETWEEN_SIZE/5, ref_value, f'{ref_value:.1f}', color="black", ha='center', va="center", 
                        rotation=0, fontsize=10)#np.rad2deg(x2[24])+70


        offset += size + PAD
    ax.text(
        0, -1.8, center_label.replace(' ','\n'), color="#333333", fontsize=12,
        fontweight="bold", ha='center', va="center", rotation=0#group_label_rotation+90
        )
    
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    del df


def violin_distribution(df, label, out_loc):
    plt.clf()
    violin = sns.violinplot(data=df, x="Drug", y="LN_IC50", hue="Candidate Gene Status", split=True, gap=.1, inner="point", cut=0, bw_adjust=0.3#, alpha=0.3,
                            )#inner_kws=dict(color="black", edgecolor='black'))
    ax = violin.axes
    for one_violin in violin.collections[::2]:
        one_violin.set_facecolor(to_rgba(one_violin.get_facecolor(), alpha=0.3))
    grouped_by_drug = df.groupby(['Drug'])[['DiffEntropy', 'Threshold', 'Category', 'LN_IC50', 'P Value']].agg(lambda x:x.value_counts().index[0])
    xticks = ax.get_xticks()
    xtick_locs = np.linspace(0,1,len(xticks)*2+1)
    category_name_fix = {'stratifying essentiality': 'Stratifying\nSen.', 'stratifying non-essentiality': 'Stratifying\nRes.',
                         'co-dependency':'Sensitive\nonMut', 'exclusive-dependency':'Resistant\nonMut'}
    for tick_id, aticklabel in enumerate(ax.get_xticklabels()):
        drug= aticklabel.get_text()
        x_pos = aticklabel.get_position()[0]
        category_name = category_name_fix[grouped_by_drug.loc[drug, "Category"]]
        
        ax.text(x_pos, ax.get_ylim()[1], f'{category_name}\nDE: {grouped_by_drug.loc[drug, "DiffEntropy"]:.2f}\nP: {grouped_by_drug.loc[drug, "P Value"]:.1e}',
            horizontalalignment='center', verticalalignment='bottom', fontsize=8
        )
        ax.axhline(y=grouped_by_drug.loc[drug, "Threshold"], 
                   xmin=xtick_locs[tick_id*2+1]-1/(len(xticks)*4), xmax=xtick_locs[tick_id*2+1]+1/(len(xticks)*4), 
                   ls='--', c='red'
        )
        
    # Add max concentration as a legend
    handles, labels = ax.get_legend_handles_labels()
    maxconc_line = Line2D([0], [0], label='Max Conc.', ls='--', c='red')
    handles.append(maxconc_line) 
    plt.legend(handles=handles, bbox_to_anchor=(1, 1), loc='upper left', borderaxespad=0, frameon=False)
    
    # Remove top and right axis, put title to the top, and save figure.
    sns.despine()
    plt.title(label, x=0.5, y=1.15, fontsize=10)
    plt.ylabel('LN(IC50)')
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    print()
    

def violin_distribution_facet(df, label, out_loc):
    if out_loc.exists():
        print('Recreating')#return 1
    plt.clf()
    category2col = {'stratifying essentiality': 'Stratifying', 'stratifying non-essentiality': 'Stratifying',
                         'co-dependency':'Full', 'exclusive-dependency':'Full'}
    category2row = {'stratifying essentiality': 'Sensitivity', 'stratifying non-essentiality': 'Resistance',
                         'co-dependency':'Sensitivity', 'exclusive-dependency':'Resistance'}
    df['Category Col'] = df['Category'].str.split(' ', expand=True)[0] #df['Category'].replace(category2col)
    df['Category Row'] = df['Category'].str.split(' ', expand=True)[1] #df['Category'].replace(category2row)
    col_order = [acol for acol in ['Stratifying', 'Full'] if acol in df['Category Col'].unique()]
    row_order = [arow for arow in ['Sensitivity', 'Resistance'] if arow in df['Category Row'].unique()]
    xranges = df.groupby(['Category Col', 'Category Row'])['Drug'].nunique().reset_index().groupby(['Category Col'])['Drug'].max().loc[col_order]*1.1
    #color_palette = {'Mut':'#ef8a62', 'MutDep':'#ef8a62', 'Others':'#67a9cf'}
    color_palette = {'Mut':ONCOGENESTATUS2COLOR['Mut'], 'MutDep':ONCOGENESTATUS2COLOR['Mut'], 'Others':ONCOGENESTATUS2COLOR['WT']}
    g = sns.FacetGrid(df, col="Category Col", row="Category Row", height=3, aspect=0.7*np.log2(max(xranges.values)+2), 
                      col_order=col_order, row_order=row_order, palette = color_palette,
                      sharex=False, sharey=False, gridspec_kws=dict(width_ratios=xranges))
    g.map_dataframe(sns.violinplot, x="Drug", y="LN_IC50", hue="Candidate Gene Status", palette = color_palette, split=True, gap=.1, inner="point", cut=0, bw_adjust=0.3)
    #g.map_dataframe(annotate)
    
    grouped_by_drug = df.groupby(['Drug'])[['DiffEntropy', 'Threshold', 'Category', 'LN_IC50', 'P Value']].agg(lambda x:x.value_counts().index[0])
    #ax = violin.axes
    for ax in g.axes.flatten():
        if len(ax.collections)==0:
            ax.set_visible(False)
            continue
        for one_violin in ax.collections[::2]:
            one_violin.set_facecolor(to_rgba(one_violin.get_facecolor(), alpha=0.3))
        xticks = ax.get_xticks()
        xtick_locs = np.linspace(0,1,len(xticks)*2+1)
        for tick_id, aticklabel in enumerate(ax.get_xticklabels()):
            drug= aticklabel.get_text()
            x_pos = aticklabel.get_position()[0]
            #category_name = category_name_fix[grouped_by_drug.loc[drug, "Category"]]
            
            ax.text(x_pos, ax.get_ylim()[1], f'DE: {grouped_by_drug.loc[drug, "DiffEntropy"]:.2f}\nP: {grouped_by_drug.loc[drug, "P Value"]:.1e}',
                horizontalalignment='center', verticalalignment='center', fontsize=8
            )
            ax.axhline(y=grouped_by_drug.loc[drug, "Threshold"], 
                    xmin=xtick_locs[tick_id*2+1]-1/(len(xticks)*4), xmax=xtick_locs[tick_id*2+1]+1/(len(xticks)*4), 
                    ls='--', c='red'
            )
    
    # Add max concentration as a legend
    g._legend_data['Max Conc.'] = Line2D([0], [0], label='Max Conc.', ls='--', c='red')
    g.add_legend(frameon=False)
    g.tight_layout()
    g.set_titles('{col_name} {row_name}', va='bottom', y=1.05, fontsize=10)
    g.set_axis_labels(x_var='Drug', y_var='LN(IC50)', weight='bold')
    g.fig.subplots_adjust(top=1.2)
    g.fig.suptitle(label, va='bottom', y=1.35, fontsize=12, weight='bold')#, x=0.5, y=1.15, fontsize=10)
    g.despine()
    # Remove top and right axis, put title to the top, and save figure.
    #sns.despine()
    #plt.title(label, x=0.5, y=1.15, fontsize=10)
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    

def swarm_per_group(plot_df, order=None, label='',  out_loc=''):
    plt.clf()    
    width_size = 8
    height_ratio = 0.8
    adjuster = width_size/4.8
    fig, ax = plt.subplots(figsize=(width_size, width_size*height_ratio))#plt.subplots(figsize=(6, max_yticks/2.0))
    hue_order = ['Resistant', 'Sensitive']
    violin = sns.swarmplot(data=plot_df, x="Group", y="LN(IC50)", hue="Status", hue_order=hue_order, palette=RESPONSE2VIBRANTCOLOR, order=order, ax=ax) 
    ax = violin.axes   
    # Remove top and right axis, put title to the top, and save figure.
    sns.despine()
    plt.title(label, x=0.6, y=1, fontsize=15*adjuster, fontweight='bold', ma='left', ha='center')
    plt.xlabel(ax.get_xlabel(), fontsize=14*adjuster, fontweight='bold')
    plt.ylabel(ax.get_ylabel(), fontsize=14*adjuster, fontweight='bold')
    ax.tick_params(axis='x', which='major', labelsize=11*adjuster)
    ax.tick_params(axis='y', which='major', labelsize=11*adjuster*height_ratio)
    if len(order)>=4:
        xlabel_yloc = ax.transData.inverted().transform(ax.xaxis.get_label().get_position())[1]
        wt_xloc = np.mean(ax.get_xticks()[:2])
        mut_xloc = np.mean(ax.get_xticks()[-2:])
        plt.text(wt_xloc, xlabel_yloc, 'Oncogene-WT', ha='center', va='top', fontsize=12*adjuster, fontweight='bold')
        plt.text(mut_xloc, xlabel_yloc, 'Oncogene-Mut', ha='center', va='top', fontsize=12*adjuster, fontweight='bold')
    plt.legend(fontsize = 12*adjuster, frameon=True, title=None, handletextpad=0.1, loc=9)
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    

def swarm_per_groupv2(plot_df, order=None, col_order=None, label='',  out_loc=''):
    plt.clf()    
    width_size = 5
    height_ratio = 1.4
    adjuster = width_size/4.8
    #fig, ax = plt.subplots(figsize=(width_size, width_size*height_ratio))#plt.subplots(figsize=(6, max_yticks/2.0))
    hue_order = ['Resistant', 'Sensitive']
    g = sns.catplot(data=plot_df, kind="swarm", x="MarkerGeneStatus", y="LN(IC50)", hue="Status", col="ScreenGeneStatus",
    hue_order=hue_order, palette=RESPONSE2VIBRANTCOLOR, size=6, zorder=1,
    height=width_size/2*height_ratio, aspect=1/height_ratio, order=order, col_order=col_order)
    g.figure.subplots_adjust(wspace=0.2)
    #violin = sns.swarmplot(data=plot_df, x="Group", y="LN(IC50)", hue="Status", hue_order=hue_order, palette=RESPONSE2VIBRANTCOLOR, order=order, ax=ax) 
    #ax = violin.axes   
    # Remove top and right axis, put title to the top, and save figure.
    sns.despine()
    #g.fig.subplots_adjust(top=0.9)
    for col_num, ax in enumerate(g.axes.flat):
        ax.set_xlabel('Marker Gene', fontsize=14*adjuster, fontweight='bold')
        ax.set_ylabel(ax.get_ylabel(), fontsize=14*adjuster, fontweight='bold')
        if col_num != 0:
            ax.spines['left'].set_visible(False)
            ax.get_yaxis().set_visible(False)
    #g.set_axis_labels(fontsize=14*adjuster, fontweight='bold')
    g.tick_params(axis='x', labelsize=12*adjuster)
    g.tick_params(axis='y', labelsize=12*adjuster)
    sns.move_legend(g, fontsize = 11*adjuster, frameon=True, title=None, handletextpad=0.1, loc='center', bbox_to_anchor = (0.47, 0.85), alignment='left', borderpad=0.1, handlelength=0.6)
    g.fig.suptitle(label, x=0.55, y=1, fontsize=14*adjuster, fontweight='bold', ma='left', ha='center', va='bottom')
    g.set_titles(col_template = '{col_name}', y=0.95, size=14*adjuster, fontweight='bold')
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    


def swarm_one_vs_rest(plot_df, order=None, label='',  out_loc=''):
    plt.clf()    
    main_group = plot_df[plot_df['Group'] == order[-1]]['LN(IC50)'].values
    rest_group = plot_df[plot_df['Group'] == order[0]]['LN(IC50)'].values
    less_stats, less_p = mannwhitneyu(main_group, rest_group, alternative="less", method='exact') #Alternative: main is less than rest
    gr_stats, gr_p = mannwhitneyu(main_group, rest_group, alternative="greater", method='exact') #Alternative: main is less than rest
        
    if np.mean(main_group)<np.mean(rest_group):
        stats, p = less_stats, less_p
    elif np.mean(main_group)>np.mean(rest_group):
        stats, p = gr_stats, gr_p
    
    width_size = 2.5
    height_ratio = 1.4
    adjuster = width_size/2.4
    fig, ax = plt.subplots(figsize=(width_size, width_size*height_ratio))#plt.subplots(figsize=(6, max_yticks/2.0))
    hue_order = ['Resistant', 'Sensitive']
    violin = sns.swarmplot(data=plot_df, x="Group", y="LN(IC50)", hue="Status", hue_order=hue_order, palette=RESPONSE2VIBRANTCOLOR, order=order, ax=ax) 
    ax = violin.axes   
    
    max_main = np.max(main_group)
    max_rest = np.max(rest_group)
    max_all = np.max([max_main, max_rest])
    ax.hlines(max_all+0.6, 0, 1, color='black')
    ax.vlines(0, max_rest+0.15, max_all+0.6, color='black')
    ax.vlines(1, max_main+0.15, max_all+0.6, color='black')
    ax.text(0.5, max_all+0.5, f'P: {p:.2e}', va='top', ha='center')
    # Remove top and right axis, put title to the top, and save figure.
    sns.despine()
    plt.title(label, x=0.5, y=1.0, fontsize=14*adjuster, fontweight='bold', ma='left', ha='center', va='bottom')
    plt.xlabel(ax.get_xlabel(), fontsize=14*adjuster, fontweight='bold')
    plt.ylabel(ax.get_ylabel(), fontsize=14*adjuster, fontweight='bold')
    ax.tick_params(axis='x', which='major', labelsize=12*adjuster)
    ax.tick_params(axis='y', which='major', labelsize=12*adjuster)
    plt.legend(fontsize = 11*adjuster, frameon=True, title=None, handletextpad=0.1, loc='center')#, bbox_to_anchor = (1, 1))
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')


def swarm_all_groups(plot_df, order=None, label='', out_loc=''):
    plt.clf()    
    hue_order = ['Resistant', 'Sensitive']
    colors = ['#1b9e77', '#d95f02']
    # Set your custom color palette
    sns.set_palette(sns.color_palette(colors))
    if V==3:
        g = sns.catplot(kind='strip', x='ScreenGeneStatus', y='LN(IC50)', hue='Status', col='HIT', hue_order=hue_order, row='Cancer',# style="MarkerGeneStatus",
                    dodge=False, palette=sns.color_palette(colors), order=order, marker='o', # sharex=False,
                    height=4, aspect=0.4, data=plot_df[plot_df['MarkerGeneStatus']=='WT'], margin_titles=True)
        g.set_titles(col_template = '{col_name}', row_template= '{row_name}')
    elif V==4:
        g = sns.catplot(kind='strip', x='ScreenGeneStatus', y='LN(IC50)', hue='Status', col='HIT_with_cancer', hue_order=hue_order, col_wrap=10, #row='Cancer',# style="MarkerGeneStatus",
                    dodge=False, palette=sns.color_palette(colors), order=order, marker='o',# sharex=False,
                    height=4, aspect=0.4, data=plot_df[plot_df['MarkerGeneStatus']=='WT'], margin_titles=False)
        g.set_titles(col_template = '{col_name}')
    for titles, ax in g.axes_dict.items():#.flat:
        if V==3:
            row_title, col_title = titles
            ax_df = plot_df[(plot_df['HIT']==col_title) & (plot_df['Cancer']==row_title)]
            title_of_plot = col_title
        elif V==4:
            ax_df = plot_df[plot_df['HIT_with_cancer']==titles]
            title_of_plot = titles
        ax.tick_params(labelbottom=True)
        # remove y-axis except for the left-most columns
        if len(ax.get_ylabel()) == 0:
            ax.spines['left'].set_alpha(0.2)#set_visible(False)
            ax.tick_params(axis='y', left=False)
        #row_title, col_title = ax.get_title().split(' | ')
        g2 = sns.stripplot(x='ScreenGeneStatus', y='LN(IC50)', hue='Status', hue_order=hue_order,
                    dodge=False, palette=sns.color_palette(colors), order=['WT', 'Mut'], marker='X', data=ax_df[ax_df['MarkerGeneStatus']=='Mut'], ax=ax)
        # use title as x-label
        colored_title = title_of_plot#r'' 
        #for a_word, a_color in zip(title_of_plot.split('\n'), ['red', 'orange', 'brown', 'green']):
        #    colored_title += r'\textcolor{'+a_color+r'}{'+a_word+r'}\n'
        #colored_title = colored_title[:-2]
        ax.set_xlabel(colored_title)
        ax.set_title('')#row_title.replace(' ', '\n'))
        if ax.get_legend() is not None:
            ax.get_legend().remove()
    plt.subplots_adjust(wspace=0.1, hspace=0.4)
    #g.despine(left=True)
    #sns.despine()
    g.fig.legends = []
    legendhandles_response = []
    for hue_name, color in zip(hue_order, colors):
        legendhandles_response.append(Line2D([], [], color=color, marker='o', linestyle='None',
                            markersize=10, label=f'{hue_name}'))
    g.fig.legend(handles = legendhandles_response, title='Response', frameon=False, loc='upper left', bbox_to_anchor=(0.95, 0.40))
    legendhandles_markerstatus = []
    for markerstatus, marker_style in zip(['WT', 'Mut'], ['o', 'X']):
        legendhandles_markerstatus.append(Line2D([], [], color='gray', marker=marker_style, linestyle='None',
                            markersize=10, label=f'{markerstatus}'))
    g.fig.legend(handles = legendhandles_markerstatus, title='Marker Gene', frameon=False, loc='upper left', bbox_to_anchor=(0.95, 0.60))
    #g.fig.suptitle(label)
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')


def swarm_all_groups_mt(plot_df, order=None, label='', out_loc=''):
    plt.clf()    
    fig, ax = plt.subplots(figsize=(max_yticks/2.0, height))#plt.subplots(figsize=(6, max_yticks/2.0))
    hue_order = ['Resistant', 'Sensitive']
    colors = ['#1b9e77', '#d95f02']
    # Set your custom color palette
    sns.set_palette(sns.color_palette(colors))
    g1 = sns.catplot(kind='strip', x='ScreenGeneStatus', y='LN(IC50)', hue='LN_SENSITIVE', col='HIT',# style="MarkerGeneStatus",
                dodge=False, palette=sns.color_palette(colors), order=['WT', 'Mut'], marker='o',
                height=5, aspect=0.5, data=plot_df[plot_df['MarkerGeneStatus']=='Mut'])
    g2 = sns.catplot(kind='strip', x='ScreenGeneStatus', y='LN(IC50)', hue='LN_SENSITIVE', col='HIT',# style="MarkerGeneStatus",
                dodge=False, palette=sns.color_palette(colors), order=['WT', 'Mut'], marker='X',
                height=5, aspect=0.5, data=plot_df[plot_df['MarkerGeneStatus']=='WT'])
    for ax in g2.axes.flat:
        # use title as x-label
        ax.set_xlabel(ax.get_title())
        ax.set_title('')
        # remove y-axis except for the left-most columns
        if len(ax.get_ylabel()) == 0:
            ax.spines['left'].set_visible(False)
            ax.tick_params(axis='y', left=False)
    plt.subplots_adjust(wspace=0)
    #g.despine(left=True)
    #sns.despine()
    plt.title(label, x=0.5, y=1.15, fontsize=10)
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')


def histogram_bar(df, label, out_loc):
    if out_loc.exists():
        print('Recreating')#return 1
    plt.clf()
    color_dict = {key.replace(' ', '\n'): item for key,item in GROUP2MATCOLOR.items()}
    order = list(color_dict.keys())
    category2id = {category_name: category_id for category_id, category_name in enumerate(order)}
    df['category'] = df['category'].str.replace(' ', '\n')
    df['category_id'] = df['category'].map(category2id)
    max_size = df.groupby('category_id').size().max()
    g = sns.histplot(df, x='category_id', hue='category', discrete=True, shrink=.8, palette=color_dict, hue_order=order)
    ax = g.axes
    # add annotations
    for c in ax.containers:
        # custom label calculates percent and add an empty string so 0 value bars don't have a number
        labels = [f'{w:.2f}%' if (w := 100.0*v.get_height()/len(df)) > 0 else '' for v in c]
        ax.bar_label(c, labels=labels, label_type='edge', fontsize=14, color='white', fontweight='bold', rotation=0, padding=-20)
    
    ax.margins(y=0.2)
    sns.despine()
    plt.title(label, x=0.5, y=1.05, fontsize=16)
    plt.ylabel('Count', fontsize=16)
    plt.xlabel('Category', fontsize=16)
    ax.set_xticks(np.arange(len(order)), order)
    ax.tick_params(axis='both', labelsize=14)
    ax.get_legend().remove()
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    

def facet_histogram_bar(df, y, label, out_loc):
    if out_loc.exists():
        print('Recreating')#return 1
    plt.clf()
    
    df['Category Col'] = df['category'].str.split(' ', expand=True)[0] #df['Category'].replace(category2col)
    df['Category Row'] = df['category'].str.split(' ', expand=True)[1] #df['Category'].replace(category2row)
    col_order = [acol for acol in ['Stratifying', 'Full'] if acol in df['Category Col'].unique()]
    row_order = [arow for arow in ['Sensitivity', 'Resistance'] if arow in df['Category Row'].unique()]
    #xranges = df.groupby(['Category Col', 'Category Row'])[].nunique().reset_index().groupby(['Category Col'])['Drug'].max().loc[col_order]*1.1
    color_palette = {'Mut':'#ef8a62', 'MutDep':'#ef8a62', 'Others':'#67a9cf'}
    g = sns.FacetGrid(df, col="Category Col", row="Category Row", height=5, aspect=2,
                      col_order=col_order, row_order=row_order, palette = color_palette,
                      sharex=False, sharey=False, gridspec_kws={"wspace":1.5, "hspace":0.3})#, gridspec_kws=dict(height_ratios=xranges))
    def sorted_bar(y, *args, **kwargs):
        data = kwargs.pop('data')
        count_data = data[y].value_counts().sort_values(ascending=False)[0:20].reset_index()
        sns.barplot(count_data, y=y, x='count', **kwargs);   
    g.map_dataframe(sorted_bar, y=y)
    g.set_titles('{col_name} {row_name}', fontsize=10)
    g.set_axis_labels(x_var='Count', y_var='Drug', weight='bold')
    #g.fig.subplots_adjust(top=1.2)
    g.fig.suptitle(label, va='bottom', y=1, fontsize=12, weight='bold')#, x=0.5, y=1.15, fontsize=10)
    g.despine()
    g.tight_layout()
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')


def scatter_pie(df, xs=['mutDep', 'others'], y='with_response_list', hue='drug_name_target', label='', gene='', include_resistance=False, out_loc='scatter_pie.png'):
    colors = ['#e41a1c','#377eb8','#4daf4a','#984ea3','#ff7f00','#ffff33','#a65628','#f781bf','#999999']
    #df = df.sort_values(by='diffEntropy', key=abs, ascending=False)
    all_drugs = df['candidateDrug_target'].values[:min(10, len(df))]
    drug2color = dict(zip(all_drugs, colors))
    all_target_ccles, mutDep_target_ccles, others_target_ccles = [], [], []
    for with_resp_list in df['all_with_response_list'].values: all_target_ccles+=with_resp_list.split(',')
    for with_resp_list in df['mutDep_with_response_list'].values: mutDep_target_ccles+=with_resp_list.split(',')
    for with_resp_list in df['others_with_response_list'].values: others_target_ccles+=with_resp_list.split(',')
    ccle_list_dict = {'ccles_in_mutDep': np.unique(mutDep_target_ccles),
                      'ccles_in_others': np.unique(others_target_ccles),
                      'ccles_in_all': np.unique(all_target_ccles)}
    max_yticks = max(len(ccle_list_dict['ccles_in_mutDep']), len(ccle_list_dict['ccles_in_others']))
    
    #Sort celllines according to the hit to drugs
    for xlabel in xs:
        hit_sizes = [sum(df[f'{xlabel}_sensitive_list'].str.contains(accle)==True) for accle in ccle_list_dict[f'ccles_in_{xlabel}']]
        ccle_list_dict[f'ccles_in_{xlabel}'] = ccle_list_dict[f'ccles_in_{xlabel}'][np.argsort(hit_sizes)[::-1]]
    
    #plt.clf()
    if include_resistance:
        height=6
        between_group_mult = 3
    else:
        height=4
        between_group_mult = 1
    fig, ax = plt.subplots(figsize=(max_yticks/2.0, height))#plt.subplots(figsize=(6, max_yticks/2.0))
    for x_idx, xlabel in enumerate(xs):
        for cellline_idx, cellline in enumerate(ccle_list_dict[f'ccles_in_{xlabel}']):
            #Draw sensitive cell lines, first an empty circle then colors with sensitive drugs
            sensitive_drugs = df[df[f'{xlabel}_sensitive_list'].str.contains(cellline)==True]['candidateDrug_target']
            x_none = np.cos(2 * np.pi * np.linspace(0, 1))
            y_none = np.sin(2 * np.pi * np.linspace(0, 1))
            xy_none = np.row_stack([[0, 0], np.column_stack([x_none, y_none])])
            ax.plot(cellline_idx, x_idx*between_group_mult, marker=xy_none,
                ms=20, markerfacecolor="None", markeredgecolor='black', linestyle='-')
            if len(sensitive_drugs)>0:
                lin_ratio = 1.0/len(sensitive_drugs)
                for a_drug_idx, a_drug in enumerate(sensitive_drugs):
                    x1 = np.cos(2 * np.pi * np.linspace(lin_ratio*a_drug_idx, lin_ratio*(a_drug_idx+1)))
                    y1 = np.sin(2 * np.pi * np.linspace(lin_ratio*a_drug_idx, lin_ratio*(a_drug_idx+1)))
                    xy1 = np.row_stack([[0, 0], np.column_stack([x1, y1])])
                    ax.plot(cellline_idx, x_idx*between_group_mult, marker=xy1,
                        ms=20, markerfacecolor=drug2color[a_drug], markeredgecolor='None', linestyle='None')
            
            resistant_drugs = df[df[f'{xlabel}_resistant_list'].str.contains(cellline)==True]['candidateDrug_target']
            x_none = np.cos(2 * np.pi * np.linspace(0, 1))
            y_none = np.sin(2 * np.pi * np.linspace(0, 1))
            xy_none = np.row_stack([[0, 0], np.column_stack([x_none, y_none])])
            ax.plot(cellline_idx, x_idx*between_group_mult+1, marker=xy_none,
                ms=20, markerfacecolor="None", markeredgecolor='black', linestyle='-')
            if len(resistant_drugs)>0 and include_resistance:
                lin_ratio = 1.0/len(resistant_drugs)
                for a_drug_idx, a_drug in enumerate(resistant_drugs):
                    x1 = np.cos(2 * np.pi * np.linspace(lin_ratio*a_drug_idx, lin_ratio*(a_drug_idx+1)))
                    y1 = np.sin(2 * np.pi * np.linspace(lin_ratio*a_drug_idx, lin_ratio*(a_drug_idx+1)))
                    xy1 = np.row_stack([[0, 0], np.column_stack([x1, y1])])
                    ax.plot(cellline_idx, x_idx*between_group_mult+1, marker=xy1,
                        ms=20, markerfacecolor=drug2color[a_drug], markeredgecolor='None', linestyle='None')
    #x axis         
    ax.set_ylim(-0.5, len(xs)*between_group_mult-between_group_mult+1.5)
    xschange = {'mutDep': f'Sensitive on {gene}-Mut', 'others': f'Sensitive on {gene}-WT',
                'sensitive_mutDep': f'Sensitive on {gene}-Mut', 'sensitive_others': f'Sensitive on {gene}-WT',
                'resistant_mutDep': f'Resistant on {gene}-Mut', 'resistant_others': f'Resistant on {gene}-WT'}
    if include_resistance: 
        ytick_labels = [xschange['sensitive_mutDep'], xschange['resistant_mutDep'], '', xschange['sensitive_others'], xschange['resistant_others']]
        ax.set_yticks(np.arange(len(xs)*2+1), ytick_labels)  # Set text labels.
    else:
        ax.set_yticks(np.arange(len(xs)), [xschange[anxs] for anxs in xs])  # Set text labels.
    
    #y axis - left
    ax.set_xlim(-1, max_yticks)  
    ax.set_xticks(np.arange(max_yticks), list(ccle_list_dict[f'ccles_in_{xs[0]}'])+['']*(max_yticks-len(ccle_list_dict[f'ccles_in_{xs[0]}'])), rotation=45, ha='right')  # Set text labels.
    
    #y axis - right
    ax2 = ax.twiny()
    ax2.set_xlim(-1, max_yticks)   
    ax2.set_xticks(np.arange(max_yticks), list(ccle_list_dict[f'ccles_in_{xs[1]}'])+['']*(max_yticks-len(ccle_list_dict[f'ccles_in_{xs[1]}'])),rotation=45, ha='left')  # Set text labels.
    
    #Legend
    legend_items = []
    for a_drug, drug_color in drug2color.items(): 
         legend_items.append(Line2D([], [], color="white", marker='o', markersize=15, markerfacecolor=drug_color, label=a_drug))
    ax.legend(handles = legend_items,bbox_to_anchor=(1.05, 0.5), loc='center left', frameon=False)
    
    ax.spines[['right']].set_visible(False)
    ax2.spines[['right']].set_visible(False)
    plt.title(label)
    #Savefigure
    fig.savefig(out_loc, dpi=300, bbox_inches='tight')


def stacked_bar(df, xs=['mutDep', 'others'], y='with_response_list', hue='drug_name_target', label='', gene='', include_resistance=False, out_loc='scatter_pie.png'):
    colors = ['#e41a1c','#377eb8','#4daf4a','#984ea3','#ff7f00','#ffff33','#a65628','#f781bf','#999999']
    #df = df.sort_values(by='diffEntropy', key=abs, ascending=False)
    all_drugs = df['candidateDrug_target'].values[:min(20, len(df))]
    drug2color = dict(zip(all_drugs, colors))
    all_target_ccles, mutDep_target_ccles, others_target_ccles = [], [], []
    for with_resp_list in df['all_with_response_list'].values: all_target_ccles+=with_resp_list.split(',')
    for with_resp_list in df['mutDep_with_response_list'].values: mutDep_target_ccles+=with_resp_list.split(',')
    for with_resp_list in df['others_with_response_list'].values: others_target_ccles+=with_resp_list.split(',')
    ccle_list_dict = {'ccles_in_mutDep': np.unique(mutDep_target_ccles),
                      'ccles_in_others': np.unique(others_target_ccles),
                      'ccles_in_all': np.unique(all_target_ccles)}
    max_yticks = max(len(ccle_list_dict['ccles_in_mutDep']), len(ccle_list_dict['ccles_in_others']))
    
    #Sort celllines according to the hit to drugs
    for xlabel in xs:
        hit_sizes = [sum(df[f'{xlabel}_sensitive_list'].str.contains(accle)==True) for accle in ccle_list_dict[f'ccles_in_{xlabel}']]
        ccle_list_dict[f'ccles_in_{xlabel}'] = ccle_list_dict[f'ccles_in_{xlabel}'][np.argsort(hit_sizes)[::-1]]
    
    #plt.clf()
    if include_resistance:
        height=6
        between_group_mult = 3
    else:
        height=4
        between_group_mult = 1
    fig, ax = plt.subplots(figsize=(max_yticks/2.0, height))#plt.subplots(figsize=(6, max_yticks/2.0))
    size_of_full_stack = 0.8
    stack_ratio = size_of_full_stack/len(drug2color.keys())
    drug_df = df.set_index('candidateDrug_target')
    for x_idx, xlabel in enumerate(xs):
        for cellline_idx, cellline in enumerate(ccle_list_dict[f'ccles_in_{xlabel}']):
            #Draw sensitive cell lines, first an stacked bar then colors with sensitive drugs
            for a_drug_idx, (a_drug, a_drug_color) in enumerate(drug2color.items()):
                for group_idx, group_name in enumerate(['sensitive', 'resistant']):
                    if not pd.isnull(drug_df.loc[a_drug, f'{xlabel}_{group_name}_list']) and cellline in drug_df.loc[a_drug, f'{xlabel}_{group_name}_list']:
                        sens_face_color = a_drug_color
                    else:
                        sens_face_color = 'None'
                    ax.bar(cellline_idx, stack_ratio, bottom=-size_of_full_stack/2.0+x_idx*between_group_mult+ group_idx+ a_drug_idx*stack_ratio, color=sens_face_color, edgecolor='black')
                
    #x axis         
    ax.set_ylim(-size_of_full_stack, len(xs)*between_group_mult-between_group_mult+size_of_full_stack*2)
    xschange = {'mutDep': f'Sensitive on {gene}-Mut', 'others': f'Sensitive on {gene}-WT',
                'sensitive_mutDep': f'Sensitive on {gene}-Mut', 'sensitive_others': f'Sensitive on {gene}-WT',
                'resistant_mutDep': f'Resistant on {gene}-Mut', 'resistant_others': f'Resistant on {gene}-WT'}
    if include_resistance: 
        ytick_labels = [xschange['sensitive_mutDep'], xschange['resistant_mutDep'], '', xschange['sensitive_others'], xschange['resistant_others']]
        ax.set_yticks(np.arange(len(xs)*2+1), ytick_labels)  # Set text labels.
    else:
        ax.set_yticks(np.arange(len(xs)), [xschange[anxs] for anxs in xs])  # Set text labels.
    
    #y axis - left
    ax.set_xlim(-1, max_yticks)  
    ax.set_xticks(np.arange(max_yticks), list(ccle_list_dict[f'ccles_in_{xs[0]}'])+['']*(max_yticks-len(ccle_list_dict[f'ccles_in_{xs[0]}'])), rotation=45, ha='right')  # Set text labels.
    
    #y axis - right
    ax2 = ax.twiny()
    ax2.set_xlim(-1, max_yticks)   
    ax2.set_xticks(np.arange(max_yticks), list(ccle_list_dict[f'ccles_in_{xs[1]}'])+['']*(max_yticks-len(ccle_list_dict[f'ccles_in_{xs[1]}'])),rotation=45, ha='left')  # Set text labels.
    
    #Legend
    legend_items = []
    for a_drug, drug_color in drug2color.items(): 
        #legend_items.append(Line2D([], [], color="white", marker='o', markersize=15, markerfacecolor=drug_color, label=a_drug))
        legend_items.append(Patch(facecolor=drug_color, label=a_drug))
        
    ax.legend(handles = legend_items,bbox_to_anchor=(1.05, 0.5), loc='center left', frameon=False)
    
    ax.spines[['right']].set_visible(False)
    ax2.spines[['right']].set_visible(False)
    plt.title(label)
    #Savefigure
    fig.savefig(out_loc, dpi=300, bbox_inches='tight')

def heatmap(df, title='', out_loc=''):
    gene, cancer = df.iloc[0]['screenGene'], df.iloc[0]['cancer']
    plt.clf()
    width_ratios = []
    for ccle_status in ['Mut', 'WT']:
        width_ratios.append(df[df[f'{ccle_status} Response']!='No Response']['Cell Line'].nunique())
    no_of_ccles = len(df['Cell Line'].unique())
    no_of_drugs = len(df['Drug (Target)'].unique())
    fig, axes = plt.subplots(1,2, figsize=(no_of_ccles/1.0, no_of_drugs/1.5), sharey=True, width_ratios=width_ratios)
    plt.subplots_adjust(wspace=0.1)
    response2int = {'Resistant': -1, 'No Response':0, 'Sensitive':1}
    n = len(response2int)     
    # discrete colormap (n samples from a given cmap)
    mycolors = ['#d7191c', '#f7f7f7', '#1a9641']
    mycolors = ['#1b9e77', '#f7f7f7', '#d95f02']
    mycolors = [RESPONSE2VIBRANTCOLOR['Resistant'], '#f7f7f7', RESPONSE2VIBRANTCOLOR['Sensitive']]
    cmap = LinearSegmentedColormap.from_list('Custom', mycolors, len(mycolors))
    sorted_drugs = df.groupby('Drug (Target)')['z_order'].max().sort_values().index.values
    for ccle_status_idx, ccle_status in enumerate(['Mut', 'WT']):
        ax = axes[ccle_status_idx]
        matrix_df = df.pivot(index='Drug (Target)', columns="Cell Line", values=f'{ccle_status} Response').replace(response2int)
        matrix_df = matrix_df.loc[:, (matrix_df != 0).any(axis=0)]
        sorted_matrix_df = matrix_df.loc[sorted_drugs,matrix_df.abs().sum().sort_values(ascending=False).index]
        sorted_matrix_df = matrix_df.sort_values(by=list(sorted_drugs), axis=1, ascending=False)
        sns_ax = sns.heatmap(sorted_matrix_df, linewidth=.5, cmap=cmap, ax=ax, cbar=bool(ccle_status_idx))
        #ax.yaxis.get_label().set_visible(False)
        ax.set_xlabel(f'{gene}-{ccle_status} Cell Lines', fontsize=16*(no_of_drugs/7.0), fontweight='bold')
        if ccle_status_idx==0:
            ax.set_ylabel('Drugs', fontsize=16*(no_of_drugs/7.0), fontweight='bold')
        else:
            ax.yaxis.get_label().set_visible(False)
        ax.tick_params(axis='x', labelsize=14*(no_of_drugs/7.0), rotation=90)
        ax.tick_params(axis='y', labelsize=14*(no_of_drugs/7.0))
        #ax.tick_params(axis='both', labelsize=14)
    
    colorbar = sns_ax.collections[0].colorbar 
    r = colorbar.vmax - colorbar.vmin 
    colorbar.set_ticks([colorbar.vmin + r / n * (0.5 + i) for i in range(n)])
    colorbar.set_ticklabels(list(response2int.keys()), fontsize=16*(no_of_drugs/7.0))  
    #axes[0].get_legend().remove()
    fig.suptitle(title, x=0.5, y=0.95, fontsize=18, fontweight='bold', ma='left', ha='center', va='bottom')#plt.title(title)
    
    config.ensure_dir(out_loc)
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')


def parallel_plot_circular_bar():
    pass

def plot_circular_bar_for_all_results(target_included=False, main_division='MutDep'):
    curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
    gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
    gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
    gdsc_nested_targets = collections.defaultdict(dict)
    for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
    #cancer_drug_gdsc = gdsc.groupby(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].apply(lambda x: ','.join(x)).reset_index()
        
    #drugs_fullname = np.unique(all_res['candidateDrug'].values)
    for cancer in []:#cancers:
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer' / f'{cancer}_drugidname_v{V}.pdf'
        if out_loc.exists():
            print(f'{cancer} cancer is already processed!')
            continue
        print(f'For cancer {cancer}')
        res = onco_res.get_result(folder_name=experiment_folder_name, cancer=cancer)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
        if res is not None:
            res['category'] = res['category'].replace(category_name_fix)
            res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
            res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
            label_add = ''
            if target_included:
                label_add = '_target'
                list_of_labels = ['name_for_cancer', 'name_for_cancer_w_drug_name_version', 'name_for_cancer_w_drug_name', 'name_for_cancer_w_drug_id_name', 
                                'name_for_gene', 'name_for_gene_w_drug_name_version', 'name_for_gene_w_drug_name', 'name_for_gene_w_drug_id_name',
                                'candidateDrug', 'drug_name_version', 'drug_name', 'drug_id_name']
                for name_col in list_of_labels:
                    res[f'{name_col}{label_add}'] = res[name_col] + ' (' + res['targets'] + ')'
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer' / f'{cancer}_drugidname_v{V}.pdf'
            config.ensure_dir(out_loc)
            circular_bar(res.copy(), sorted=True, labels_name=f'name_for_cancer{label_add}', center_label = f'{cancer}', out_loc=out_loc)
            
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer' / f'{cancer}_drugnameversion_v{V}.pdf'
            circular_bar(res.copy(), sorted=True, labels_name=f'name_for_cancer_w_drug_name_version{label_add}', center_label = f'{cancer}', out_loc=out_loc)
            
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer' / f'{cancer}_drugname_v{V}.pdf'
            circular_bar(res.copy(), sorted=True, labels_name=f'name_for_cancer_w_drug_name{label_add}', center_label = f'{cancer}', out_loc=out_loc)
            
            for drug_v in []:#[1,2]:
                res_drugv = res[res['drug_version']==f'GDSC{drug_v}']
                out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer' / f'{cancer}_drugv={drug_v}_idname_v{V}.pdf'
                circular_bar(res_drugv.copy(), sorted=True, labels_name=f'name_for_cancer_w_drug_id_name{label_add}', center_label = f'{cancer}\nGDSC{drug_v}', out_loc=out_loc)
                
                out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer' / f'{cancer}_drugv={drug_v}_name_v{V}.pdf'
                circular_bar(res_drugv.copy(), sorted=True, labels_name=f'name_for_cancer_w_drug_name{label_add}', center_label = f'{cancer}\nGDSC{drug_v}', out_loc=out_loc)
        del res
                
    
    for gene in genes:
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_gene' / f'{gene}_drugidname_v{V}.pdf'
        if out_loc.exists():
            print(f'{gene} gene is already processed!')
            continue
        print(f'For gene {gene}')
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
        if res is not None:
            res['category'] = res['category'].replace(category_name_fix)
            res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
            res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
            label_add = ''
            if target_included:
                label_add = '_target'
                list_of_labels = ['name_for_cancer', 'name_for_cancer_w_drug_name_version', 'name_for_cancer_w_drug_name', 'name_for_cancer_w_drug_id_name', 
                                'name_for_gene', 'name_for_gene_w_drug_name_version', 'name_for_gene_w_drug_name', 'name_for_gene_w_drug_id_name',
                                'candidateDrug', 'drug_name_version', 'drug_name', 'drug_id_name']
                for name_col in list_of_labels:
                    res[f'{name_col}{label_add}'] = res[name_col] + ' (' + res['targets'] + ')'
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_gene' / f'{gene}_drugidname_v{V}.pdf'
            config.ensure_dir(out_loc)
            circular_bar(res.copy(), sorted=True, labels_name=f'name_for_gene{label_add}', center_label = f'{gene}', out_loc=out_loc)
            
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_gene' / f'{gene}_drugnameversion_v{V}.pdf'
            circular_bar(res.copy(), sorted=True, labels_name=f'name_for_gene_w_drug_name_version{label_add}', center_label = f'{gene}', out_loc=out_loc)
            
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_gene' / f'{gene}_drugname_v{V}.pdf'
            circular_bar(res.copy(), sorted=True, labels_name=f'name_for_gene_w_drug_name{label_add}', center_label = f'{gene}', out_loc=out_loc)
            
            for drug_v in []:#[1,2]:
                res_drugv = res[res['drug_version']==f'GDSC{drug_v}']
                if len(res_drugv)>0:
                    out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_gene' / f'{gene}_drugv={drug_v}_idname_v{V}.pdf'
                    circular_bar(res_drugv.copy(), sorted=True, labels_name=f'name_for_gene_w_drug_id_name{label_add}', center_label = f'{gene}\nGDSC{drug_v}', out_loc=out_loc)
                    
                    out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_gene' / f'{gene}_drugv={drug_v}_name_v{V}.pdf'
                    circular_bar(res_drugv.copy(), sorted=True, labels_name=f'name_for_gene_w_drug_name{label_add}', center_label = f'{gene}\nGDSC{drug_v}', out_loc=out_loc)
        del res
                    
    for cancer in cancers:
        for gene in genes:
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugidname_v{V}.pdf'
            if False:#out_loc.exists():
                print(f'{cancer}-{gene} gene is already processed!')
                continue
            print(f'For cancer-gene: {cancer}-{gene}')
            res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer)
            res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
            if res is not None:
                res['category'] = res['category'].replace(category_name_fix)
                res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
                res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
                label_add = ''
                if target_included:
                    label_add = '_target'
                    list_of_labels = ['name_for_cancer', 'name_for_cancer_w_drug_name_version', 'name_for_cancer_w_drug_name', 'name_for_cancer_w_drug_id_name', 
                                    'name_for_gene', 'name_for_gene_w_drug_name_version', 'name_for_gene_w_drug_name', 'name_for_gene_w_drug_id_name',
                                    'candidateDrug', 'drug_name_version', 'drug_name', 'drug_id_name']
                    for name_col in list_of_labels:
                        res[f'{name_col}{label_add}'] = res[name_col] + ' (' + res['targets'] + ')'
                out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugidname_v{V}.pdf'
                config.ensure_dir(out_loc)
                circular_bar(res.copy(), sorted=True, labels_name=f'candidateDrug{label_add}', center_label = f'{cancer}_{gene}', out_loc=out_loc)
                
                out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugnameversion_v{V}.pdf'
                circular_bar(res.copy(), sorted=True, labels_name=f'drug_name_version{label_add}', center_label = f'{cancer}_{gene}', out_loc=out_loc)
                
                out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugname_v{V}.pdf'
                circular_bar(res.copy(), sorted=True, labels_name=f'drug_name{label_add}', center_label = f'{cancer}_{gene}', out_loc=out_loc)
                
                for drug_v in []:#[1,2]:
                    res_drugv = res[res['drug_version']==f'GDSC{drug_v}']
                    if len(res_drugv)>0:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugv={drug_v}_idname_v{V}.pdf'
                        circular_bar(res_drugv.copy(), sorted=True, labels_name=f'drug_id_name{label_add}', center_label = f'{cancer}_{gene}\nGDSC{drug_v}', out_loc=out_loc)
                        
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugv={drug_v}_name_v{V}.pdf'
                        circular_bar(res_drugv.copy(), sorted=True, labels_name=f'drug_name{label_add}', center_label = f'{cancer}_{gene}\nGDSC{drug_v}', out_loc=out_loc)
            del res

def plot_ic50_violin_per_cancer_gene_drug(drug_version=None, main_division='mutdep', verbose=0):
    #Loop over cancers and genes:
    for cancer in cancers:
        for gene in genes:
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'by_cancer_gene' / f'{cancer}_{gene}_drugidname_v{V}.pdf'
            if out_loc.exists():
                print(f'{cancer}-{gene} gene is already processed!')
                continue
            print(f'For cancer-gene: {cancer}-{gene}')
            #Get result per cancer-gene pair, categorize the results and use 0.05 as p-value threshold.
            res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer)
            res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
            
            if res is not None:                
                #Divide the set according to the GDSC version if drug_version is given.
                if drug_version is not None: 
                    res = res[res['drug_version']==drug_version]
                res_drug_list = []
                
                #Loop over results (Per drug)
                for row_id, row in res.iterrows():
                    if 'stratifying' in row['category'].lower():
                        if verbose==1:
                            print(f'For cancer-gene-drug: {cancer} - {gene} - {drug}')
                        #If drug version is used remove it from the drug name on the plot.
                        if drug_version is None:
                            drug = f"{row['drug_id']}\n{row['drug_name']}\n{row['drug_version']}"
                        else:
                            drug = f"{row['drug_id']}\n{row['drug_name']}"
                        
                        #Annotate
                        full_drug = row['candidateDrug']
                        gdsc_drug_cellline = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])
                        mutdep, others = row['mutDep_with_response_list'].split(','), row['others_with_response_list'].split(',')
                        all_cell_lines = np.concatenate([mutdep, others], axis=0)
                        mutdep_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in mutdep])
                        others_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in others])
                        all_lnmaxconc = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_MAX_CONC'] for celline in all_cell_lines])
                        res_drug_list.append(pd.DataFrame({
                            'Cell Lines': np.concatenate([mutdep, others],axis=0),
                            'LN_IC50': np.concatenate([mutdep_lnic50, others_lnic50],axis=0),
                            'Candidate Gene Status': ['MutDep']*len(mutdep)+['Others']*len(others),
                            'Drug': [drug]*len(all_cell_lines),
                            'Threshold': all_lnmaxconc,
                            'DiffEntropy': row['diffEntropy'],
                            'P Value': row['p'],
                            'MutDep Entropy': row['mutDep_entropy'],
                            'Others Entropy': row['others_entropy'],
                            'Category': row['category'],
                        }))
                if len(res_drug_list)>0:
                    res_drugs = pd.concat(res_drug_list)
                    if drug_version is None:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene' / f'{cancer}_{gene}_v3.pdf'
                        title_label = f'{cancer}\n{gene}'
                    else:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene_gdsc_version' / f'{cancer}_{gene}_{drug_version}_v3.pdf'
                        title_label = f'{cancer}\n{gene}\n{drug_version}'
                    config.ensure_dir(out_loc)
                    violin_distribution(res_drugs, label=title_label, out_loc=out_loc)
                    
                    

def plot_ic50_violin_per_cancer_gene_drug_facet(drug_version=None, main_division='mutdep', verbose=0):
    print(f'Violin_facet started!')
    #Loop over cancers and genes:
    for cancer in cancers:
        for gene in genes:
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene' / f'{cancer}_{gene}_facet_v1.pdf'
            if out_loc.exists():
                print(f'{cancer}-{gene} gene is already processed!')
                continue
            print(f'For cancer-gene: {cancer}-{gene}')
            #Get result per cancer-gene pair, categorize the results and use 0.05 as p-value threshold.
            res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer)
            res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
            
            if res is not None:                
                #Divide the set according to the GDSC version if drug_version is given.
                if drug_version is not None: 
                    res = res[res['drug_version']==drug_version]
                res_drug_list = []
                res['category'] = res['category'].replace(category_name_fix)
                #Loop over results (Per drug)
                for row_id, row in res.iterrows():
                    #If drug version is used remove it from the drug name on the plot.
                    if drug_version is None:
                        drug = f"{row['drug_id']}\n{row['drug_name']}\n{row['drug_version']}"
                    else:
                        drug = f"{row['drug_id']}\n{row['drug_name']}"
                    if verbose==1:
                        print(f'For cancer-gene-drug: {cancer} - {gene} - {drug}')
                    
                    #Annotate
                    full_drug = row['candidateDrug']
                    gdsc_drug_cellline = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])
                    mutdep, others = row['mutDep_with_response_list'].split(','), row['others_with_response_list'].split(',')
                    all_cell_lines = np.concatenate([mutdep, others], axis=0)
                    mutdep_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in mutdep])
                    others_lnic50 = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_IC50'] for celline in others])
                    all_lnmaxconc = np.array([gdsc_drug_cellline.loc[(full_drug,celline), 'LN_MAX_CONC'] for celline in all_cell_lines])
                    res_drug_list.append(pd.DataFrame({
                        'Cell Lines': np.concatenate([mutdep, others],axis=0),
                        'LN_IC50': np.concatenate([mutdep_lnic50, others_lnic50],axis=0),
                        'Candidate Gene Status': [main_division]*len(mutdep)+['Others']*len(others),
                        'Drug': [drug]*len(all_cell_lines),
                        'Threshold': all_lnmaxconc,
                        'DiffEntropy': row['diffEntropy'],
                        'P Value': row['p'],
                        'MutDep Entropy': row['mutDep_entropy'],
                        'Others Entropy': row['others_entropy'],
                        'Category': row['category'],
                    }))
                if len(res_drug_list)>0:
                    res_drugs = pd.concat(res_drug_list)
                    if drug_version is None:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene' / f'{cancer}_{gene}_facet_v1.pdf'
                        title_label = f'{cancer}\n{gene}'
                    else:
                        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'violin' /'by_cancer_gene_gdsc_version' / f'{cancer}_{gene}_{drug_version}_facet_v1.pdf'
                        title_label = f'{cancer}\n{gene}\n{drug_version}'
                    config.ensure_dir(out_loc)
                    violin_distribution_facet(res_drugs, label=title_label, out_loc=out_loc)
                        

def plot_all_for_drug_target_match_distribution(main_division = ''):
    cache_name = f'{experiment_folder_name}/combined_{args.cancer}_{args.main_gene}_results_with_targetinfo_tmp.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        res = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=args.main_gene, cancer=args.cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)

        cache_dict = {'data': res}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)
    
    
    #Load targets into the dataframe
    res = add_target_columns(res, only_targets=True)
    label_add='_target'
    
    #Plot for all cancer, gene pairs
    #Cicular plot of target matching distribution
    out_circular_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{args.cancer}_{args.main_gene}_circbar_drugname_v{V}.pdf'
    config.ensure_dir(out_circular_loc)
    if not out_circular_loc.exists():
        circular_bar(res.copy(), sorted=True, labels_name=f'name_w_drug_name{label_add}', center_label = f'All', out_loc=out_circular_loc)
    #Histogram Bar of target matching distribution
    out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{args.cancer}_{args.main_gene}_hist_v{V}.pdf'
    config.ensure_dir(out_loc)
    if True:#not out_loc.exists():
        title = "Distribution of drugs targeting the oncogene"
        histogram_bar(res, label=title, out_loc=out_loc)
    
    #Plot for each cancer
    for cancer in cancers:
        res_cancer = res[res['cancer']==cancer]
        if len(res_cancer)==0:
            continue
        #Cicular plot of target matching distribution
        out_circular_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{cancer}_{args.main_gene}_circbar_drugname_v{V}.pdf'
        if not out_circular_loc.exists():
            circular_bar(res_cancer.copy(), sorted=True, labels_name=f'name_for_cancer_w_drug_name{label_add}', center_label = f'{cancer}', out_loc=out_circular_loc)
        
        #Histogram Bar of target matching distribution
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{cancer}_{args.main_gene}_hist_v{V}.pdf'
        if True:#not out_loc.exists():
            title = f"Distribution of drugs that match with its target into categories in {cancer}"
            histogram_bar(res_cancer.copy(), label=title, out_loc=out_loc)
        
        #Violin plots
        res_drugs_cancer = create_drug_df_for_violin(res_cancer, main_division)
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{cancer}_{args.main_gene}_facet_violin_v{V}.pdf'
        if len(res_drugs_cancer)>0 and not out_loc.exists() and False:
            violin_distribution_facet(res_drugs_cancer, label=f'{cancer}', out_loc=out_loc)
        
        del res_cancer, res_drugs_cancer
    
    #Plot for each gene
    for gene in genes:
        res_gene = res[res['screenGene']==gene]
        if len(res_gene)==0:
            continue
        #Cicular plot of target matching distribution
        out_circular_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{args.cancer}_{gene}_circbar_drugname_v{V}.pdf'
        if not out_circular_loc.exists():
            circular_bar(res_gene.copy(), sorted=True, labels_name=f'name_for_gene_w_drug_name{label_add}', center_label = f'{gene}', out_loc=out_circular_loc)
        
        #Histogram Bar of target matching distribution
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{args.cancer}_{gene}_hist_v{V}.pdf'
        if True:#not out_loc.exists():
            title = f"Distribution of drugs that match with its target into categories in {gene}"
            histogram_bar(res_gene.copy(), label=title, out_loc=out_loc)
        
        #Violin plots
        res_drugs_gene = create_drug_df_for_violin(res_gene, main_division)
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'category_distribution' / f'{args.cancer}_{gene}_facet_violin_v{V}.pdf'
        if len(res_drugs_gene)>0 and not out_loc.exists() and False:
            violin_distribution_facet(res_drugs_gene, label=f'{gene}', out_loc=out_loc)
            
        del res_gene, res_drugs_gene
        

def plot_histogram_of_significant_drugs(main_division = '', drug_label=''):
    cache_name = f'{experiment_folder_name}/combined_{args.cancer}_{args.main_gene}_results_with_targetinfo_tmp.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        res = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=args.main_gene, cancer=args.cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)

        cache_dict = {'data': res}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)

    res = add_target_columns(res, only_targets=False)
    #Histogram Bar of target matching distribution
    for acol in [col for col in res.columns if '_target' in col]:
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'drug_distribution' / f'{args.cancer}_{args.main_gene}_{acol}_hist_v{V}.pdf'
        config.ensure_dir(out_loc)
        if True:#not out_loc.exists():
            title = "Histogram of drugs into categories"
            facet_histogram_bar(res, y=acol, label=title, out_loc=out_loc)


def plot_differential_distribution(main_division = '', cancer=None, gene=None, variant_version='cmp_2022'):
    cache_name = f'{experiment_folder_name}/combined_{args.cancer}_{args.main_gene}_results_with_targetinfo_tmp.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        res = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=args.main_gene, cancer=args.cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True)
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)

        cache_dict = {'data': res}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)

    res = add_target_columns(res, only_targets=False)
    #Histogram Bar of target matching distribution
    for acol in [col for col in res.columns if '_target' in col]:
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'drug_distribution' / f'{args.cancer}_{args.main_gene}_{acol}_hist_v{V}.pdf'
        config.ensure_dir(out_loc)
        if True:#not out_loc.exists():
            title = "Histogram of drugs into categories"
            facet_histogram_bar(res, y=acol, label=title, out_loc=out_loc)
            


def plot_celllines_scatter_or_stackedbar_with_drug(main_division = '', cancer=None, gene=None, variant_version='cmp_2022', plot_type='scatter_pie'):
    cache_name = f'{experiment_folder_name}/combined_{args.cancer}_{args.main_gene}_stratifying_results_with_targetinfo_ordered_tmp.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        res = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True, returned_categories=['stratifying essentiality', 'stratifying non-essentiality'])
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
        res = add_target_columns(res, only_targets=False)
        sign_markers = onco_res.analyze_differential_mutation_results(main_division=main_division, cancer=cancer, gene=gene, variant_version=variant_version)
        potential_drugs = sign_markers['candidateDrug'].unique()
        res['z_order']=1
        res_drug_index = res.set_index('candidateDrug')
        res_drug_index.loc[potential_drugs, 'z_order']=0
        res = res_drug_index.sort_values(by='z_order').reset_index()
        cache_dict = {'data': res}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)

    out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'sensitivity_scatter' / f'{args.cancer}' / f'{args.main_gene}' / f'stratifying_v{V}_horizontal_res.pdf'
    config.ensure_dir(out_loc)
    title = f'Drugs on {gene}-Mut vs {gene}-WT {cancer} Cell Lines'
    if plot_type == 'scatter_pie':
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'sensitivity_scatter' / f'{args.cancer}' / f'{args.main_gene}' / f'stratifying_v{V}_horizontal_scatter_pie_res.pdf'
        scatter_pie(res, xs=['mutDep', 'others'], y='with_response_list', hue='drug_name_target',  label=title, gene=gene, include_resistance=True, out_loc=out_loc)
    if plot_type == 'stacked_bar':
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'sensitivity_scatter' / f'{args.cancer}' / f'{args.main_gene}' / f'stratifying_v{V}_horizontal_stacked_bar_res.pdf'
        stacked_bar(res, xs=['mutDep', 'others'], y='with_response_list', hue='drug_name_target',  label=title, gene=gene, include_resistance=True, out_loc=out_loc)


def plot_drug_cellline_heatmap(main_division = '', cancer=None, gene=None, variant_version='cmp_2022'):
    cache_name = f'{experiment_folder_name}/combined_{cancer}_{gene}_cellline_based_df.xz'
    cache_dict = config.cache_numpy(cache_name=cache_name, is_pickle=True)
    if cache_dict is not None:
        cellline_drug_df = cache_dict['data']
    else:
        res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer, parallel=True)
        res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True, returned_categories=['stratifying essentiality', 'stratifying non-essentiality'])
        
        curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
        gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
        gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
        gdsc_nested_targets = collections.defaultdict(dict)
        for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
        
        res['category'] = res['category'].replace(category_name_fix)
        res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
        res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
        res = add_target_columns(res, only_targets=False)
        sign_markers = onco_res.analyze_differential_mutation_results(main_division=main_division, cancer=cancer, gene=gene, variant_version=variant_version)
        potential_drugs = sign_markers['candidateDrug'].unique() if sign_markers is not None else []
        res['z_order']=1
        res_drug_index = res.set_index('candidateDrug')
        res_drug_index.loc[potential_drugs, 'z_order']=0
        res = res_drug_index.sort_values(by='z_order')#.reset_index()
        
        res = res.reset_index().set_index('candidateDrug_target')
        all_target_ccles, mutDep_target_ccles, others_target_ccles = [], [], []
        for with_resp_list in res['all_with_response_list'].values: all_target_ccles+=with_resp_list.split(',')
        res_list = []
        unique_drugs = np.unique(res.index.values)
        
        for a_ccle in np.unique(all_target_ccles):
            for a_drug in unique_drugs:
                response_in_mutdep = 'Sensitive' if a_ccle in str(res.loc[a_drug, 'mutDep_sensitive_list']) else 'Resistant' if a_ccle in str(res.loc[a_drug, 'mutDep_resistant_list']) else 'No Response' #x = 2 if i>100 else 1 if i<100 else 0
                response_in_others = 'Sensitive' if a_ccle in str(res.loc[a_drug, 'others_sensitive_list']) else 'Resistant' if a_ccle in str(res.loc[a_drug, 'others_resistant_list']) else 'No Response'
                one_res = [a_ccle, a_drug, response_in_mutdep, response_in_others]
                for other_column in res.columns:
                    if '_n' != other_column[-2:] and '_list' not in other_column:
                        one_res.append(res.loc[a_drug, other_column])
                res_list.append(one_res)
        new_columns = ['Cell Line', 'Drug (Target)', 'Mut Response', 'WT Response']
        for other_column in res.columns:
            if '_n' != other_column[-2:] and '_list' not in other_column:
                new_columns.append(other_column)
        cellline_drug_df = pd.DataFrame(res_list, columns=new_columns)
        
        cache_dict = {'data': cellline_drug_df}
        config.cache_numpy(cache_name=cache_name, data=cache_dict, is_pickle=True)
    for cancer_loop in cellline_drug_df['cancer'].unique():
        cancer_cellline_drug_df = cellline_drug_df[cellline_drug_df['cancer']==cancer_loop]
        for gene_loop in cancer_cellline_drug_df['screenGene'].unique():
            gene_cancer_cellline_drug_df = cancer_cellline_drug_df[cancer_cellline_drug_df['screenGene']==gene_loop]
            title = f'Drugs on {gene_loop}-Mut vs {gene_loop}-WT {cancer_loop} Cell Lines'
            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'drug_cellline_heatmap' / f'{cancer_loop}' / f'{gene_loop}' / f'heatmap_{V}.pdf'
            if True:#not out_loc.exists():
                heatmap(gene_cancer_cellline_drug_df, title=title, out_loc=out_loc)
                


def plot_response_with_mutational_signature(main_division = '', variant_version='cmp_2022', drug_version=None, verbose=0):
    sign_markers_loc = config.FHT_RESULTS_DIR / f'oncostratifier_by_drug_{args.main_division.lower()}' / 'analysis' / f'differential_mutation' / 'only_significant.csv'
    sign_markers = pd.read_csv(sign_markers_loc, index_col=None)
    if V>=3:
        #order= [f'{gene}-WT\n{marker}-WT', f'{gene}-WT\n{marker}-Mut', f'{gene}-Mut\n{marker}-WT', f'{gene}-Mut\n{marker}-Mut']
        all_possible_plot_df_list = []
    for cancer in cancers:
        sign_markers_cancer = sign_markers[sign_markers['cancer']==cancer]
        for gene in genes:
            sign_markers_cancer_gene = sign_markers_cancer[sign_markers_cancer['screenGene']==gene]
            if len(sign_markers_cancer_gene)>0:
                print(f'For cancer-gene: {cancer}-{gene} started!')
            else:
                print(f'For cancer-gene: {cancer}-{gene} cannot found significant marker!')
                continue
            #Get result per cancer-gene pair, categorize the results and use 0.05 as p-value threshold.
            res = onco_res.get_result(folder_name=experiment_folder_name, gene=gene, cancer=cancer)
            res = onco_res.categorize_drugs(res, cutoff=0.05, only_significant=True, returned_categories=['stratifying essentiality', 'stratifying non-essentiality'])
            if res is None:
                continue
            
            curated_target = {'PIK3CA':'PI3K', 'PIK3CB':'PI3K'}
            gdsc['PUTATIVE_TARGET'] = gdsc['PUTATIVE_TARGET'].astype(str)
            gdsc_targets = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])['PUTATIVE_TARGET'].to_dict()
            gdsc_nested_targets = collections.defaultdict(dict)
            for (key1, key2), value in gdsc_targets.items(): gdsc_nested_targets[key1][key2]=value
            
            res['category'] = res['category'].replace(category_name_fix)
            res['targets'] = res.apply(lambda x: ','.join(np.unique([gdsc_targets[(x['candidateDrug'], a_ccle)] for a_ccle in x['all_with_response_list'].split(',')])), axis=1)
            res['target_hit'] = res.apply(lambda x: (RE_D.findall(x.screenGene) and (x.screenGene.rsplit(RE_D.findall(x.screenGene)[-1])[0] in x.targets)) or (x.screenGene in x.targets)  or (curated_target.get(x.screenGene, 'None') in x.targets), axis=1)
            res = add_target_columns(res, only_targets=False)
            #sign_markers = onco_res.analyze_differential_mutation_results(main_division=main_division, cancer=cancer, gene=gene, variant_version=variant_version)
            potential_drugs = sign_markers_cancer_gene['candidateDrug'].unique()
            res['z_order']=1
            res_drug_index = res.set_index('candidateDrug')
            res_drug_index.loc[potential_drugs, 'z_order']=0
            res = res_drug_index.sort_values(by='z_order')#.reset_index()
            res = res[res['z_order']==0]
            res = pd.merge(sign_markers_cancer_gene.reset_index(), res, how='left', on=['screenGene', 'cancer', 'candidateDrug', 'label'])
            
            if res is not None:                
                #Divide the set according to the GDSC version if drug_version is given.
                if drug_version is not None: 
                    res = res[res['drug_version']==drug_version]
                    
                #Loop over results (Per drug)
                for row_id, row in res.iterrows():
                    #If drug version is used remove it from the drug name on the plot.
                    if drug_version is None:
                        drug = row['drug_name_target']#f"{row['drug_id']}\n{row['drug_name']}\n{row['drug_version']}"
                    else:
                        drug = f"{row['drug_id']}\n{row['drug_name']}"
                    marker =row['differentialGene']
                    if verbose==1:
                        print(f'For cancer-gene-drug: {cancer} - {gene} - {drug} - {marker}')
                    out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'response_with_mut_marker' / f'{cancer}_{gene}_{drug}_{marker}_v{V}.pdf'
                    if False:#out_loc.exists():
                        print(f'{cancer}-{gene}-{drug}-{marker} is already processed!')
                        continue
                    #Annotate
                    full_drug = row['candidateDrug']
                    gdsc_drug_cellline = gdsc.set_index(['UNIQUE_DRUG', 'CELL_LINE_NAME'])
                    mutdep, others = row['mutDep_with_response_list'].split(','), row['others_with_response_list'].split(',')
                    mutdep_mutated, others_mutated = row['mutDep_mutated_list'].split(','), row['others_mutated_list'].split(',')
                    mutdep_notmutated, others_notmutated = row['mutDep_not_mutated_list'].split(','), row['others_not_mutated_list'].split(',')
                    marker_mutated = mutdep_mutated+others_mutated
                    marker_notmutated = mutdep_notmutated+others_notmutated
                    if V==1 or V==2 or V==6:
                        if V==1:
                            order= [f'{gene}-WT', f'{gene}-Mut', f'{gene}-Mut\n{marker}-Mut']
                        elif V==2 or V==6:
                            order= [f'{gene}-WT\n{marker}-WT', f'{gene}-WT\n{marker}-Mut', '', f'{gene}-Mut\n{marker}-WT', f'{gene}-Mut\n{marker}-Mut']
                        mutdep_lnic50 = np.column_stack(([f'{gene}-Mut']*len(mutdep), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in mutdep]))
                        others_lnic50 = np.column_stack(([f'{gene}-WT']*len(others), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in others]))
                        mutdep_mutated_lnic50 = np.column_stack(([f'{gene}-Mut\n{marker}-Mut']*len(mutdep_mutated), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in mutdep_mutated]))
                        others_mutated_lnic50 = np.column_stack(([f'{gene}-WT\n{marker}-Mut']*len(others_mutated), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in others_mutated]))
                        mutdep_notmutated_lnic50 = np.column_stack(([f'{gene}-Mut\n{marker}-WT']*len(mutdep_notmutated), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in mutdep_notmutated]))
                        others_notmutated_lnic50 = np.column_stack(([f'{gene}-WT\n{marker}-WT']*len(others_notmutated), [gdsc_drug_cellline.loc[(full_drug,celline), ['LN_IC50', 'LN_SENSITIVE']].values for celline in others_notmutated]))
                        all_lnic50 = np.concatenate([mutdep_lnic50, others_lnic50, mutdep_mutated_lnic50, others_mutated_lnic50, mutdep_notmutated_lnic50, others_notmutated_lnic50], axis=0)
                        plot_df = pd.DataFrame(all_lnic50, columns=['Group', 'LN(IC50)', 'LN_SENSITIVE'])
                    elif V==5:
                        all_ccles = mutdep+others
                        ccles_res_list = []
                        for cellline in all_ccles:
                            screen_gene_status = f'{gene}-Mut' if cellline in mutdep else f'{gene}-WT' if cellline in others else pd.nan
                            marker_gene_status = f'{marker}\nMut' if cellline in marker_mutated else f'{marker}\nWT' if cellline in marker_notmutated else pd.nan
                            response_info = gdsc_drug_cellline.loc[(full_drug,cellline)]
                            one_ccle_res = [screen_gene_status, marker_gene_status, cellline, response_info['LN_IC50'], response_info['LN_SENSITIVE']]
                            ccles_res_list.append(one_ccle_res)
                        plot_df = pd.DataFrame(ccles_res_list, columns=['ScreenGeneStatus', 'MarkerGeneStatus', 'CellLine', 'LN(IC50)', 'LN_SENSITIVE'])
                    else:
                        all_ccles = mutdep+others
                        ccles_res_list = []
                        for cellline in all_ccles:
                            screen_gene_status = 'Mut' if cellline in mutdep else 'WT' if cellline in others else pd.nan
                            marker_gene_status = 'Mut' if cellline in marker_mutated else 'WT' if cellline in marker_notmutated else pd.nan
                            response_info = gdsc_drug_cellline.loc[(full_drug,cellline)]
                            one_ccle_res = [screen_gene_status, marker_gene_status, cellline, response_info['LN_IC50'], response_info['LN_SENSITIVE']]
                            ccles_res_list.append(one_ccle_res)
                        plot_df = pd.DataFrame(ccles_res_list, columns=['ScreenGeneStatus', 'MarkerGeneStatus', 'CellLine', 'LN(IC50)', 'LN_SENSITIVE'])
                    
                    plot_df['ScreenGene'] = gene
                    plot_df['Cancer'] = cancer
                    cancer_abbv = ''.join([s[:3] for s in cancer.split()])
                    plot_df['Cancer_Abbv'] = cancer_abbv
                    plot_df['Drug'] = drug
                    plot_df['Marker'] = marker
                    plot_df['Status'] = plot_df['LN_SENSITIVE'].astype(float).astype(int).replace({0:'Resistant', 1:'Sensitive'})
                    plot_df['LN(IC50)'] = plot_df['LN(IC50)'].astype(float)
                    plot_df['HIT'] = f'{gene}\n{row["drug_name"]}\n{marker}'
                    plot_df['HIT_with_cancer'] = f'{cancer_abbv}\n{gene}\n{row["drug_name"]}\n{marker}'
                    if V==3 or V==4:
                        all_possible_plot_df_list.append(plot_df)
                    elif V==1 or V==2:
                        plot_df = plot_df[plot_df['Group'].isin(order)]   #r"$\bf{" + str(number) + "}$"
                        title_label = r'$\bf{Cancer: }$'+f'{cancer}\n'+r'$\bf{Oncogene: }$'+f'{gene}\n'+r'$\bf{Drug: }$'+f'{drug}\n'+r'$\bf{Marker: }$'+f'{marker}'
                        config.ensure_dir(out_loc)
                        swarm_per_group(plot_df, order=order, label=title_label, out_loc=out_loc)
                    elif V==5:
                        title_label = r'$\bf{Cancer: }$'+f'{cancer}\n'+r'$\bf{Oncogene: }$'+f'{gene}\n'+r'$\bf{Drug: }$'+f'{drug}\n'+r'$\bf{Marker: }$'+f'{marker}'
                        config.ensure_dir(out_loc)
                        col_order= [f'{gene}-WT', f'{gene}-Mut']
                        order = [f'{marker}\nWT', f'{marker}\nMut']
                        swarm_per_groupv2(plot_df, order=order, col_order=col_order, label=title_label, out_loc=out_loc)
                    elif V==6:
                        title_label = r'$\bf{Cancer: }$'+f'{cancer}\n'+r'$\bf{Oncogene: }$'+f'{gene}\n'+r'$\bf{Drug: }$'+f'{drug}\n'+r'$\bf{Marker: }$'+f'{marker}'
                        plot_df = plot_df[plot_df['Group'].isin(order)]
                        for chosen_group in plot_df['Group'].unique():
                            chosen_group_for_filename = "_".join(chosen_group.split("\n"))
                            out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'response_with_mut_marker' / f'{cancer}_{gene}_{drug}_{marker}' / f'{chosen_group_for_filename}_v{V}.pdf'
                            config.ensure_dir(out_loc)
                            plot_df_per_group = plot_df.copy()
                            plot_df_per_group.loc[plot_df_per_group['Group']!=chosen_group, 'Group'] = 'Rest'
                            order = [f'Rest', f'{chosen_group}']
                            swarm_one_vs_rest(plot_df_per_group, order=order, label=title_label, out_loc=out_loc)
    
    if V>=3 and V<5 and len(all_possible_plot_df_list)>0:
        all_plot_df = pd.concat(all_possible_plot_df_list)
        all_plot_df = all_plot_df.sort_values(by=['Cancer_Abbv', 'ScreenGene', 'Drug', 'Marker'])
        out_loc = ONCO_BY_DRUG_FOLDER / 'images' / 'response_with_mut_marker' / f'All_{args.cancer}_{args.main_gene}_v{V}.pdf'
        config.ensure_dir(out_loc)
        title_label = f''#f'All Stratifying Drugs with A Mutational Marker'
        order= ['WT', 'Mut']
        swarm_all_groups(all_plot_df, order=order, label=title_label, out_loc=out_loc)
    

def main():
    if args.visual_type == 'circular_bar':
        plot_circular_bar_for_all_results(target_included=True, main_division=args.main_division)
    elif args.visual_type == 'violin': 
        plot_ic50_violin_per_cancer_gene_drug(drug_version='GDSC2', main_division='Mut')
    elif args.visual_type == 'violin_facet': 
        plot_ic50_violin_per_cancer_gene_drug_facet(drug_version=None, main_division=args.main_division)
    elif args.visual_type == 'target_match_distribution': 
        plot_all_for_drug_target_match_distribution(main_division=args.main_division)
    elif args.visual_type == 'significant_drug_distribution': 
        plot_histogram_of_significant_drugs(main_division=args.main_division, drug_label='')
    elif args.visual_type == 'differential_mutation': 
        plot_differential_distribution(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
    elif args.visual_type == 'cellline_scatter_with_drug_sensitivity': 
        plot_celllines_scatter_or_stackedbar_with_drug(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version, plot_type='stacked_bar')
    elif args.visual_type == 'drug_cellline_heatmap': 
        plot_drug_cellline_heatmap(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
    elif args.visual_type == 'response_with_mutational_signature': 
        plot_response_with_mutational_signature(main_division=args.main_division, variant_version=args.variant_version)
    elif args.visual_type == 'drug_coverage': 
        plot_drug_coverage(main_division=args.main_division, cancer=args.cancer, gene=args.main_gene, variant_version=args.variant_version)
    
#analyze_differential_mutation_results(main_division=args.main_division.lower(), cancer=cancer, gene=args.main_gene, variant_version=args.variant_version)#args.cancer
if __name__ == "__main__":
    main()