import sys, os
sys.path.insert(0, os.getcwd())
import numpy as np
import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt
from matplotlib.lines import Line2D
from matplotlib.colors import to_rgba, LinearSegmentedColormap
from matplotlib.patches import Patch
import collections
import re
import warnings
if not sys.warnoptions:
warnings.simplefilter("ignore")
os.environ["PYTHONWARNINGS"] = "ignore" # Also affect subprocesses

def get_label_rotation(angle, offset):
    # Rotation must be specified in degrees :(
    rotation = np.rad2deg(angle + offset)
    if rotation%360<270 and rotation%360>90:
        alignment = "right"
        rotation = rotation +180
    else: 
        alignment = "left"
        rotation = rotation
    return rotation, alignment

def add_labels(angles, values, labels, colors, offset, ax):
    # This is the space between the end of the bar and the label
    padding = 0.05
    
    # Iterate over angles, values, and labels, to add all of them.
    for angle, value, label, color in zip(angles, values, labels, colors):
        angle = angle
        
        # Obtain text rotation and alignment
        rotation, alignment = get_label_rotation(angle, offset)

        # And finally add the text
        if value>=0:
            ax.text(
                x=angle, 
                y=value + padding, 
                s=f'{label}', 
                ha=alignment, 
                va="center", 
                rotation=rotation, 
                rotation_mode="anchor",
                fontsize=6, 
                color=color
            )
        else:
            ax.text(
                x=angle, 
                y=padding, 
                s=f'{label}', 
                ha=alignment, 
                va="center", 
                rotation=rotation, 
                rotation_mode="anchor",
                fontsize=6, 
                color=color
            )

def circular_bar(df, value_col='diffEntropy', color_col='target_hit', group_col='category', sorted=False, labels_name='name', center_label = '', out_loc='trial.pdf'):
    if out_loc.exists():
        print('Recreating')
    plt.clf()
    # Reorder the dataframe
    if sorted:
        df = (
            df.groupby([group_col]) 
            .apply(lambda x: x.sort_values([value_col], ascending = False, key=abs)).reset_index(drop=True)
        )

    VALUES = df[value_col].values
    LABELS = df[labels_name].values
    LABEL_COLORS = df[color_col].map({False:'black', True:'red'}).values
    GROUP = df[group_col].values

    PAD = 2
    ANGLES_N = len(VALUES) + PAD * len(np.unique(GROUP))
    ANGLES = np.linspace(0, 2 * np.pi, num=ANGLES_N, endpoint=False)
    WIDTH = (2 * np.pi) / len(ANGLES)

    GROUPS_SIZE = [len(i[1]) for i in df.groupby(group_col)]
    GROUPS_NAME = [i[0] for i in df.groupby(group_col)]

    offset = 0
    IDXS = []
    for size in GROUPS_SIZE:
        IDXS += list(range(offset + PAD, offset + size + PAD))
        offset += size + PAD

    fig, ax = plt.subplots(figsize=(10, 10), subplot_kw={"projection": "polar"})
    ax.set_theta_offset(offset)
    ax.set_ylim(-1.8, 1.5)
    ax.set_frame_on(False)
    ax.xaxis.grid(False)
    ax.yaxis.grid(False)
    ax.set_xticks([])
    ax.set_yticks([])
    ax.tick_params(grid_color="palegoldenrod")

    GROUPS_SIZE = [len(i[1]) for i in df.groupby(group_col)]
    COLORS = [f"C{i}" for i, size in enumerate(GROUPS_SIZE) for _ in range(size)]

    ax.bar(
        ANGLES[IDXS], VALUES, width=WIDTH, color=COLORS, 
        edgecolor="white", linewidth=2
    )

    add_labels(ANGLES[IDXS], VALUES, LABELS, LABEL_COLORS, offset, ax)

    # Extra customization below here --------------------

    # This iterates over the sizes of the groups adding reference
    # lines and annotations.

    offset = 0 
    ANGLE_BETWEEN_SIZE = (ANGLES[1]-ANGLES[0])/3
    for group_id, (group, size) in enumerate(zip(GROUPS_NAME, GROUPS_SIZE)):
        # Add line below bars
        x1 = np.linspace(ANGLES[offset + PAD]-ANGLE_BETWEEN_SIZE, ANGLES[offset + size + PAD - 1]+ANGLE_BETWEEN_SIZE, num=50)
        ax.plot(x1, [0] * 50, color="#333333")
        
        # Add text to indicate group and reference lines
        #x2 = np.linspace(ANGLES[offset]-ANGLE_BETWEEN_SIZE, ANGLES[offset + PAD - 1]+ANGLE_BETWEEN_SIZE, num=50)
        if 'stratifying' in group.lower():
            ax.text(
                np.mean(x1), 1.8, group.replace(' ','\n'), color="#333333", fontsize=8, 
                fontweight="bold", ha='center', va="center", rotation=0#group_label_rotation+90
            )
        else:
            ax.text(
                np.mean(x1), 1.8, group.replace(' ','\n'), color="#333333", fontsize=8, 
                fontweight="bold", ha='center', va="center", rotation=0#group_label_rotation+90
            )
        x2 = np.linspace(ANGLES[offset+PAD]+ANGLE_BETWEEN_SIZE, ANGLES[offset]-ANGLE_BETWEEN_SIZE, num=50)
        if 'stratifying' in GROUPS_NAME[group_id].lower():
            for ref_value in np.arange(0.0, 1.1, 0.2):
                #ax.plot(x2[:23], [ref_value] * 23, color="#bebebe", lw=0.8)
                #ax.plot(x2[:], [ref_value] * 50, color="#bebebe", lw=0.8)
                ax.text(ANGLES[offset+PAD-1]+ANGLE_BETWEEN_SIZE, ref_value, f'{ref_value:.1f}', color="black", ha='center', va="center", 
                        rotation=0, fontsize=6)
        else:
            for ref_value in np.arange(-1.0, 0.1, 0.2):
                #ax.plot(x2[:23], [ref_value] * 23, color="#bebebe", lw=0.8)
                #ax.plot(x2[:], [ref_value] * 50, color="#bebebe", lw=0.8)
                ax.text(ANGLES[offset+PAD-1]+ANGLE_BETWEEN_SIZE/5, ref_value, f'{ref_value:.1f}', color="black", ha='center', va="center", 
                        rotation=0, fontsize=6)#np.rad2deg(x2[24])+70


        offset += size + PAD
    ax.text(
        0, -1.8, center_label.replace(' ','\n'), color="#333333", fontsize=12,
        fontweight="bold", ha='center', va="center", rotation=0#group_label_rotation+90
        )
    
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    del df


def violin_distribution(df, label, out_loc):
    plt.clf()
    violin = sns.violinplot(data=df, x="Drug", y="LN_IC50", hue="Candidate Gene Status", split=True, gap=.1, inner="point", cut=0, bw_adjust=0.3#, alpha=0.3,
                            )#inner_kws=dict(color="black", edgecolor='black'))
    ax = violin.axes
    for one_violin in violin.collections[::2]:
        one_violin.set_facecolor(to_rgba(one_violin.get_facecolor(), alpha=0.3))
    grouped_by_drug = df.groupby(['Drug'])[['DiffEntropy', 'Threshold', 'Category', 'LN_IC50', 'P Value']].agg(lambda x:x.value_counts().index[0])
    xticks = ax.get_xticks()
    xtick_locs = np.linspace(0,1,len(xticks)*2+1)
    category_name_fix = {'stratifying essentiality': 'Stratifying\nSen.', 'stratifying non-essentiality': 'Stratifying\nRes.',
                        'co-dependency':'Sensitive\nonMut', 'exclusive-dependency':'Resistant\nonMut'}
    for tick_id, aticklabel in enumerate(ax.get_xticklabels()):
        drug= aticklabel.get_text()
        x_pos = aticklabel.get_position()[0]
        category_name = category_name_fix[grouped_by_drug.loc[drug, "Category"]]
        
        ax.text(x_pos, ax.get_ylim()[1], f'{category_name}\nDE: {grouped_by_drug.loc[drug, "DiffEntropy"]:.2f}\nP: {grouped_by_drug.loc[drug, "P Value"]:.1e}',
            horizontalalignment='center', verticalalignment='bottom', fontsize=8
        )
        ax.axhline(y=grouped_by_drug.loc[drug, "Threshold"], 
                xmin=xtick_locs[tick_id*2+1]-1/(len(xticks)*4), xmax=xtick_locs[tick_id*2+1]+1/(len(xticks)*4), 
                ls='--', c='red'
        )
        
    # Add max concentration as a legend
    handles, labels = ax.get_legend_handles_labels()
    maxconc_line = Line2D([0], [0], label='Max Conc.', ls='--', c='red')
    handles.append(maxconc_line) 
    plt.legend(handles=handles, bbox_to_anchor=(1, 1), loc='upper left', borderaxespad=0, frameon=False)
    
    # Remove top and right axis, put title to the top, and save figure.
    sns.despine()
    plt.title(label, x=0.5, y=1.15, fontsize=10)
    plt.ylabel('LN(IC50)')
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    print()
    

def violin_distribution_facet(df, label, out_loc):
    if out_loc.exists():
        print('Recreating')#return 1
    plt.clf()
    category2col = {'stratifying essentiality': 'Stratifying', 'stratifying non-essentiality': 'Stratifying',
                        'co-dependency':'Full', 'exclusive-dependency':'Full'}
    category2row = {'stratifying essentiality': 'Sensitivity', 'stratifying non-essentiality': 'Resistance',
                        'co-dependency':'Sensitivity', 'exclusive-dependency':'Resistance'}
    df['Category Col'] = df['Category'].str.split(' ', expand=True)[0] #df['Category'].replace(category2col)
    df['Category Row'] = df['Category'].str.split(' ', expand=True)[1] #df['Category'].replace(category2row)
    col_order = [acol for acol in ['Stratifying', 'Full'] if acol in df['Category Col'].unique()]
    row_order = [arow for arow in ['Sensitivity', 'Resistance'] if arow in df['Category Row'].unique()]
    xranges = df.groupby(['Category Col', 'Category Row'])['Drug'].nunique().reset_index().groupby(['Category Col'])['Drug'].max().loc[col_order]*1.1
    color_palette = {'Mut':'#ef8a62', 'MutDep':'#ef8a62', 'Others':'#67a9cf'}
    g = sns.FacetGrid(df, col="Category Col", row="Category Row", height=3, aspect=0.7*np.log2(max(xranges.values)+2), 
                    col_order=col_order, row_order=row_order, palette = color_palette,
                    sharex=False, sharey=False, gridspec_kws=dict(width_ratios=xranges))
    g.map_dataframe(sns.violinplot, x="Drug", y="LN_IC50", hue="Candidate Gene Status", palette = color_palette, split=True, gap=.1, inner="point", cut=0, bw_adjust=0.3)
    #g.map_dataframe(annotate)
    
    grouped_by_drug = df.groupby(['Drug'])[['DiffEntropy', 'Threshold', 'Category', 'LN_IC50', 'P Value']].agg(lambda x:x.value_counts().index[0])
    #ax = violin.axes
    for ax in g.axes.flatten():
        if len(ax.collections)==0:
            ax.set_visible(False)
            continue
        for one_violin in ax.collections[::2]:
            one_violin.set_facecolor(to_rgba(one_violin.get_facecolor(), alpha=0.3))
        xticks = ax.get_xticks()
        xtick_locs = np.linspace(0,1,len(xticks)*2+1)
        for tick_id, aticklabel in enumerate(ax.get_xticklabels()):
            drug= aticklabel.get_text()
            x_pos = aticklabel.get_position()[0]
            #category_name = category_name_fix[grouped_by_drug.loc[drug, "Category"]]
            
            ax.text(x_pos, ax.get_ylim()[1], f'DE: {grouped_by_drug.loc[drug, "DiffEntropy"]:.2f}\nP: {grouped_by_drug.loc[drug, "P Value"]:.1e}',
                horizontalalignment='center', verticalalignment='center', fontsize=8
            )
            ax.axhline(y=grouped_by_drug.loc[drug, "Threshold"], 
                    xmin=xtick_locs[tick_id*2+1]-1/(len(xticks)*4), xmax=xtick_locs[tick_id*2+1]+1/(len(xticks)*4), 
                    ls='--', c='red'
            )
    
    # Add max concentration as a legend
    g._legend_data['Max Conc.'] = Line2D([0], [0], label='Max Conc.', ls='--', c='red')
    g.add_legend(frameon=False)
    g.tight_layout()
    g.set_titles('{col_name} {row_name}', va='bottom', y=1.05, fontsize=10)
    g.set_axis_labels(x_var='Drug', y_var='LN(IC50)', weight='bold')
    g.fig.subplots_adjust(top=1.2)
    g.fig.suptitle(label, va='bottom', y=1.35, fontsize=12, weight='bold')#, x=0.5, y=1.15, fontsize=10)
    g.despine()
    # Remove top and right axis, put title to the top, and save figure.
    #sns.despine()
    #plt.title(label, x=0.5, y=1.15, fontsize=10)
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    

def swarm_per_group(plot_df, order=None, label='', out_loc=''):
    plt.clf()    
    hue_order = ['Resistant', 'Sensitive']
    violin = sns.swarmplot(data=plot_df, x="Group", y="LN(IC50)", hue="Status", hue_order=hue_order, order=order)    
    # Remove top and right axis, put title to the top, and save figure.
    sns.despine()
    plt.title(label, x=0.5, y=1.15, fontsize=10)
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')


def histogram_bar(df, label, out_loc):
    if out_loc.exists():
        print('Recreating')#return 1
    plt.clf()
    df['category'] = df['category'].str.replace(' ', '\n')
    g = sns.histplot(df, x='category', discrete=True, shrink=.8)
    sns.despine()
    plt.title(label, x=0.5, y=1.05, fontsize=10)
    plt.ylabel('Count')
    plt.xlabel('Category')
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    

def facet_histogram_bar(df, y, label, out_loc):
    if out_loc.exists():
        print('Recreating')#return 1
    plt.clf()
    
    df['Category Col'] = df['category'].str.split(' ', expand=True)[0] #df['Category'].replace(category2col)
    df['Category Row'] = df['category'].str.split(' ', expand=True)[1] #df['Category'].replace(category2row)
    col_order = [acol for acol in ['Stratifying', 'Full'] if acol in df['Category Col'].unique()]
    row_order = [arow for arow in ['Sensitivity', 'Resistance'] if arow in df['Category Row'].unique()]
    #xranges = df.groupby(['Category Col', 'Category Row'])[].nunique().reset_index().groupby(['Category Col'])['Drug'].max().loc[col_order]*1.1
    color_palette = {'Mut':'#ef8a62', 'MutDep':'#ef8a62', 'Others':'#67a9cf'}
    g = sns.FacetGrid(df, col="Category Col", row="Category Row", height=5, aspect=2,
                    col_order=col_order, row_order=row_order, palette = color_palette,
                    sharex=False, sharey=False, gridspec_kws={"wspace":1.5, "hspace":0.3})#, gridspec_kws=dict(height_ratios=xranges))
    def sorted_bar(y, *args, **kwargs):
        data = kwargs.pop('data')
        count_data = data[y].value_counts().sort_values(ascending=False)[0:20].reset_index()
        sns.barplot(count_data, y=y, x='count', **kwargs);   
    g.map_dataframe(sorted_bar, y=y)
    g.set_titles('{col_name} {row_name}', fontsize=10)
    g.set_axis_labels(x_var='Count', y_var='Drug', weight='bold')
    #g.fig.subplots_adjust(top=1.2)
    g.fig.suptitle(label, va='bottom', y=1, fontsize=12, weight='bold')#, x=0.5, y=1.15, fontsize=10)
    g.despine()
    g.tight_layout()
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')


def scatter_pie(df, xs=['mutDep', 'others'], y='with_response_list', hue='drug_name_target', label='', gene='', include_resistance=False, out_loc='scatter_pie.png'):
    colors = ['#e41a1c','#377eb8','#4daf4a','#984ea3','#ff7f00','#ffff33','#a65628','#f781bf','#999999']
    #df = df.sort_values(by='diffEntropy', key=abs, ascending=False)
    all_drugs = df['candidateDrug_target'].values[:min(10, len(df))]
    drug2color = dict(zip(all_drugs, colors))
    all_target_ccles, mutDep_target_ccles, others_target_ccles = [], [], []
    for with_resp_list in df['all_with_response_list'].values: all_target_ccles+=with_resp_list.split(',')
    for with_resp_list in df['mutDep_with_response_list'].values: mutDep_target_ccles+=with_resp_list.split(',')
    for with_resp_list in df['others_with_response_list'].values: others_target_ccles+=with_resp_list.split(',')
    ccle_list_dict = {'ccles_in_mutDep': np.unique(mutDep_target_ccles),
                    'ccles_in_others': np.unique(others_target_ccles),
                    'ccles_in_all': np.unique(all_target_ccles)}
    max_yticks = max(len(ccle_list_dict['ccles_in_mutDep']), len(ccle_list_dict['ccles_in_others']))
    
    #Sort celllines according to the hit to drugs
    for xlabel in xs:
        hit_sizes = [sum(df[f'{xlabel}_sensitive_list'].str.contains(accle)==True) for accle in ccle_list_dict[f'ccles_in_{xlabel}']]
        ccle_list_dict[f'ccles_in_{xlabel}'] = ccle_list_dict[f'ccles_in_{xlabel}'][np.argsort(hit_sizes)[::-1]]
    
    #plt.clf()
    if include_resistance:
        height=6
        between_group_mult = 3
    else:
        height=4
        between_group_mult = 1
    fig, ax = plt.subplots(figsize=(max_yticks/2.0, height))#plt.subplots(figsize=(6, max_yticks/2.0))
    for x_idx, xlabel in enumerate(xs):
        for cellline_idx, cellline in enumerate(ccle_list_dict[f'ccles_in_{xlabel}']):
            #Draw sensitive cell lines, first an empty circle then colors with sensitive drugs
            sensitive_drugs = df[df[f'{xlabel}_sensitive_list'].str.contains(cellline)==True]['candidateDrug_target']
            x_none = np.cos(2 * np.pi * np.linspace(0, 1))
            y_none = np.sin(2 * np.pi * np.linspace(0, 1))
            xy_none = np.row_stack([[0, 0], np.column_stack([x_none, y_none])])
            ax.plot(cellline_idx, x_idx*between_group_mult, marker=xy_none,
                ms=20, markerfacecolor="None", markeredgecolor='black', linestyle='-')
            if len(sensitive_drugs)>0:
                lin_ratio = 1.0/len(sensitive_drugs)
                for a_drug_idx, a_drug in enumerate(sensitive_drugs):
                    x1 = np.cos(2 * np.pi * np.linspace(lin_ratio*a_drug_idx, lin_ratio*(a_drug_idx+1)))
                    y1 = np.sin(2 * np.pi * np.linspace(lin_ratio*a_drug_idx, lin_ratio*(a_drug_idx+1)))
                    xy1 = np.row_stack([[0, 0], np.column_stack([x1, y1])])
                    ax.plot(cellline_idx, x_idx*between_group_mult, marker=xy1,
                        ms=20, markerfacecolor=drug2color[a_drug], markeredgecolor='None', linestyle='None')
            
            resistant_drugs = df[df[f'{xlabel}_resistant_list'].str.contains(cellline)==True]['candidateDrug_target']
            x_none = np.cos(2 * np.pi * np.linspace(0, 1))
            y_none = np.sin(2 * np.pi * np.linspace(0, 1))
            xy_none = np.row_stack([[0, 0], np.column_stack([x_none, y_none])])
            ax.plot(cellline_idx, x_idx*between_group_mult+1, marker=xy_none,
                ms=20, markerfacecolor="None", markeredgecolor='black', linestyle='-')
            if len(resistant_drugs)>0 and include_resistance:
                lin_ratio = 1.0/len(resistant_drugs)
                for a_drug_idx, a_drug in enumerate(resistant_drugs):
                    x1 = np.cos(2 * np.pi * np.linspace(lin_ratio*a_drug_idx, lin_ratio*(a_drug_idx+1)))
                    y1 = np.sin(2 * np.pi * np.linspace(lin_ratio*a_drug_idx, lin_ratio*(a_drug_idx+1)))
                    xy1 = np.row_stack([[0, 0], np.column_stack([x1, y1])])
                    ax.plot(cellline_idx, x_idx*between_group_mult+1, marker=xy1,
                        ms=20, markerfacecolor=drug2color[a_drug], markeredgecolor='None', linestyle='None')
    #x axis         
    ax.set_ylim(-0.5, len(xs)*between_group_mult-between_group_mult+1.5)
    xschange = {'mutDep': f'Sensitive on {gene}-Mut', 'others': f'Sensitive on {gene}-WT',
                'sensitive_mutDep': f'Sensitive on {gene}-Mut', 'sensitive_others': f'Sensitive on {gene}-WT',
                'resistant_mutDep': f'Resistant on {gene}-Mut', 'resistant_others': f'Resistant on {gene}-WT'}
    if include_resistance: 
        ytick_labels = [xschange['sensitive_mutDep'], xschange['resistant_mutDep'], '', xschange['sensitive_others'], xschange['resistant_others']]
        ax.set_yticks(np.arange(len(xs)*2+1), ytick_labels)  # Set text labels.
    else:
        ax.set_yticks(np.arange(len(xs)), [xschange[anxs] for anxs in xs])  # Set text labels.
    
    #y axis - left
    ax.set_xlim(-1, max_yticks)  
    ax.set_xticks(np.arange(max_yticks), list(ccle_list_dict[f'ccles_in_{xs[0]}'])+['']*(max_yticks-len(ccle_list_dict[f'ccles_in_{xs[0]}'])), rotation=45, ha='right')  # Set text labels.
    
    #y axis - right
    ax2 = ax.twiny()
    ax2.set_xlim(-1, max_yticks)   
    ax2.set_xticks(np.arange(max_yticks), list(ccle_list_dict[f'ccles_in_{xs[1]}'])+['']*(max_yticks-len(ccle_list_dict[f'ccles_in_{xs[1]}'])),rotation=45, ha='left')  # Set text labels.
    
    #Legend
    legend_items = []
    for a_drug, drug_color in drug2color.items(): 
        legend_items.append(Line2D([], [], color="white", marker='o', markersize=15, markerfacecolor=drug_color, label=a_drug))
    ax.legend(handles = legend_items,bbox_to_anchor=(1.05, 0.5), loc='center left', frameon=False)
    
    ax.spines[['right']].set_visible(False)
    ax2.spines[['right']].set_visible(False)
    plt.title(label)
    #Savefigure
    fig.savefig(out_loc, dpi=300, bbox_inches='tight')


def stacked_bar(df, xs=['mutDep', 'others'], y='with_response_list', hue='drug_name_target', label='', gene='', include_resistance=False, out_loc='scatter_pie.png'):
    colors = ['#e41a1c','#377eb8','#4daf4a','#984ea3','#ff7f00','#ffff33','#a65628','#f781bf','#999999']
    #df = df.sort_values(by='diffEntropy', key=abs, ascending=False)
    all_drugs = df['candidateDrug_target'].values[:min(20, len(df))]
    drug2color = dict(zip(all_drugs, colors))
    all_target_ccles, mutDep_target_ccles, others_target_ccles = [], [], []
    for with_resp_list in df['all_with_response_list'].values: all_target_ccles+=with_resp_list.split(',')
    for with_resp_list in df['mutDep_with_response_list'].values: mutDep_target_ccles+=with_resp_list.split(',')
    for with_resp_list in df['others_with_response_list'].values: others_target_ccles+=with_resp_list.split(',')
    ccle_list_dict = {'ccles_in_mutDep': np.unique(mutDep_target_ccles),
                    'ccles_in_others': np.unique(others_target_ccles),
                    'ccles_in_all': np.unique(all_target_ccles)}
    max_yticks = max(len(ccle_list_dict['ccles_in_mutDep']), len(ccle_list_dict['ccles_in_others']))
    
    #Sort celllines according to the hit to drugs
    for xlabel in xs:
        hit_sizes = [sum(df[f'{xlabel}_sensitive_list'].str.contains(accle)==True) for accle in ccle_list_dict[f'ccles_in_{xlabel}']]
        ccle_list_dict[f'ccles_in_{xlabel}'] = ccle_list_dict[f'ccles_in_{xlabel}'][np.argsort(hit_sizes)[::-1]]
    
    #plt.clf()
    if include_resistance:
        height=6
        between_group_mult = 3
    else:
        height=4
        between_group_mult = 1
    fig, ax = plt.subplots(figsize=(max_yticks/2.0, height))#plt.subplots(figsize=(6, max_yticks/2.0))
    size_of_full_stack = 0.8
    stack_ratio = size_of_full_stack/len(drug2color.keys())
    drug_df = df.set_index('candidateDrug_target')
    for x_idx, xlabel in enumerate(xs):
        for cellline_idx, cellline in enumerate(ccle_list_dict[f'ccles_in_{xlabel}']):
            #Draw sensitive cell lines, first an stacked bar then colors with sensitive drugs
            for a_drug_idx, (a_drug, a_drug_color) in enumerate(drug2color.items()):
                for group_idx, group_name in enumerate(['sensitive', 'resistant']):
                    if not pd.isnull(drug_df.loc[a_drug, f'{xlabel}_{group_name}_list']) and cellline in drug_df.loc[a_drug, f'{xlabel}_{group_name}_list']:
                        sens_face_color = a_drug_color
                    else:
                        sens_face_color = 'None'
                    ax.bar(cellline_idx, stack_ratio, bottom=-size_of_full_stack/2.0+x_idx*between_group_mult+ group_idx+ a_drug_idx*stack_ratio, color=sens_face_color, edgecolor='black')
                
    #x axis         
    ax.set_ylim(-size_of_full_stack, len(xs)*between_group_mult-between_group_mult+size_of_full_stack*2)
    xschange = {'mutDep': f'Sensitive on {gene}-Mut', 'others': f'Sensitive on {gene}-WT',
                'sensitive_mutDep': f'Sensitive on {gene}-Mut', 'sensitive_others': f'Sensitive on {gene}-WT',
                'resistant_mutDep': f'Resistant on {gene}-Mut', 'resistant_others': f'Resistant on {gene}-WT'}
    if include_resistance: 
        ytick_labels = [xschange['sensitive_mutDep'], xschange['resistant_mutDep'], '', xschange['sensitive_others'], xschange['resistant_others']]
        ax.set_yticks(np.arange(len(xs)*2+1), ytick_labels)  # Set text labels.
    else:
        ax.set_yticks(np.arange(len(xs)), [xschange[anxs] for anxs in xs])  # Set text labels.
    
    #y axis - left
    ax.set_xlim(-1, max_yticks)  
    ax.set_xticks(np.arange(max_yticks), list(ccle_list_dict[f'ccles_in_{xs[0]}'])+['']*(max_yticks-len(ccle_list_dict[f'ccles_in_{xs[0]}'])), rotation=45, ha='right')  # Set text labels.
    
    #y axis - right
    ax2 = ax.twiny()
    ax2.set_xlim(-1, max_yticks)   
    ax2.set_xticks(np.arange(max_yticks), list(ccle_list_dict[f'ccles_in_{xs[1]}'])+['']*(max_yticks-len(ccle_list_dict[f'ccles_in_{xs[1]}'])),rotation=45, ha='left')  # Set text labels.
    
    #Legend
    legend_items = []
    for a_drug, drug_color in drug2color.items(): 
        #legend_items.append(Line2D([], [], color="white", marker='o', markersize=15, markerfacecolor=drug_color, label=a_drug))
        legend_items.append(Patch(facecolor=drug_color, label=a_drug))
        
    ax.legend(handles = legend_items,bbox_to_anchor=(1.05, 0.5), loc='center left', frameon=False)
    
    ax.spines[['right']].set_visible(False)
    ax2.spines[['right']].set_visible(False)
    plt.title(label)
    #Savefigure
    fig.savefig(out_loc, dpi=300, bbox_inches='tight')

def heatmap(df, title='', out_loc=''):
    gene, cancer = df.iloc[0]['screenGene'], df.iloc[0]['cancer']
    plt.clf()
    width_ratios = []
    for ccle_status in ['Mut', 'WT']:
        width_ratios.append(df[df[f'{ccle_status} Response']!='No Response']['Cell Line'].nunique())
    fig, axes = plt.subplots(1,2, figsize=(len(df['Cell Line'].unique())/2.0, len(df['Drug (Target)'].unique())/3.0), sharey=True, width_ratios=width_ratios)
    response2int = {'Resistant': -1, 'No Response':0, 'Sensitive':1}
    n = len(response2int)     
    # discrete colormap (n samples from a given cmap)
    mycolors = ['#d7191c', '#f7f7f7', '#1a9641']
    mycolors = ['#1b9e77', '#f7f7f7', '#d95f02']
    cmap = LinearSegmentedColormap.from_list('Custom', mycolors, len(mycolors))
    sorted_drugs = df.groupby('Drug (Target)')['z_order'].max().sort_values().index.values
    for ccle_status_idx, ccle_status in enumerate(['Mut', 'WT']):
        matrix_df = df.pivot(index='Drug (Target)', columns="Cell Line", values=f'{ccle_status} Response').replace(response2int)
        matrix_df = matrix_df.loc[:, (matrix_df != 0).any(axis=0)]
        sorted_matrix_df = matrix_df.loc[sorted_drugs,matrix_df.abs().sum().sort_values(ascending=False).index]
        sorted_matrix_df = matrix_df.sort_values(by=list(sorted_drugs), axis=1, ascending=False)
        sns_ax = sns.heatmap(sorted_matrix_df, linewidth=.5, cmap=cmap, ax=axes[ccle_status_idx], cbar=bool(ccle_status_idx))
        axes[ccle_status_idx].yaxis.get_label().set_visible(False)
        axes[ccle_status_idx].set_xlabel(f'{gene}-{ccle_status} Cell Lines')
    
    colorbar = sns_ax.collections[0].colorbar 
    r = colorbar.vmax - colorbar.vmin 
    colorbar.set_ticks([colorbar.vmin + r / n * (0.5 + i) for i in range(n)])
    colorbar.set_ticklabels(list(response2int.keys()))  
    #axes[0].get_legend().remove()
    plt.title(title)
    config.ensure_dir(out_loc)
    plt.savefig(out_loc, dpi=300, bbox_inches='tight')
    
    
    
